@@ -34,7 +34,10 @@ imMessage.prototype = {
   _displayMessage: null,
 
   get displayMessage() {
-    return this._displayMessage || this.prplMessage.originalMessage;
+    // Explicitly test for null so that blank messages don't fall back to
+    // the original. Especially problematic in encryption extensions like OTR.
+    return this._displayMessage !== null ?
+      this._displayMessage : this.prplMessage.originalMessage;
   },
   set displayMessage(aMsg) { this._displayMessage = aMsg; },
 
@@ -203,7 +206,8 @@ UIConversation.prototype = {
 
     if (!Services.prefs.getBoolPref("messenger.conversations.alwaysClose") &&
         (this.isChat && !this.left ||
-         !this.isChat && this.unreadIncomingMessageCount != 0))
+         !this.isChat && (this.unreadIncomingMessageCount != 0 ||
+                          Services.prefs.getBoolPref("messenger.conversations.holdByDefault"))))
       return false;
 
     this.close();
