@@ -17,7 +17,7 @@ DEFAULTS = {
 
   # LDAPSDKS
   'LDAPSDKS_REPO': 'https://hg.mozilla.org/projects/ldap-sdks/',
-  'LDAPSDKS_REV': 'LDAPCSDK_6_0_7F_RTM',
+  'LDAPSDKS_REV': 'LDAPCSDK_6_0_7G_RTM',
 
   # URL of the default hg repository to clone for ChatZilla.
   'CHATZILLA_REPO': 'https://hg.mozilla.org/chatzilla/',
