@@ -123,18 +123,10 @@ function appUpdater()
   XPCOMUtils.defineLazyServiceGetter(this, "um",
                                      "@mozilla.org/updates/update-manager;1",
                                      "nsIUpdateManager");
-  XPCOMUtils.defineLazyServiceGetter(this, "bs",
-                                     "@mozilla.org/extensions/blocklist;1",
-                                     "nsIBlocklistService");
 
   this.bundle = Services.strings.
                 createBundle("chrome://messenger/locale/messenger.properties");
 
-  this.updateBtn = document.getElementById("updateButton");
-
-  // The button label value must be set so its height is correct.
-  this.setupUpdateButton("update.checkInsideButton");
-
   let manualURL = Services.urlFormatter.formatURLPref("app.update.url.manual");
   document.getElementById("manualLink")
           .setAttribute("onclick", 'openURL("' + manualURL + '");');
@@ -146,15 +138,8 @@ function appUpdater()
     return;
   }
 
-  if (this.isPending) {
-    this.setupUpdateButton("update.restart." +
-                           (this.isMajor ? "upgradeButton" : "updateButton"));
-    return;
-  }
-
-  if (this.isApplied) {
-    this.setupUpdateButton("update.restart." +
-                           (this.isMajor ? "upgradeButton" : "restartButton"));
+  if (this.isPending || this.isApplied) {
+    this.selectPanel("apply");
     return;
   }
 
@@ -163,12 +148,21 @@ function appUpdater()
     return;
   }
 
-  if (this.updateEnabled && this.updateAuto) {
-    this.selectPanel("checkingForUpdates");
-    this.isChecking = true;
-    this.checker.checkForUpdates(this.updateCheckListener, true);
+  // Honor the "Never check for updates" option by not only disabling background
+  // update checks, but also in the About dialog, by presenting a
+  // "Check for updates" button.
+  // If updates are found, the user is then asked if he wants to "Update to <version>".
+  if (!this.updateEnabled) {
+    this.selectPanel("checkForUpdates");
     return;
   }
+
+  // That leaves the options
+  // "Check for updates, but let me choose whether to install them", and
+  // "Automatically install updates".
+  // In both cases, we check for updates without asking.
+  // In the "let me choose" case, we ask before downloading though, in onCheckComplete.
+  this.checkForUpdates();
 }
 
 appUpdater.prototype =
@@ -179,7 +173,7 @@ appUpdater.prototype =
   // true when there is an update already staged / ready to be applied.
   get isPending() {
     if (this.update) {
-      return this.update.state == "pending" || 
+      return this.update.state == "pending" ||
              this.update.state == "pending-service";
     }
     return this.um.activeUpdate &&
@@ -189,7 +183,7 @@ appUpdater.prototype =
 
   get isApplied() {
     if (this.update) {
-      return this.update.state == "applied" || 
+      return this.update.state == "applied" ||
              this.update.state == "applied-service";
     }
     return this.um.activeUpdate &&
@@ -205,13 +199,6 @@ appUpdater.prototype =
            this.um.activeUpdate.state == "downloading";
   },
 
-  // true when the update type is major.
-  get isMajor() {
-    if (this.update)
-      return this.update.type == "major";
-    return this.um.activeUpdate.type == "major";
-  },
-
   // true when updating is disabled by an administrator.
   get updateDisabledAndLocked() {
     return !this.updateEnabled &&
@@ -243,36 +230,65 @@ appUpdater.prototype =
   },
 
   /**
-   * Sets the deck's selected panel.
+   * Sets the panel of the updateDeck.
    *
    * @param  aChildID
-   *         The id of the deck's child to select.
+   *         The id of the deck's child to select, e.g. "apply".
    */
+
   selectPanel: function(aChildID) {
-    this.updateDeck.selectedPanel = document.getElementById(aChildID);
-    this.updateBtn.disabled = (aChildID != "updateButtonBox");
+    let panel = document.getElementById(aChildID);
+
+    if (panel.firstChild.localName == "button") {
+      let button = panel.firstChild;
+      if (aChildID == "downloadAndInstall") {
+        let updateVersion = gAppUpdater.update.displayVersion;
+        button.label = this.bundle.formatStringFromName("update.downloadAndInstallButton.label", [updateVersion], 1);
+        button.accessKey = this.bundle.GetStringFromName("update.downloadAndInstallButton.accesskey");
+      }
+      this.updateDeck.selectedPanel = panel;
+      if (!document.commandDispatcher.focusedElement || // don't steal the focus
+          document.commandDispatcher.focusedElement.localName == "button") // except from the other buttons
+        button.focus();
+
+    } else {
+      this.updateDeck.selectedPanel = panel;
+    }
   },
 
   /**
-   * Sets the update button's label and accesskey.
-   *
-   * @param  aKeyPrefix
-   *         The prefix for the properties file entry to use for setting the
-   *         label and accesskey.
+   * Check for updates
    */
-  setupUpdateButton: function(aKeyPrefix) {
-    this.updateBtn.label = this.bundle.GetStringFromName(aKeyPrefix + ".label");
-    this.updateBtn.accessKey = this.bundle.GetStringFromName(aKeyPrefix + ".accesskey");
-    if (!document.commandDispatcher.focusedElement ||
-        document.commandDispatcher.focusedElement == this.updateBtn)
-      this.updateBtn.focus();
+  checkForUpdates: function() {
+    this.selectPanel("checkingForUpdates");
+    this.isChecking = true;
+    this.checker.checkForUpdates(this.updateCheckListener, true);
+    // after checking, onCheckComplete() is called
   },
 
   /**
-   * Handles oncommand for the update button.
+   * Check for addon compat, or start the download right away
    */
-  buttonOnCommand: function() {
-    if (this.isPending || this.isApplied) {
+  doUpdate: function() {
+    // skip the compatibility check if the update doesn't provide appVersion,
+    // or the appVersion is unchanged, e.g. nightly update
+    if (!this.update.appVersion ||
+        Services.vc.compare(gAppUpdater.update.appVersion,
+                            Services.appinfo.version) == 0) {
+      this.startDownload();
+    } else {
+      this.checkAddonCompatibility();
+    }
+  },
+
+  /**
+   * Handles oncommand for the "Restart to Update" button
+   * which is presented after the download has been downloaded.
+   */
+  buttonRestartAfterDownload: function() {
+    if (!this.isPending && !this.isApplied)
+      return;
+
       // Notify all windows that an application quit has been requested.
       let cancelQuit = Components.classes["@mozilla.org/supports-PRBool;1"].
                        createInstance(Components.interfaces.nsISupportsPRBool);
@@ -291,25 +307,19 @@ appUpdater.prototype =
 
       Services.startup.quit(Components.interfaces.nsIAppStartup.eAttemptQuit |
                             Components.interfaces.nsIAppStartup.eRestart);
-      return;
-    }
+    },
 
+  /**
+   * Handles oncommand for the "Apply Update…" button
+   * which is presented if we need to show the billboard or license.
+   */
+  buttonApplyBillboard: function() {
     const URI_UPDATE_PROMPT_DIALOG = "chrome://mozapps/content/update/updates.xul";
-    // Thunderbird no longer displays a license for updates and the licenseURL check
-    // is just in case a distibution does.
-    if (this.update && (this.update.billboardURL || this.update.licenseURL ||
-        this.addons.length != 0)) {
-      var ary = toXPCOMArray([this.update],
-        Components.interfaces.nsIMutableArray);
-      var openFeatures = "chrome,centerscreen,dialog=no,resizable=no,titlebar,toolbar=no";
-      Services.ww.openWindow(null, URI_UPDATE_PROMPT_DIALOG, "", openFeatures, ary);
-      window.close();
-      return;
-    }
-
-    this.selectPanel("checkingForUpdates");
-    this.isChecking = true;
-    this.checker.checkForUpdates(this.updateCheckListener, true);
+    var ary = toXPCOMArray([this.update],
+    Components.interfaces.nsIMutableArray);
+    var openFeatures = "chrome,centerscreen,dialog=no,resizable=no,titlebar,toolbar=no";
+    Services.ww.openWindow(null, URI_UPDATE_PROMPT_DIALOG, "", openFeatures, ary);
+    window.close();
   },
 
   /**
@@ -330,6 +340,17 @@ appUpdater.prototype =
         return;
       }
 
+      if (gAppUpdater.update.unsupported) {
+        let unsupportedLink = document.getElementById("unsupportedLink");
+        if (gAppUpdater.update.detailsURL)
+          unsupportedLink.href = gAppUpdater.update.detailsURL;
+        else
+          unsupportedLink.setAttribute("hidden", true);
+
+        gAppUpdater.selectPanel("unsupportedSystem");
+        return;
+      }
+
       if (!gAppUpdater.aus.canApplyUpdates) {
         gAppUpdater.selectPanel("manualUpdate");
         return;
@@ -338,21 +359,14 @@ appUpdater.prototype =
       // Thunderbird no longer displays a license for updates and the licenseURL
       // check is just in case a distibution does.
       if (gAppUpdater.update.billboardURL || gAppUpdater.update.licenseURL) {
-        gAppUpdater.selectPanel("updateButtonBox");
-        gAppUpdater.setupUpdateButton("update.openUpdateUI." +
-                                      (this.isMajor ? "upgradeButton"
-                                                    : "applyButton"));
+        gAppUpdater.selectPanel("applyBillboard");
         return;
       }
 
-      if (!gAppUpdater.update.appVersion ||
-          Services.vc.compare(gAppUpdater.update.appVersion,
-                              Services.appinfo.version) == 0) {
-        gAppUpdater.startDownload();
-        return;
-      }
-
-      gAppUpdater.checkAddonCompatibility();
+      if (gAppUpdater.updateAuto) // automatically download and install
+        gAppUpdater.doUpdate();
+      else // ask
+        gAppUpdater.selectPanel("downloadAndInstall");
     },
 
     /**
@@ -458,9 +472,9 @@ appUpdater.prototype =
    * See XPIProvider.jsm
    */
   onUpdateAvailable: function(aAddon, aInstall) {
-    if (!this.bs.isAddonBlocklisted(aAddon.id, aInstall.version,
-                                    this.update.appVersion,
-                                    this.update.platformVersion)) {
+    if (!Services.blocklist.isAddonBlocklisted(aAddon.id, aInstall.version,
+                                               this.update.appVersion,
+                                               this.update.platformVersion)) {
       // Compatibility or new version updates mean the same thing here.
       this.onCompatibilityUpdateAvailable(aAddon);
     }
@@ -481,9 +495,7 @@ appUpdater.prototype =
       return;
     }
 
-    this.selectPanel("updateButtonBox");
-    this.setupUpdateButton("update.openUpdateUI." +
-                           (this.isMajor ? "upgradeButton" : "applyButton"));
+    this.selectPanel("apply");
   },
 
   /**
@@ -498,7 +510,7 @@ appUpdater.prototype =
     this.aus.pauseDownload();
     let state = this.aus.downloadUpdate(this.update, false);
     if (state == "failed") {
-      this.selectPanel("downloadFailed");      
+      this.selectPanel("downloadFailed");
       return;
     }
 
@@ -545,28 +557,24 @@ appUpdater.prototype =
         this.selectPanel("applying");
         let update = this.um.activeUpdate;
         let self = this;
-        Services.obs.addObserver(function (aSubject, aTopic, aData) {
+        Services.obs.addObserver(function selectPanelOnUpdate(aSubject, aTopic, aData) {
           // Update the UI when the background updater is finished
           let status = update.state;
           if (status == "applied" || status == "applied-service" ||
               status == "pending" || status == "pending-service") {
             // If the update is successfully applied, or if the updater has
-            // fallen back to non-staged updates, show the Restart to Update
+            // fallen back to non-staged updates, show the "Restart to Update"
             // button.
-            self.selectPanel("updateButtonBox");
-            self.setupUpdateButton("update.restart." +
-                                   (self.isMajor ? "upgradeButton" : "restartButton"));
+            self.selectPanel("apply");
           } else if (status == "failed") {
             // Background update has failed, let's show the UI responsible for
             // prompting the user to update manually.
             self.selectPanel("downloadFailed");
           }
-          Services.obs.removeObserver(arguments.callee, "update-staged");
+          Services.obs.removeObserver(selectPanelOnUpdate, "update-staged");
         }, "update-staged", false);
       } else {
-        this.selectPanel("updateButtonBox");
-        this.setupUpdateButton("update.restart." +
-                               (this.isMajor ? "upgradeButton" : "updateButton"));
+        this.selectPanel("apply");
       }
       break;
     default:
@@ -574,7 +582,6 @@ appUpdater.prototype =
       this.selectPanel("downloadFailed");
       break;
     }
-
   },
 
   /**
