@@ -61,7 +61,9 @@ function fillMailContextMenu(event)
 
   goUpdateCommand('cmd_printpreview');
   goUpdateCommand('cmd_print');
-  
+
+  updateCheckedStateForIgnoreAndWatchThreadCmds();
+
   gContextMenu = new nsContextMenu(event.target, event.shiftKey);
   return gContextMenu.shouldDisplay;
 }
@@ -322,7 +324,8 @@ function fillFolderPaneContextMenu()
   var haveOnlySubscribableFolders = folders.every(checkCanSubscribeToFolder);
 
   function checkIsNewsgroup(folder) {
-    return !folder.isServer && folder.server.type == "nntp";
+    return !folder.isServer && folder.server.type == "nntp" &&
+           !folder.getFlag(nsMsgFolderFlags.Virtual);
   }
   var haveOnlyNewsgroups = folders.every(checkIsNewsgroup);
 
