@@ -112,7 +112,7 @@ function overlayOnLoad()
 
 function overlayRepositionDialog()
 {
-  // Position the dialog so it is fully visible on the screen  
+  // Position the dialog so it is fully visible on the screen
   // (if possible)
 
   // Seems to be necessary to get the correct dialog height/width
@@ -164,8 +164,8 @@ function CustomizeMailToolbar(toolboxId, customizePopupId)
     // Open the panel, but make it invisible until the iframe has loaded so
     // that the user doesn't see a white flash.
     panel.style.visibility = "hidden";
-    toolbox.addEventListener("beforecustomization", function () {
-      toolbox.removeEventListener("beforecustomization", arguments.callee, false);
+    toolbox.addEventListener("beforecustomization", function removeProp() {
+      toolbox.removeEventListener("beforecustomization", removeProp, false);
       panel.style.removeProperty("visibility");
     }, false);
     panel.openPopup(toolbox, "after_start", 0, 0);
@@ -243,7 +243,7 @@ function MailToolboxCustomizeDone(aEvent, customizePopupId)
             continue;
           if ("_teardown" in child)
             child._teardown();
-          popup.removeChild(child);
+          child.remove();
         }
       }
     }
@@ -264,7 +264,7 @@ function onViewToolbarsPopupShowing(aEvent, toolboxIds, aInsertPoint)
     // Remove all of the nodes with the iscollapsible
     // attribute.
     if (deadItem.hasAttribute("iscollapsible"))
-      popup.removeChild(deadItem);
+      deadItem.remove();
   }
 
   // We'll insert the menuitems before the first item in the list if no insert
@@ -448,13 +448,14 @@ function openAddonsMgr(aView)
 
   openContentTab("about:addons", "tab", "addons.mozilla.org");
 
+
   if (aView) {
     // This must be a new load, else the ping/pong would have
     // found the window above.
-    Services.obs.addObserver(function (aSubject, aTopic, aData) {
-        Services.obs.removeObserver(arguments.callee, aTopic);
-        aSubject.loadView(aView);
-      }, "EM-loaded", false);
+    Services.obs.addObserver(function loadViewOnLoad(aSubject, aTopic, aData) {
+      Services.obs.removeObserver(loadViewOnLoad, aTopic);
+      aSubject.loadView(aView);
+    }, "EM-loaded", false);
   }
 }
 
@@ -507,7 +508,7 @@ function openSavedFilesWnd()
 function SetBusyCursor(window, enable)
 {
     // setCursor() is only available for chrome windows.
-    // However one of our frames is the start page which 
+    // However one of our frames is the start page which
     // is a non-chrome window, so check if this window has a
     // setCursor method
     if ("setCursor" in window) {
