@@ -261,7 +261,7 @@ var WindowWatcher = {
     //  time in the sun.
     controller.sleep(0);
     this._firstWindowOpened = true;
-    // wrap the creation because 
+    // wrap the creation because
     mark_action("winhelp", "new MozMillController()", [aWindowType]);
     let c = new controller.MozMillController(domWindow);
     mark_action("winhelp", "/new MozMillController()", [aWindowType]);
@@ -312,6 +312,7 @@ var WindowWatcher = {
 
       let self = this;
       function startTest() {
+        self.planForWindowClose(troller.window);
         try {
           let runner = new frame.Runner(collector);
           runner.wrapper(self.subTestFunc, troller);
@@ -324,6 +325,7 @@ var WindowWatcher = {
         // except I'm not sure how to easily figure that out...
         // so just close it no matter what.
         troller.window.close();
+        self.waitForWindowClose();
 
         self.waitingList.delete(self.waitingForOpen);
         // now we are waiting for it to close...
@@ -399,8 +401,8 @@ var WindowWatcher = {
                   "Timeout waiting for window to close!",
       WINDOW_CLOSE_TIMEOUT_MS, WINDOW_CLOSE_CHECK_INTERVAL_MS, this);
     let didDisappear = (this.waitingList.get(this.waitingForClose) == null);
-    this.waitingList.delete(windowType);
     let windowType = this.waitingForClose;
+    this.waitingList.delete(windowType);
     this.waitingForClose = null;
     if (!didDisappear)
       throw new Error(windowType + " window did not disappear!");
@@ -723,7 +725,7 @@ function wait_for_frame_load(aFrame, aURLOrPredicate) {
  * Generic function to wait for some sort of document to load. We expect
  * aDetails to have three fields:
  * - webProgress: an nsIWebProgress associated with the contentWindow.
- * - currentURI: the currently loaded page (nsIURI). 
+ * - currentURI: the currently loaded page (nsIURI).
  * - contentWindow: the content window.
  */
 function _wait_for_generic_load(aDetails, aURLOrPredicate) {
@@ -955,7 +957,6 @@ var AugmentEverybodyWith = {
     click_menus_in_sequence: function _click_menus(aRootPopup, aActions, aKeepOpen) {
       if (aRootPopup.state == "closed")
         aRootPopup.openPopup(null, "", 0, 0, true, true);
-      aRootPopup.focus(); // This is a hack that can be removed once the focus issues on Linux are solved.
       if (aRootPopup.state != "open") { // handle "showing"
         utils.waitFor(function() { return aRootPopup.state == "open"; },
                       "Popup never opened! id=" + aRootPopup.id +
@@ -967,32 +968,48 @@ var AugmentEverybodyWith = {
 
       let curPopup = aRootPopup;
       for each (let [iAction, actionObj] in Iterator(aActions)) {
-        let matchingNode = null;
+        /**
+         * Check if aNode attributes match all those given in actionObj.
+         * Nodes that are obvious containers are skipped, and their children
+         * will be used to recursively find a match instead.
+         */
+        let findMatch = function(aNode) {
+          // Ignore some elements and just use their children instead.
+          if (aNode.localName == "hbox" || aNode.localName == "vbox" ||
+              aNode.localName == "splitmenu") {
+            for (let i = 0; i < aNode.children.length; i++) {
+              let childMatch = findMatch(aNode.children[i]);
+              if (childMatch)
+                return childMatch;
+            }
+            return null;
+          }
 
-        let kids = curPopup.children;
-        for (let iKid=0; iKid < kids.length; iKid++) {
-          let node = kids[iKid];
           let matchedAll = true;
           for each (let [name, value] in Iterator(actionObj)) {
-            if (!node.hasAttribute(name) ||
-                node.getAttribute(name) != value) {
+            if (!aNode.hasAttribute(name) ||
+                aNode.getAttribute(name) != value) {
               matchedAll = false;
               break;
             }
           }
+          return (matchedAll) ? aNode : null;
+        };
 
-          if (matchedAll) {
-            matchingNode = node;
+        let matchingNode = null;
+        let kids = curPopup.children;
+        for (let iKid = 0; iKid < kids.length; iKid++) {
+          let node = kids[iKid];
+          matchingNode = findMatch(node);
+          if (matchingNode)
             break;
-          }
         }
 
         if (!matchingNode)
           throw new Error("Did not find matching menu item for action index " +
-                          iAction);
+                          iAction + ": " + JSON.stringify(actionObj));
 
         this.click(new elib.Elem(matchingNode));
-        matchingNode.focus(); // This is a hack that can be removed once the focus issues on Linux are solved.
         if ("menupopup" in matchingNode) {
           curPopup = matchingNode.menupopup;
           closeStack.push(curPopup);
@@ -1162,11 +1179,11 @@ var PerWindowTypeAugmentations = {
       aController.window.MessageDisplayWidget.prototype
                  .SUMMARIZATION_SELECTION_STABILITY_INTERVAL_MS = 0;
     },
-    
+
     /**
      * Used to wrap methods on a class prototype in order to generate
      *  mark_action data about the call.
-     */         
+     */
     debugTrace: [
       // wrap 3pane unload function to notice when it explodes
       {
@@ -1229,21 +1246,21 @@ var PerWindowTypeAugmentations = {
       },
       // Message summarization annotations
       {
-        method: "summarize",
-        onConstructor: "MultiMessageSummary",
-        reportAs: "MD_MultiMessageSummary_summarize",
+        method: "summarizeThread",
+        onGlobal: true,
+        reportAs: "summarizeThread",
         showArgs: false,
       },
       {
-        method: "onQueryCompleted",
-        onConstructor: "MultiMessageSummary",
-        reportAs: "MD_*Summary_onQueryCompleted",
+        method: "summarizeMultipleSelection",
+        onGlobal: true,
+        reportAs: "summarizeMultipleSelection",
         showArgs: false,
       },
       {
-        method: "summarize",
-        onConstructor: "ThreadSummary",
-        reportAs: "MD_ThreadSummary_summarize",
+        method: "summarizeFolder",
+        onGlobal: true,
+        reportAs: "summarizeFolder",
         showArgs: false,
       },
     ],
