@@ -31,6 +31,7 @@
 #include "nsNetUtil.h"
 #include "nsMsgUtils.h"
 #include "nsIMutableArray.h"
+#include "nsIMsgMailSession.h"
 #include "nsArrayUtils.h"
 #include "nsCOMArray.h"
 #include "nsIMsgFilterCustomAction.h"
@@ -43,7 +44,7 @@
 #include "nsAutoPtr.h"
 #include "nsIMsgFilter.h"
 
-NS_IMPL_ISUPPORTS1(nsMsgFilterService, nsIMsgFilterService)
+NS_IMPL_ISUPPORTS(nsMsgFilterService, nsIMsgFilterService)
 
 nsMsgFilterService::nsMsgFilterService()
 {
@@ -176,7 +177,7 @@ nsresult nsMsgFilterService::AlertBackingUpFilterFile(nsIMsgWindow *aMsgWindow)
 }
 
 nsresult //Do not use this routine if you have to call it very often because it creates a new bundle each time
-nsMsgFilterService::GetStringFromBundle(const char *aMsgName, PRUnichar **aResult)
+nsMsgFilterService::GetStringFromBundle(const char *aMsgName, char16_t **aResult)
 {
   NS_ENSURE_ARG_POINTER(aResult);
 
@@ -209,10 +210,17 @@ nsMsgFilterService::ThrowAlertMsg(const char*aMsgName, nsIMsgWindow *aMsgWindow)
 {
   nsString alertString;
   nsresult rv = GetStringFromBundle(aMsgName, getter_Copies(alertString));
-  if (NS_SUCCEEDED(rv) && !alertString.IsEmpty() && aMsgWindow)
+  nsCOMPtr<nsIMsgWindow> msgWindow(do_QueryInterface(aMsgWindow));
+  if (!msgWindow) {
+    nsCOMPtr<nsIMsgMailSession> mailSession ( do_GetService(NS_MSGMAILSESSION_CONTRACTID, &rv));
+    if (NS_SUCCEEDED(rv))
+      rv = mailSession->GetTopmostMsgWindow(getter_AddRefs(msgWindow));
+  }
+
+  if (NS_SUCCEEDED(rv) && !alertString.IsEmpty() && msgWindow)
   {
     nsCOMPtr <nsIDocShell> docShell;
-    aMsgWindow->GetRootDocShell(getter_AddRefs(docShell));
+    msgWindow->GetRootDocShell(getter_AddRefs(docShell));
     if (docShell)
     {
       nsCOMPtr<nsIPrompt> dialog(do_GetInterface(docShell));
@@ -245,7 +253,6 @@ class nsMsgFilterAfterTheFact : public nsIUrlListener, public nsIMsgSearchNotify
 {
 public:
   nsMsgFilterAfterTheFact(nsIMsgWindow *aMsgWindow, nsIMsgFilterList *aFilterList, nsIArray *aFolderList);
-  virtual ~nsMsgFilterAfterTheFact();
   NS_DECL_ISUPPORTS
   NS_DECL_NSIURLLISTENER
   NS_DECL_NSIMSGSEARCHNOTIFY
@@ -253,11 +260,12 @@ public:
 
   nsresult  AdvanceToNextFolder();  // kicks off the process
 protected:
+  virtual ~nsMsgFilterAfterTheFact();
   virtual   nsresult  RunNextFilter();
   nsresult  ApplyFilter(bool *aApplyMore = nullptr);
   nsresult  OnEndExecution(nsresult executionStatus); // do what we have to do to cleanup.
   bool      ContinueExecutionPrompt();
-  nsresult  DisplayConfirmationPrompt(nsIMsgWindow *msgWindow, const PRUnichar *confirmString, bool *confirmed);
+  nsresult  DisplayConfirmationPrompt(nsIMsgWindow *msgWindow, const char16_t *confirmString, bool *confirmed);
   nsCOMPtr<nsIMsgWindow>      m_msgWindow;
   nsCOMPtr<nsIMsgFilterList>  m_filters;
   nsCOMPtr<nsIArray>          m_folders;
@@ -274,7 +282,7 @@ protected:
   uint32_t                    m_nextAction; // next filter action to perform
 };
 
-NS_IMPL_ISUPPORTS3(nsMsgFilterAfterTheFact, nsIUrlListener, nsIMsgSearchNotify, nsIMsgCopyServiceListener)
+NS_IMPL_ISUPPORTS(nsMsgFilterAfterTheFact, nsIUrlListener, nsIMsgSearchNotify, nsIMsgCopyServiceListener)
 
 nsMsgFilterAfterTheFact::nsMsgFilterAfterTheFact(nsIMsgWindow *aMsgWindow, nsIMsgFilterList *aFilterList, nsIArray *aFolderList)
 {
@@ -897,8 +905,10 @@ nsMsgApplyFiltersToMessages::nsMsgApplyFiltersToMessages(nsIMsgWindow *aMsgWindo
     bool hasMore;
     while (NS_SUCCEEDED(msgEnumerator->HasMoreElements(&hasMore)) && hasMore)
     {
+      nsCOMPtr<nsISupports> supports;
       nsCOMPtr<nsIMsgDBHdr> msgHdr;
-      if (NS_SUCCEEDED(msgEnumerator->GetNext(getter_AddRefs(msgHdr))) && msgHdr)
+      if (NS_SUCCEEDED(msgEnumerator->GetNext(getter_AddRefs(supports))) &&
+          (msgHdr = do_QueryInterface(supports)))
         m_msgHdrList.AppendObject(msgHdr);
     }
   }
@@ -1055,11 +1065,11 @@ bool nsMsgFilterAfterTheFact::ContinueExecutionPrompt()
   m_curFilter->GetFilterName(filterName);
   nsString formatString;
   nsString confirmText;
-  const PRUnichar *formatStrings[] =
+  const char16_t *formatStrings[] =
   {
     filterName.get()
   };
-  nsresult rv = bundle->FormatStringFromName(NS_LITERAL_STRING("continueFilterExecution").get(),
+  nsresult rv = bundle->FormatStringFromName(MOZ_UTF16("continueFilterExecution"),
                                              formatStrings, 1, getter_Copies(confirmText));
   if (NS_FAILED(rv))
     return false;
@@ -1069,7 +1079,7 @@ bool nsMsgFilterAfterTheFact::ContinueExecutionPrompt()
 }
 
 nsresult
-nsMsgFilterAfterTheFact::DisplayConfirmationPrompt(nsIMsgWindow *msgWindow, const PRUnichar *confirmString, bool *confirmed)
+nsMsgFilterAfterTheFact::DisplayConfirmationPrompt(nsIMsgWindow *msgWindow, const char16_t *confirmString, bool *confirmed)
 {
   if (msgWindow)
   {
