@@ -3,9 +3,6 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#ifdef MOZ_LOGGING
-#define FORCE_PR_LOG /* Allow logging in the release build */
-#endif
 #include "nsMsgCopyService.h"
 #include "nsCOMArray.h"
 #include "nspr.h"
@@ -427,7 +424,7 @@ nsMsgCopyService::FindRequest(nsISupports* aSupport,
   return copyRequest;
 }
 
-NS_IMPL_ISUPPORTS1(nsMsgCopyService, nsIMsgCopyService)
+NS_IMPL_ISUPPORTS(nsMsgCopyService, nsIMsgCopyService)
 
 NS_IMETHODIMP
 nsMsgCopyService::CopyMessages(nsIMsgFolder* srcFolder, /* UI src folder */
