@@ -118,6 +118,8 @@ nsIAtom * nsMsgFolderDataSource::kInVFEditSearchScopeAtom = nullptr;
 
 static const uint32_t kDisplayBlankCount = 0xFFFFFFFE;
 static const uint32_t kDisplayQuestionCount = 0xFFFFFFFF;
+static const int64_t kDisplayBlankCount64 = -2;
+static const int64_t kDisplayQuestionCount64 = -1;
 
 nsMsgFolderDataSource::nsMsgFolderDataSource()
 {
@@ -183,19 +185,19 @@ nsMsgFolderDataSource::nsMsgFolderDataSource()
     rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_RENAME), &kNC_Rename);
     rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_EMPTYTRASH), &kNC_EmptyTrash);
 
-    kTotalMessagesAtom           = MsgNewAtom("TotalMessages").get();
-    kTotalUnreadMessagesAtom     = MsgNewAtom("TotalUnreadMessages").get();
-    kFolderSizeAtom              = MsgNewAtom("FolderSize").get();
-    kBiffStateAtom               = MsgNewAtom("BiffState").get();
-    kSortOrderAtom               = MsgNewAtom("SortOrder").get();
-    kNewMessagesAtom             = MsgNewAtom("NewMessages").get();
-    kNameAtom                    = MsgNewAtom("Name").get();
-    kSynchronizeAtom             = MsgNewAtom("Synchronize").get();
-    kOpenAtom                    = MsgNewAtom("open").get();
-    kIsDeferredAtom              = MsgNewAtom("isDeferred").get();
-    kIsSecureAtom                = MsgNewAtom("isSecure").get();
-    kCanFileMessagesAtom         = MsgNewAtom("canFileMessages").get();
-    kInVFEditSearchScopeAtom     = MsgNewAtom("inVFEditSearchScope").get();
+    kTotalMessagesAtom           = MsgNewAtom("TotalMessages").take();
+    kTotalUnreadMessagesAtom     = MsgNewAtom("TotalUnreadMessages").take();
+    kFolderSizeAtom              = MsgNewAtom("FolderSize").take();
+    kBiffStateAtom               = MsgNewAtom("BiffState").take();
+    kSortOrderAtom               = MsgNewAtom("SortOrder").take();
+    kNewMessagesAtom             = MsgNewAtom("NewMessages").take();
+    kNameAtom                    = MsgNewAtom("Name").take();
+    kSynchronizeAtom             = MsgNewAtom("Synchronize").take();
+    kOpenAtom                    = MsgNewAtom("open").take();
+    kIsDeferredAtom              = MsgNewAtom("isDeferred").take();
+    kIsSecureAtom                = MsgNewAtom("isSecure").take();
+    kCanFileMessagesAtom         = MsgNewAtom("canFileMessages").take();
+    kInVFEditSearchScopeAtom     = MsgNewAtom("inVFEditSearchScope").take();
   }
 
   CreateLiterals(rdf);
@@ -337,7 +339,7 @@ nsresult nsMsgFolderDataSource::CreateArcsOutEnumerator()
 NS_IMPL_ADDREF_INHERITED(nsMsgFolderDataSource, nsMsgRDFDataSource)
 NS_IMPL_RELEASE_INHERITED(nsMsgFolderDataSource, nsMsgRDFDataSource)
 
-NS_IMPL_QUERY_INTERFACE_INHERITED1(nsMsgFolderDataSource, nsMsgRDFDataSource, nsIFolderListener)
+NS_IMPL_QUERY_INTERFACE_INHERITED(nsMsgFolderDataSource, nsMsgRDFDataSource, nsIFolderListener)
 
  // nsIRDFDataSource methods
 NS_IMETHODIMP nsMsgFolderDataSource::GetURI(char* *uri)
@@ -806,8 +808,8 @@ nsMsgFolderDataSource::OnItemPropertyChanged(nsIMsgFolder *resource,
 NS_IMETHODIMP
 nsMsgFolderDataSource::OnItemIntPropertyChanged(nsIMsgFolder *folder,
                                                 nsIAtom *property,
-                                                int32_t oldValue,
-                                                int32_t newValue)
+                                                int64_t oldValue,
+                                                int64_t newValue)
 {
   nsCOMPtr<nsIRDFResource> resource(do_QueryInterface(folder));
   if (kTotalMessagesAtom == property)
@@ -1454,19 +1456,19 @@ nsMsgFolderDataSource::createFolderSizeNode(nsIMsgFolder *folder, nsIRDFNode **t
   nsresult rv = folder->GetIsServer(&isServer);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  int32_t folderSize;
-  if(isServer)
-    folderSize = kDisplayBlankCount;
+  int64_t folderSize;
+  if (isServer) {
+    folderSize = kDisplayBlankCount64;
+  }
   else
   {
     // XXX todo, we are asserting here for news
-    // for offline news, we'd know the size on disk, right?
-    rv = folder->GetSizeOnDisk((uint32_t *) &folderSize);
+    // for offline news, we'd know the size on disk, right? Yes, bug 851275.
+    rv = folder->GetSizeOnDisk(&folderSize);
     NS_ENSURE_SUCCESS(rv, rv);
   }
-  GetFolderSizeNode(folderSize, target);
 
-  return rv;
+  return GetFolderSizeNode(folderSize, target);
 }
 
 nsresult
@@ -1758,7 +1760,7 @@ nsMsgFolderDataSource::OnFolderSortOrderPropertyChanged(nsIRDFResource *folderRe
 }
 
 nsresult
-nsMsgFolderDataSource::OnFolderSizePropertyChanged(nsIRDFResource *folderResource, int32_t oldValue, int32_t newValue)
+nsMsgFolderDataSource::OnFolderSizePropertyChanged(nsIRDFResource *folderResource, int64_t oldValue, int64_t newValue)
 {
   nsCOMPtr<nsIRDFNode> newNode;
   GetFolderSizeNode(newValue, getter_AddRefs(newNode));
@@ -1789,18 +1791,17 @@ nsMsgFolderDataSource::GetNumMessagesNode(int32_t aNumMessages, nsIRDFNode **nod
 }
 
 nsresult
-nsMsgFolderDataSource::GetFolderSizeNode(int32_t aFolderSize, nsIRDFNode **aNode)
+nsMsgFolderDataSource::GetFolderSizeNode(int64_t aFolderSize, nsIRDFNode **aNode)
 {
   nsresult rv;
-  uint32_t folderSize = aFolderSize;
-  if (folderSize == kDisplayBlankCount || folderSize == 0)
+  if (aFolderSize == kDisplayBlankCount64 || aFolderSize == 0)
     createNode(EmptyString().get(), aNode, getRDFService());
-  else if(folderSize == kDisplayQuestionCount)
+  else if (aFolderSize == kDisplayQuestionCount64)
     createNode(MOZ_UTF16("???"), aNode, getRDFService());
   else
   {
     nsAutoString sizeString;
-    rv = FormatFileSize(folderSize, true, sizeString);
+    rv = FormatFileSize(aFolderSize, true, sizeString);
     NS_ENSURE_SUCCESS(rv, rv);
 
     createNode(sizeString.get(), aNode, getRDFService());
