@@ -1,39 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #ifndef _nsMsgCompUtils_H_
 #define _nsMsgCompUtils_H_
@@ -61,6 +29,8 @@ public:
   NS_DECL_NSIMSGCOMPUTILS
 
   nsMsgCompUtils();
+
+private:
   virtual ~nsMsgCompUtils();
 };
 
@@ -78,6 +48,13 @@ char        *nsMsgCreateTempFileName(const char *tFileName);
 // Various utilities for building parts of MIME encoded 
 // messages during message composition
 //
+
+nsresult    mime_sanity_check_fields_recipients (
+                            const char *to,
+                            const char *cc,
+                            const char *bcc,
+                            const char *newsgroups);
+
 nsresult    mime_sanity_check_fields (
                             const char *from,
                             const char *reply_to,
@@ -94,10 +71,11 @@ nsresult    mime_sanity_check_fields (
 
 char        *mime_generate_headers (nsMsgCompFields *fields,
                                     const char *charset,
-                                    nsMsgDeliverMode deliver_mode, nsIPrompt * aPrompt, PRInt32 *status);
+                                    nsMsgDeliverMode deliver_mode,
+                                    nsIPrompt * aPrompt, nsresult *status);
 
 char        *mime_make_separator(const char *prefix);
-char        *mime_gen_content_id(PRUint32 aPartNum, const char *aEmailAddress);
+char        *mime_gen_content_id(uint32_t aPartNum, const char *aEmailAddress);
 
 char        *mime_generate_attachment_headers (
                            const char *type,
@@ -118,7 +96,7 @@ char        *mime_generate_attachment_headers (
 
 char        *msg_generate_message_id (nsIMsgIdentity*);
 
-bool        mime_7bit_data_p (const char *string, PRUint32 size);
+bool        mime_7bit_data_p (const char *string, uint32_t size);
 
 char        *mime_fix_header_1 (const char *string, bool addr_p, bool news_p);
 char        *mime_fix_header (const char *string);
@@ -131,7 +109,7 @@ bool        mime_type_needs_charset (const char *type);
 char        *msg_make_filename_qtext(const char *srcText, bool stripCRLFs);
 
 // Rip apart the URL and extract a reasonable value for the `real_name' slot.
-void        msg_pick_real_name (nsMsgAttachmentHandler *attachment, const PRUnichar *proposedName, const char *charset);
+void        msg_pick_real_name (nsMsgAttachmentHandler *attachment, const char16_t *proposedName, const char *charset);
 
 //
 // Informational calls...
