@@ -161,8 +161,10 @@ Feed.prototype =
     // Only order what you're going to eat...
     this.request.responseType = "document";
     this.request.overrideMimeType("text/xml");
+    this.request.timeout = FeedUtils.REQUEST_TIMEOUT;
     this.request.onload = this.onDownloaded;
     this.request.onerror = this.onDownloadError;
+    this.request.ontimeout = this.onDownloadError;
     FeedCache.putFeed(this);
     this.request.send(null);
   },
@@ -244,6 +246,19 @@ Feed.prototype =
     FeedCache.removeFeed(aFeed.url);
   },
 
+  onUrlChange: function(aFeed, aOldUrl)
+  {
+    if (!aFeed)
+      return;
+
+    // Simulate a cancel after a url update; next cycle will check the new url.
+    aFeed.mInvalidFeed = true;
+    if (aFeed.downloadCallback)
+      aFeed.downloadCallback.downloaded(aFeed, FeedUtils.kNewsBlogCancel);
+
+    FeedCache.removeFeed(aOldUrl);
+  },
+
   get url()
   {
     let ds = FeedUtils.getSubscriptionsDS(this.server);
@@ -251,7 +266,7 @@ Feed.prototype =
     if (url)
       url = url.QueryInterface(Ci.nsIRDFLiteral).Value;
     else
-      url = this.resource.Value;
+      url = this.resource.ValueUTF8;
 
     return url;
   },
@@ -303,9 +318,6 @@ Feed.prototype =
                 old_lastmodified, aLastModified);
     else
       ds.Assert(this.resource, FeedUtils.DC_LASTMODIFIED, aLastModified, true);
-
-    // Do we need to flush every time this property changes?
-    ds.Flush();
   },
 
   get quickMode ()
@@ -336,6 +348,38 @@ Feed.prototype =
                 aNewQuickMode, true);
   },
 
+  get options ()
+  {
+    let ds = FeedUtils.getSubscriptionsDS(this.server);
+    let options = ds.GetTarget(this.resource, FeedUtils.FZ_OPTIONS, true);
+    if (options)
+      return JSON.parse(options.QueryInterface(Ci.nsIRDFLiteral).Value);
+
+    return null;
+  },
+
+  set options (aOptions)
+  {
+    let newOptions = aOptions ? FeedUtils.newOptions(aOptions) :
+                                FeedUtils._optionsDefault;
+    let ds = FeedUtils.getSubscriptionsDS(this.server);
+    newOptions = FeedUtils.rdf.GetLiteral(JSON.stringify(newOptions));
+    let oldOptions = ds.GetTarget(this.resource, FeedUtils.FZ_OPTIONS, true);
+    if (oldOptions)
+      ds.Change(this.resource, FeedUtils.FZ_OPTIONS, oldOptions, newOptions);
+    else
+      ds.Assert(this.resource, FeedUtils.FZ_OPTIONS, newOptions, true);
+  },
+
+  categoryPrefs: function ()
+  {
+    let categoryPrefsAcct = FeedUtils.getOptionsAcct(this.server).category;
+    if (!this.options)
+      return categoryPrefsAcct;
+
+    return this.options.category;
+  },
+
   get link ()
   {
     let ds = FeedUtils.getSubscriptionsDS(this.server);
@@ -364,9 +408,9 @@ Feed.prototype =
   {
     // Create a feed parser which will parse the feed.
     let parser = new FeedParser();
-    this.itemsToStore = parser.parseFeed(this,
-                                         this.request.responseXML,
-                                         this.request.channel.URI);
+    this.itemsToStore = parser.parseFeed(this, this.request.responseXML);
+    delete parser;
+
     if (this.mInvalidFeed)
     {
       this.request = null;
@@ -520,7 +564,7 @@ Feed.prototype =
         item.feed.folder.callFilterPlugins(null);
       }
 
-      this.cleanupParsingState(item.feed, FeedUtils.kNewsBlogSuccess);
+      this.cleanupParsingState(this, FeedUtils.kNewsBlogSuccess);
     }
   },
 
@@ -538,9 +582,6 @@ Feed.prototype =
     ds.Flush();
     FeedUtils.log.debug("Feed.cleanupParsingState: items stored - " + this.itemsStored);
 
-    if (aFeed.downloadCallback)
-      aFeed.downloadCallback.downloaded(aFeed, aCode);
-
     // Force the xml http request to go away.  This helps reduce some nasty
     // assertions on shut down.
     this.request = null;
@@ -548,6 +589,9 @@ Feed.prototype =
     this.itemsToStoreIndex = 0;
     this.itemsStored = 0;
     this.storeItemsTimer = null;
+
+    if (aFeed.downloadCallback)
+      aFeed.downloadCallback.downloaded(aFeed, aCode);
   },
 
   // nsITimerCallback
