@@ -10,11 +10,6 @@
   Text import addressbook interfaces
 
 */
-#ifdef MOZ_LOGGING
-// sorry, this has to be before the pre-compiled header
-#define FORCE_PR_LOG /* Allow logging in the release build */
-#endif
-
 #include "nscore.h"
 #include "nsIServiceManager.h"
 #include "nsCOMPtr.h"
@@ -53,7 +48,7 @@
 static NS_DEFINE_IID(kISupportsIID, NS_ISUPPORTS_IID);
 PRLogModuleInfo* TEXTIMPORTLOGMODULE;
 
-class ImportAddressImpl : public nsIImportAddressBooks
+class ImportAddressImpl MOZ_FINAL : public nsIImportAddressBooks
 {
 public:
   ImportAddressImpl(nsIStringBundle* aStringBundle);
@@ -105,6 +100,7 @@ private:
   static void SanitizeSampleData(nsString& val);
 
 private:
+  ~ImportAddressImpl() {}
   nsTextAddress m_text;
   bool m_haveDelim;
   nsCOMPtr<nsIFile> m_fileLoc;
@@ -132,7 +128,7 @@ nsTextImport::~nsTextImport()
   IMPORT_LOG0("nsTextImport Module Deleted\n");
 }
 
-NS_IMPL_ISUPPORTS1(nsTextImport, nsIImportModule)
+NS_IMPL_ISUPPORTS(nsTextImport, nsIImportModule)
 
 NS_IMETHODIMP nsTextImport::GetName(char16_t **name)
 {
@@ -219,7 +215,7 @@ ImportAddressImpl::ImportAddressImpl(nsIStringBundle* aStringBundle) :
   m_haveDelim = false;
 }
 
-NS_IMPL_ISUPPORTS1(ImportAddressImpl, nsIImportAddressBooks)
+NS_IMPL_ISUPPORTS(ImportAddressImpl, nsIImportAddressBooks)
 
 
 NS_IMETHODIMP ImportAddressImpl::GetAutoFind(char16_t **addrDescription, bool *_retval)
