@@ -1,40 +1,7 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Pierre Phaneuf <pp@ludusdesign.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "msgCore.h"    // precompiled header...
 #include "nsCOMPtr.h"
@@ -68,9 +35,9 @@ nsMailboxService::nsMailboxService()
 nsMailboxService::~nsMailboxService()
 {}
 
-NS_IMPL_ISUPPORTS4(nsMailboxService, nsIMailboxService, nsIMsgMessageService, nsIProtocolHandler, nsIMsgMessageFetchPartService)
+NS_IMPL_ISUPPORTS(nsMailboxService, nsIMailboxService, nsIMsgMessageService, nsIProtocolHandler, nsIMsgMessageFetchPartService)
 
-nsresult nsMailboxService::ParseMailbox(nsIMsgWindow *aMsgWindow, nsILocalFile *aMailboxPath, nsIStreamListener *aMailboxParser,
+nsresult nsMailboxService::ParseMailbox(nsIMsgWindow *aMsgWindow, nsIFile *aMailboxPath, nsIStreamListener *aMailboxParser,
                     nsIUrlListener * aUrlListener, nsIURI ** aURL)
 {
   NS_ENSURE_ARG_POINTER(aMailboxPath);
@@ -85,21 +52,22 @@ nsresult nsMailboxService::ParseMailbox(nsIMsgWindow *aMsgWindow, nsILocalFile *
     nsCString mailboxPath;
 
     aMailboxPath->GetNativePath(mailboxPath);
-    nsCAutoString buf;
+    nsAutoCString buf;
     MsgEscapeURL(mailboxPath,
                  nsINetUtil::ESCAPE_URL_MINIMAL | nsINetUtil::ESCAPE_URL_FORCED, buf);
     nsEscapeNativePath(buf);
     url->SetUpdatingFolder(true);
     url->SetMsgWindow(aMsgWindow);
-    nsCAutoString uriSpec("mailbox://");
+    nsAutoCString uriSpec("mailbox://");
     uriSpec.Append(buf);
-    url->SetSpec(uriSpec);
+    rv = url->SetSpec(uriSpec);
+    NS_ENSURE_SUCCESS(rv, rv);
 
     mailboxurl->SetMailboxParser(aMailboxParser);
     if (aUrlListener)
       url->RegisterListener(aUrlListener);
 
-    rv = RunMailboxUrl(url, nsnull);
+    rv = RunMailboxUrl(url, nullptr);
     NS_ENSURE_SUCCESS(rv, rv);
 
     if (aURL)
@@ -122,10 +90,10 @@ nsresult nsMailboxService::CopyMessage(const char * aSrcMailboxURI,
     nsMailboxAction mailboxAction = nsIMailboxUrl::ActionMoveMessage;
     if (!moveMessage)
         mailboxAction = nsIMailboxUrl::ActionCopyMessage;
-  return FetchMessage(aSrcMailboxURI, aMailboxCopyHandler, aMsgWindow, aUrlListener, nsnull, mailboxAction, nsnull, aURL);
+  return FetchMessage(aSrcMailboxURI, aMailboxCopyHandler, aMsgWindow, aUrlListener, nullptr, mailboxAction, nullptr, aURL);
 }
 
-nsresult nsMailboxService::CopyMessages(PRUint32 aNumKeys,
+nsresult nsMailboxService::CopyMessages(uint32_t aNumKeys,
                                         nsMsgKey* aMsgKeys,
                                         nsIMsgFolder *srcFolder,
                                         nsIStreamListener * aMailboxCopyHandler,
@@ -187,11 +155,11 @@ nsresult nsMailboxService::FetchMessage(const char* aMessageURI,
   nsMailboxAction actionToUse = mailboxAction;
   nsCOMPtr<nsIURI> url;
   nsCOMPtr<nsIMsgMailNewsUrl> msgUrl;
-  nsCAutoString uriString(aMessageURI);
+  nsAutoCString uriString(aMessageURI);
 
   if (!strncmp(aMessageURI, "file:", 5))
   {
-    PRInt64 fileSize;
+    int64_t fileSize;
     nsCOMPtr<nsIURI> fileUri;
     rv = NS_NewURI(getter_AddRefs(fileUri), aMessageURI);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -211,7 +179,7 @@ nsresult nsMailboxService::FetchMessage(const char* aMessageURI,
     {
       msgUrl->SetMsgWindow(aMsgWindow);
       nsCOMPtr <nsIMailboxUrl> mailboxUrl = do_QueryInterface(msgUrl, &rv);
-      mailboxUrl->SetMessageSize((PRUint32) fileSize);
+      mailboxUrl->SetMessageSize((uint32_t) fileSize);
       nsCOMPtr <nsIMsgHeaderSink> headerSink;
        // need to tell the header sink to capture some headers to create a fake db header
        // so we can do reply to a .eml file or a rfc822 msg attachment.
@@ -222,7 +190,7 @@ nsresult nsMailboxService::FetchMessage(const char* aMessageURI,
         nsCOMPtr <nsIMsgDBHdr> dummyHeader;
         headerSink->GetDummyMsgHeader(getter_AddRefs(dummyHeader));
         if (dummyHeader)
-          dummyHeader->SetMessageSize((PRUint32) fileSize);
+          dummyHeader->SetMessageSize((uint32_t) fileSize);
       }
     }
   }
@@ -230,7 +198,7 @@ nsresult nsMailboxService::FetchMessage(const char* aMessageURI,
   {
     // this happens with forward inline of message/rfc822 attachment
     // opened in a stand-alone msg window.
-    PRInt32 typeIndex = uriString.Find("&type=application/x-message-display");
+    int32_t typeIndex = uriString.Find("&type=application/x-message-display");
     if (typeIndex != -1)
     {
       uriString.Cut(typeIndex, sizeof("&type=application/x-message-display") - 1);
@@ -302,7 +270,7 @@ NS_IMETHODIMP nsMailboxService::DisplayMessage(const char* aMessageURI,
                                                nsIURI ** aURL)
 {
   return FetchMessage(aMessageURI, aDisplayConsumer,
-    aMsgWindow,aUrlListener, nsnull,
+    aMsgWindow,aUrlListener, nullptr,
     nsIMailboxUrl::ActionFetchMessage, aCharsetOveride, aURL);
 }
 
@@ -320,7 +288,7 @@ nsMailboxService::StreamMessage(const char *aMessageURI,
     // "header=attach" to decide if it wants to convert the data instead of
     // using aConvertData. It turns out to be way too hard to pass aConvertData
     // all the way over to the mailbox protocol object.
-    nsCAutoString aURIString(aMessageURI);
+    nsAutoCString aURIString(aMessageURI);
     if (!aAdditionalHeader.IsEmpty())
     {
       aURIString.FindChar('?') == -1 ? aURIString += "?" : aURIString += "&";
@@ -328,8 +296,8 @@ nsMailboxService::StreamMessage(const char *aMessageURI,
       aURIString += aAdditionalHeader;
     }
 
-    return FetchMessage(aURIString.get(), aConsumer, aMsgWindow, aUrlListener, nsnull,
-                                        nsIMailboxUrl::ActionFetchMessage, nsnull, aURL);
+    return FetchMessage(aURIString.get(), aConsumer, aMsgWindow, aUrlListener, nullptr,
+                                        nsIMailboxUrl::ActionFetchMessage, nullptr, aURL);
 }
 
 NS_IMETHODIMP nsMailboxService::StreamHeaders(const char *aMessageURI,
@@ -340,7 +308,7 @@ NS_IMETHODIMP nsMailboxService::StreamHeaders(const char *aMessageURI,
 {
   NS_ENSURE_ARG_POINTER(aMessageURI);
   NS_ENSURE_ARG_POINTER(aConsumer);
-  nsCAutoString folderURI;
+  nsAutoCString folderURI;
   nsMsgKey msgKey;
   nsCOMPtr<nsIMsgFolder> folder;
   nsresult rv = DecomposeMailboxURI(aMessageURI, getter_AddRefs(folder), &msgKey);
@@ -348,10 +316,14 @@ NS_IMETHODIMP nsMailboxService::StreamHeaders(const char *aMessageURI,
     return NS_MSG_MESSAGE_NOT_FOUND;
 
   nsCOMPtr<nsIInputStream> inputStream;
-  PRInt64 messageOffset;
-  PRUint32 messageSize;
+  int64_t messageOffset;
+  uint32_t messageSize;
   rv = folder->GetOfflineFileStream(msgKey, &messageOffset, &messageSize, getter_AddRefs(inputStream));
   NS_ENSURE_SUCCESS(rv, rv);
+
+  // GetOfflineFileStream returns NS_OK but null inputStream when there is an error getting the database
+  if (!inputStream)
+    return NS_ERROR_FAILURE;
   return MsgStreamMsgHeaders(inputStream, aConsumer);
 }
 
@@ -373,7 +345,7 @@ NS_IMETHODIMP nsMailboxService::OpenAttachment(const char *aContentType,
                                                nsIUrlListener *aUrlListener)
 {
   nsCOMPtr <nsIURI> URL;
-  nsCAutoString urlString(aUrl);
+  nsAutoCString urlString(aUrl);
   urlString += "&type=";
   urlString += aContentType;
   urlString += "&filename=";
@@ -381,10 +353,6 @@ NS_IMETHODIMP nsMailboxService::OpenAttachment(const char *aContentType,
   CreateStartupUrl(urlString.get(), getter_AddRefs(URL));
   nsresult rv;
 
-  nsCOMPtr<nsIMsgMailNewsUrl> mailboxUrl(do_QueryInterface(URL, &rv));
-  if (NS_SUCCEEDED(rv) && mailboxUrl)
-    mailboxUrl->SetMsgWindow(aMsgWindow);
-
   // try to run the url in the docshell...
   nsCOMPtr<nsIDocShell> docShell(do_QueryInterface(aDisplayConsumer, &rv));
   // if we were given a docShell, run the url in the docshell..otherwise just run it normally.
@@ -446,7 +414,7 @@ NS_IMETHODIMP nsMailboxService::GetUrlForUri(const char *aMessageURI, nsIURI **a
 
   nsresult rv = NS_OK;
   nsCOMPtr<nsIMailboxUrl> mailboxurl;
-  rv = PrepareMessageUrl(aMessageURI, nsnull, nsIMailboxUrl::ActionFetchMessage, getter_AddRefs(mailboxurl), aMsgWindow);
+  rv = PrepareMessageUrl(aMessageURI, nullptr, nsIMailboxUrl::ActionFetchMessage, getter_AddRefs(mailboxurl), aMsgWindow);
   if (NS_SUCCEEDED(rv) && mailboxurl)
     rv = CallQueryInterface(mailboxurl, aURL);
   return rv;
@@ -489,7 +457,7 @@ nsresult nsMailboxService::PrepareMessageUrl(const char * aSrcMsgMailboxURI, nsI
   {
     // okay now generate the url string
     char * urlSpec;
-    nsCAutoString folderURI;
+    nsAutoCString folderURI;
     nsMsgKey msgKey;
     nsCString folderPath;
     const char *part = PL_strstr(aSrcMsgMailboxURI, "part=");
@@ -501,7 +469,7 @@ nsresult nsMailboxService::PrepareMessageUrl(const char * aSrcMsgMailboxURI, nsI
     if (NS_SUCCEEDED(rv))
     {
       // set up the url spec and initialize the url with it.
-      nsCAutoString buf;
+      nsAutoCString buf;
       MsgEscapeURL(folderPath,
                    nsINetUtil::ESCAPE_URL_DIRECTORY | nsINetUtil::ESCAPE_URL_FORCED, buf);
       if (mPrintingOperation)
@@ -514,7 +482,9 @@ nsresult nsMailboxService::PrepareMessageUrl(const char * aSrcMsgMailboxURI, nsI
         urlSpec = PR_smprintf("mailbox://%s?number=%lu", buf.get(), msgKey);
 
       nsCOMPtr <nsIMsgMailNewsUrl> url = do_QueryInterface(*aMailboxUrl);
-      url->SetSpec(nsDependentCString(urlSpec));
+      rv = url->SetSpec(nsDependentCString(urlSpec));
+      NS_ENSURE_SUCCESS(rv, rv);
+
       PR_smprintf_free(urlSpec);
 
       (*aMailboxUrl)->SetMailboxAction(aMailboxAction);
@@ -543,14 +513,14 @@ NS_IMETHODIMP nsMailboxService::GetScheme(nsACString &aScheme)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMailboxService::GetDefaultPort(PRInt32 *aDefaultPort)
+NS_IMETHODIMP nsMailboxService::GetDefaultPort(int32_t *aDefaultPort)
 {
   NS_ENSURE_ARG_POINTER(aDefaultPort);
   *aDefaultPort = -1;  // mailbox doesn't use a port!!!!!
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMailboxService::AllowPort(PRInt32 port, const char *scheme, bool *_retval)
+NS_IMETHODIMP nsMailboxService::AllowPort(int32_t port, const char *scheme, bool *_retval)
 {
   NS_ENSURE_ARG_POINTER(_retval);
   // don't override anything.
@@ -558,7 +528,7 @@ NS_IMETHODIMP nsMailboxService::AllowPort(PRInt32 port, const char *scheme, bool
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMailboxService::GetProtocolFlags(PRUint32 *result)
+NS_IMETHODIMP nsMailboxService::GetProtocolFlags(uint32_t *result)
 {
   NS_ENSURE_ARG_POINTER(result);
   *result = URI_STD | URI_FORBIDS_AUTOMATIC_DOCUMENT_REPLACEMENT |
@@ -577,35 +547,42 @@ NS_IMETHODIMP nsMailboxService::NewURI(const nsACString &aSpec,
 {
   NS_ENSURE_ARG_POINTER(_retval);
   *_retval = 0;
-  nsresult rv = NS_OK;
+  nsresult rv;
   nsCOMPtr<nsIURI> aMsgUri = do_CreateInstance(NS_MAILBOXURL_CONTRACTID, &rv);
-
-  if (NS_SUCCEEDED(rv))
+  NS_ENSURE_SUCCESS(rv, rv);
+  // SetSpec calls below may fail if the mailbox url is of the form
+  // mailbox://<account>/<mailbox name>?... instead of
+  // mailbox://<path to folder>?.... This is the case for pop3 download urls.
+  // We know this, and the failure is harmless.
+  if (aBaseURI)
   {
-    if (aBaseURI)
-    {
-      nsCAutoString newSpec;
-      rv = aBaseURI->Resolve(aSpec, newSpec);
-      if (NS_FAILED(rv))
-        return rv;
-      aMsgUri->SetSpec(newSpec);
-    }
-    else
-    {
-      aMsgUri->SetSpec(aSpec);
-    }
-    aMsgUri.swap(*_retval);
+    nsAutoCString newSpec;
+    rv = aBaseURI->Resolve(aSpec, newSpec);
+    NS_ENSURE_SUCCESS(rv, rv);
+    (void) aMsgUri->SetSpec(newSpec);
   }
+  else
+  {
+    (void) aMsgUri->SetSpec(aSpec);
+  }
+  aMsgUri.swap(*_retval);
 
   return rv;
 }
 
 NS_IMETHODIMP nsMailboxService::NewChannel(nsIURI *aURI, nsIChannel **_retval)
+{
+  return NewChannel2(aURI, nullptr, _retval);
+}
+
+NS_IMETHODIMP nsMailboxService::NewChannel2(nsIURI *aURI,
+                                            nsILoadInfo *aLoadInfo,
+                                            nsIChannel **_retval)
 {
   NS_ENSURE_ARG_POINTER(aURI);
   NS_ENSURE_ARG_POINTER(_retval);
   nsresult rv = NS_OK;
-  nsCAutoString spec;
+  nsAutoCString spec;
   aURI->GetSpec(spec);
 
   if (spec.Find("?uidl=") >= 0 || spec.Find("&uidl=") >= 0)
@@ -645,8 +622,8 @@ nsresult nsMailboxService::DisplayMessageForPrinting(const char* aMessageURI,
                                                      nsIURI ** aURL)
 {
   mPrintingOperation = true;
-  nsresult rv = FetchMessage(aMessageURI, aDisplayConsumer, aMsgWindow,aUrlListener, nsnull,
-    nsIMailboxUrl::ActionFetchMessage, nsnull, aURL);
+  nsresult rv = FetchMessage(aMessageURI, aDisplayConsumer, aMsgWindow,aUrlListener, nullptr,
+    nsIMailboxUrl::ActionFetchMessage, nullptr, aURL);
   mPrintingOperation = false;
   return rv;
 }
@@ -664,7 +641,7 @@ nsMailboxService::DecomposeMailboxURI(const char * aMessageURI, nsIMsgFolder **
   NS_ENSURE_ARG_POINTER(aMsgKey);
 
   nsresult rv = NS_OK;
-  nsCAutoString folderURI;
+  nsAutoCString folderURI;
   rv = nsParseLocalMessageURI(aMessageURI, folderURI, aMsgKey);
   NS_ENSURE_SUCCESS(rv,rv);
 
