@@ -1832,21 +1832,19 @@ int32_t nsParseNewMailState::PublishMsgHeader(nsIMsgWindow *msgWindow)
               {
                 nsCOMPtr <nsIMsgFolder> trash;
                 GetTrashFolder(getter_AddRefs(trash));
-                if (trash)
-                {
+                if (trash) {
                   uint32_t newFlags;
-                bool msgMoved;
+                  bool msgMoved;
                   m_newMsgHdr->AndFlags(~nsMsgMessageFlags::New, &newFlags);
-                nsCOMPtr<nsIMsgPluggableStore> msgStore;
-                rv = m_downloadFolder->GetMsgStore(getter_AddRefs(msgStore));
-                if (NS_SUCCEEDED(rv))
-                  msgStore->MoveNewlyDownloadedMessage(m_newMsgHdr, trash, &msgMoved);
-                if (!msgMoved)
-                {
-                  MoveIncorporatedMessage(m_newMsgHdr, m_mailDB, trash,
-                                                          nullptr, msgWindow);
-                  m_mailDB->RemoveHeaderMdbRow(m_newMsgHdr);
-                }
+                  nsCOMPtr<nsIMsgPluggableStore> msgStore;
+                  rv = m_downloadFolder->GetMsgStore(getter_AddRefs(msgStore));
+                  if (NS_SUCCEEDED(rv))
+                    rv = msgStore->MoveNewlyDownloadedMessage(m_newMsgHdr, trash, &msgMoved);
+                  if (NS_SUCCEEDED(rv) && !msgMoved) {
+                    MoveIncorporatedMessage(m_newMsgHdr, m_mailDB, trash,
+                                            nullptr, msgWindow);
+                    m_mailDB->RemoveHeaderMdbRow(m_newMsgHdr);
+                  }
                 }
               }
               break;
@@ -1992,6 +1990,9 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
     nsMsgRuleActionType actionType;
     if (NS_SUCCEEDED(filterAction->GetType(&actionType)))
     {
+      if (loggingEnabled)
+        (void)filter->LogRuleHit(filterAction, msgHdr);
+
       nsCString actionTargetFolderUri;
       if (actionType == nsMsgFilterAction::MoveToFolder ||
           actionType == nsMsgFilterAction::CopyToFolder)
@@ -2045,9 +2046,7 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
             NS_ENSURE_TRUE(m_moveCoalescer, NS_ERROR_OUT_OF_MEMORY);
             nsMsgKey msgKey;
             (void) msgHdr->GetMessageKey(&msgKey);
-            m_moveCoalescer->AddMove(destIFolder , msgKey);
-            if (loggingEnabled)
-              (void)filter->LogRuleHit(filterAction, msgHdr);
+            m_moveCoalescer->AddMove(destIFolder, msgKey);
             err = NS_OK;
             msgIsNew = false;
           }
@@ -2056,16 +2055,11 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
             nsCOMPtr<nsIMsgPluggableStore> msgStore;
             err = m_downloadFolder->GetMsgStore(getter_AddRefs(msgStore));
             if (NS_SUCCEEDED(err))
-              msgStore->MoveNewlyDownloadedMessage(msgHdr, destIFolder, &msgMoved);
-            if (!msgMoved)
+              err = msgStore->MoveNewlyDownloadedMessage(msgHdr, destIFolder, &msgMoved);
+            if (NS_SUCCEEDED(err) && !msgMoved)
               err = MoveIncorporatedMessage(msgHdr, m_mailDB, destIFolder,
                                             filter, msgWindow);
             m_msgMovedByFilter = NS_SUCCEEDED(err);
-            if (m_msgMovedByFilter)
-            {
-              if (loggingEnabled)
-                (void)filter->LogRuleHit(filterAction, msgHdr);
-            }
           }
         }
         *applyMore = false;
@@ -2249,8 +2243,6 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
       default:
         break;
       }
-      if (loggingEnabled && actionType != nsMsgFilterAction::MoveToFolder && actionType != nsMsgFilterAction::Delete)
-        (void)filter->LogRuleHit(filterAction, msgHdr);
     }
   }
   if (!msgIsNew)
@@ -2426,6 +2418,10 @@ nsresult nsParseNewMailState::AppendMsgFromStream(nsIInputStream *fileStream,
   return store->FinishNewMessage(destOutputStream, aHdr);
 }
 
+/*
+ * Moves message pointed to by mailHdr into folder destIFolder.
+ * After successful move mailHdr is no longer usable by the caller.
+ */
 nsresult nsParseNewMailState::MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr,
                                                       nsIMsgDatabase *sourceDB,
                                                       nsIMsgFolder *destIFolder,
