@@ -47,7 +47,7 @@ nsPop3Service::nsPop3Service()
 nsPop3Service::~nsPop3Service()
 {}
 
-NS_IMPL_ISUPPORTS3(nsPop3Service,
+NS_IMPL_ISUPPORTS(nsPop3Service,
                    nsIPop3Service,
                    nsIProtocolHandler,
                    nsIMsgProtocolInfo)
@@ -476,6 +476,13 @@ void nsPop3Service::AlertServerBusy(nsIMsgMailNewsUrl *url)
 }
 
 NS_IMETHODIMP nsPop3Service::NewChannel(nsIURI *aURI, nsIChannel **_retval)
+{
+  return NewChannel2(aURI, nullptr, _retval);
+}
+
+NS_IMETHODIMP nsPop3Service::NewChannel2(nsIURI *aURI,
+                                         nsILoadInfo *aLoadInfo,
+                                         nsIChannel **_retval)
 {
   NS_ENSURE_ARG_POINTER(aURI);
   nsresult rv;
