@@ -2,10 +2,6 @@
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
-#ifdef MOZ_LOGGING
-#define FORCE_PR_LOG
-#endif
-
 
 #include "msgCore.h"    // precompiled header...
 #include "nsPop3Sink.h"
@@ -44,7 +40,7 @@
 
 extern PRLogModuleInfo *POP3LOGMODULE;
 
-NS_IMPL_ISUPPORTS1(nsPop3Sink, nsIPop3Sink)
+NS_IMPL_ISUPPORTS(nsPop3Sink, nsIPop3Sink)
 
 nsPop3Sink::nsPop3Sink()
 {
