@@ -123,6 +123,7 @@ public:
   {
   }
 
+private:
   virtual ~MimePgpeData()
   {
   }
@@ -222,7 +223,7 @@ MimePgpe_free(void *output_closure)
 
 
 ////////////////////////////////////////////////////////////////////////////
-NS_IMPL_ISUPPORTS5(nsPgpMimeProxy,
+NS_IMPL_ISUPPORTS(nsPgpMimeProxy,
                               nsIPgpMimeProxy,
                               nsIRequestObserver,
                               nsIStreamListener,
@@ -551,7 +552,6 @@ nsPgpMimeProxy::OnDataAvailable(nsIRequest* aRequest, nsISupports* aContext,
   NS_ENSURE_TRUE(mInitialized, NS_ERROR_NOT_INITIALIZED);
 
   NS_ENSURE_ARG(aInputStream);
-  NS_ENSURE_ARG_MIN(aLength, 0);
 
   char buf[kCharMax];
   uint32_t readCount, readMax;
