@@ -446,8 +446,28 @@ nsNntpIncomingServer::CloseCachedConnections()
   return NS_OK;
 }
 
-NS_IMPL_SERVERPREF_INT(nsNntpIncomingServer, MaximumConnectionsNumber,
-                       "max_cached_connections")
+NS_IMETHODIMP
+nsNntpIncomingServer::GetMaximumConnectionsNumber(int32_t *aMaxConnections)
+{
+  NS_ENSURE_ARG_POINTER(aMaxConnections);
+
+  nsresult rv = GetIntValue("max_cached_connections", aMaxConnections);
+  // Get our maximum connection count. We need at least 1. If the value is 0,
+  // we use the default. If it's negative, we treat that as 1.
+  if (NS_SUCCEEDED(rv) && *aMaxConnections > 0)
+    return NS_OK;
+
+  *aMaxConnections = (NS_FAILED(rv) || (*aMaxConnections == 0)) ? 2 : 1;
+  (void)SetMaximumConnectionsNumber(*aMaxConnections);
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsNntpIncomingServer::SetMaximumConnectionsNumber(int32_t aMaxConnections)
+{
+  return SetIntValue("max_cached_connections", aMaxConnections);
+}
 
 bool
 nsNntpIncomingServer::ConnectionTimeOut(nsINNTPProtocol* aConnection)
@@ -492,27 +512,13 @@ nsNntpIncomingServer::CreateProtocolInstance(nsINNTPProtocol ** aNntpConnection,
   return rv;
 }
 
-/* By default, allow the user to open at most this many connections to one news host */
-#define kMaxConnectionsPerHost 2
 
 nsresult
 nsNntpIncomingServer::GetNntpConnection(nsIURI * aUri, nsIMsgWindow *aMsgWindow,
                                         nsINNTPProtocol ** aNntpConnection)
 {
-  // Get our maximum connection count. We need at least 1. If the value is 0,
-  // we use the default. If it's negative, we treat that as 1.
-  int32_t maxConnections = kMaxConnectionsPerHost;
-  nsresult rv = GetMaximumConnectionsNumber(&maxConnections);
-  if (NS_FAILED(rv) || maxConnections == 0)
-  {
-    maxConnections = kMaxConnectionsPerHost;
-    SetMaximumConnectionsNumber(maxConnections);
-  }
-  else if (maxConnections < 1)
-  {
-    maxConnections = 1;
-    SetMaximumConnectionsNumber(maxConnections);
-  }
+  int32_t maxConnections;
+  (void)GetMaximumConnectionsNumber(&maxConnections);
 
   // Find a non-busy connection
   nsCOMPtr<nsINNTPProtocol> connection;
@@ -546,7 +552,7 @@ nsNntpIncomingServer::GetNntpConnection(nsIURI * aUri, nsIMsgWindow *aMsgWindow,
   {
     // We have room for another connection. Create this connection and return
     // it to the caller.
-    rv = CreateProtocolInstance(aNntpConnection, aUri, aMsgWindow);
+    nsresult rv = CreateProtocolInstance(aNntpConnection, aUri, aMsgWindow);
     NS_ENSURE_SUCCESS(rv, rv);
   }
   else
@@ -794,7 +800,6 @@ nsNntpIncomingServer::WriteHostInfoFile()
 {
   if (!mHostInfoHasChanged)
     return NS_OK;
-  int32_t firstnewdate = (int32_t)mFirstNewDate;
 
   mLastUpdatedTime = uint32_t(PR_Now() / PR_USEC_PER_SEC);
 
@@ -824,9 +829,6 @@ nsNntpIncomingServer::WriteHostInfoFile()
   nsAutoCString dateStr("lastgroupdate=");
   dateStr.AppendInt(mLastUpdatedTime);
   WriteLine(hostInfoStream, dateStr);
-  dateStr ="firstnewdate=";
-  dateStr.AppendInt(firstnewdate);
-  WriteLine(hostInfoStream, dateStr);
   dateStr = "uniqueid=";
   dateStr.AppendInt(mUniqueId);
   WriteLine(hostInfoStream, dateStr);
@@ -1160,13 +1162,13 @@ nsNntpIncomingServer::GetSubscribeListener(nsISubscribeListener **aListener)
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::Subscribe(const PRUnichar *aUnicharName)
+nsNntpIncomingServer::Subscribe(const char16_t *aUnicharName)
 {
   return SubscribeToNewsgroup(NS_ConvertUTF16toUTF8(aUnicharName));
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::Unsubscribe(const PRUnichar *aUnicharName)
+nsNntpIncomingServer::Unsubscribe(const char16_t *aUnicharName)
 {
   NS_ENSURE_ARG_POINTER(aUnicharName);
 
@@ -1246,8 +1248,6 @@ nsNntpIncomingServer::HandleLine(const char* line, uint32_t line_size)
       *equalPos++ = '\0';
       if (PL_strcmp(line, "lastgroupdate") == 0) {
         mLastUpdatedTime = strtoul(equalPos, nullptr, 10);
-      } else if (PL_strcmp(line, "firstnewdate") == 0) {
-        mFirstNewDate = strtol(equalPos, nullptr, 16);
       } else if (PL_strcmp(line, "uniqueid") == 0) {
         mUniqueId = strtol(equalPos, nullptr, 16);
       } else if (PL_strcmp(line, "version") == 0) {
@@ -1575,10 +1575,10 @@ nsNntpIncomingServer::GroupNotFound(nsIMsgWindow *aMsgWindow,
   NS_ConvertUTF8toUTF16 hostStr(hostname);
 
   nsString groupName(aName);
-  const PRUnichar *formatStrings[2] = { groupName.get(), hostStr.get() };
+  const char16_t *formatStrings[2] = { groupName.get(), hostStr.get() };
   nsString confirmText;
   rv = bundle->FormatStringFromName(
-                    NS_LITERAL_STRING("autoUnsubscribeText").get(),
+                    MOZ_UTF16("autoUnsubscribeText"),
                     formatStrings, 2,
                     getter_Copies(confirmText));
   NS_ENSURE_SUCCESS(rv,rv);
@@ -1737,7 +1737,7 @@ nsNntpIncomingServer::GetCellProperties(int32_t row, nsITreeColumn* col, nsAStri
 
     NS_ENSURE_ARG_POINTER(col);
 
-    const PRUnichar* colID;
+    const char16_t* colID;
     col->GetIdConst(&colID);
     if (colID[0] == 's') {
         // if <name> is in our temporary list of subscribed groups
@@ -1863,7 +1863,7 @@ nsNntpIncomingServer::GetCellText(int32_t row, nsITreeColumn* col, nsAString& _r
 
     NS_ENSURE_ARG_POINTER(col);
 
-    const PRUnichar* colID;
+    const char16_t* colID;
     col->GetIdConst(&colID);
 
     nsresult rv = NS_OK;
@@ -1970,19 +1970,19 @@ nsNntpIncomingServer::SetCellText(int32_t row, nsITreeColumn* col, const nsAStri
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::PerformAction(const PRUnichar *action)
+nsNntpIncomingServer::PerformAction(const char16_t *action)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::PerformActionOnRow(const PRUnichar *action, int32_t row)
+nsNntpIncomingServer::PerformActionOnRow(const char16_t *action, int32_t row)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::PerformActionOnCell(const PRUnichar *action, int32_t row, nsITreeColumn* col)
+nsNntpIncomingServer::PerformActionOnCell(const char16_t *action, int32_t row, nsITreeColumn* col)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
