@@ -68,7 +68,7 @@ nsNntpService::~nsNntpService()
   // do nothing
 }
 
-NS_IMPL_ISUPPORTS7(nsNntpService, nsINntpService, nsIMsgMessageService,
+NS_IMPL_ISUPPORTS(nsNntpService, nsINntpService, nsIMsgMessageService,
   nsIProtocolHandler, nsIMsgProtocolInfo, nsICommandLineHandler,
   nsIMsgMessageFetchPartService, nsIContentHandler)
 
@@ -1231,6 +1231,13 @@ NS_IMETHODIMP nsNntpService::NewURI(const nsACString &aSpec,
 }
 
 NS_IMETHODIMP nsNntpService::NewChannel(nsIURI *aURI, nsIChannel **_retval)
+{
+  return NewChannel2(aURI, nullptr, _retval);
+}
+
+NS_IMETHODIMP nsNntpService::NewChannel2(nsIURI *aURI,
+                                         nsILoadInfo *aLoadInfo,
+                                         nsIChannel **_retval)
 {
   NS_ENSURE_ARG_POINTER(aURI);
   nsresult rv = NS_OK;
