@@ -5,12 +5,13 @@
 MOZ_APP_BASENAME=B2G
 MOZ_APP_VENDOR=Mozilla
 
-MOZ_APP_VERSION=18.0
+MOZ_APP_VERSION=36.0
 MOZ_APP_UA_NAME=Firefox
 
 MOZ_UA_OS_AGNOSTIC=1
 
-MOZ_B2G_VERSION=1.0.0
+MOZ_B2G_VERSION=2.2.0.0-prerelease
+MOZ_B2G_OS_NAME=Boot2Gecko
 
 MOZ_BRANDING_DIRECTORY=b2g/branding/unofficial
 MOZ_OFFICIAL_BRANDING_DIRECTORY=b2g/branding/official
@@ -18,12 +19,16 @@ MOZ_OFFICIAL_BRANDING_DIRECTORY=b2g/branding/official
 
 MOZ_SAFE_BROWSING=
 MOZ_SERVICES_COMMON=1
-MOZ_SERVICES_HEALTHREPORT=1
 MOZ_SERVICES_METRICS=1
+MOZ_CAPTIVEDETECT=1
 
 MOZ_WEBSMS_BACKEND=1
-MOZ_DISABLE_DOMCRYPTO=1
+MOZ_NO_SMART_CARDS=1
 MOZ_APP_STATIC_INI=1
+NSS_NO_LIBPKIX=1
+NSS_DISABLE_DBM=1
+MOZ_NO_EV_CERTS=1
+MOZ_DISABLE_EXPORT_JS=1
 
 if test "$OS_TARGET" = "Android"; then
 MOZ_CAPTURE=1
@@ -40,13 +45,25 @@ else
 MOZ_XULRUNNER=
 fi
 
+MOZ_MEDIA_NAVIGATOR=1
+
 MOZ_APP_ID={3c2e2abc-06d4-11e1-ac3b-374f68613e61}
-MOZ_EXTENSION_MANAGER=1
 
-MOZ_SYS_MSG=1
 MOZ_TIME_MANAGER=1
 
 MOZ_PAY=1
 MOZ_TOOLKIT_SEARCH=
 MOZ_PLACES=
 MOZ_B2G=1
+
+if test "$OS_TARGET" = "Android"; then
+MOZ_NUWA_PROCESS=1
+MOZ_B2G_LOADER=1
+fi
+
+MOZ_JSDOWNLOADS=1
+
+MOZ_BUNDLED_FONTS=1
+
+export JSGC_GENERATIONAL=1
+export JS_GC_SMALL_CHUNK_SIZE=1
