@@ -7,7 +7,7 @@
  */
 
 function spawnTest() {
-  let [target, debuggee, panel] = yield initWebAudioEditor(SIMPLE_CONTEXT_URL);
+  let { target, panel } = yield initWebAudioEditor(SIMPLE_CONTEXT_URL);
   let { panelWin } = panel;
   let { gFront, $ } = panelWin;
 
@@ -24,12 +24,12 @@ function spawnTest() {
 
   reload(target);
 
-  let [actors] = yield Promise.all([
+  [actors] = yield Promise.all([
     get3(gFront, "create-node"),
     waitForGraphRendered(panelWin, 3, 2)
   ]);
 
-  let { nodes, edges } = countGraphObjects(panelWin);
+  ({ nodes, edges } = countGraphObjects(panelWin));
   ise(nodes, 3, "after reload, should only be 3 nodes.");
   ise(edges, 2, "after reload, should only be 2 edges.");
 
