@@ -9,6 +9,7 @@
 
 // JS lexical scanner interface.
 
+#include "mozilla/ArrayUtils.h"
 #include "mozilla/DebugOnly.h"
 #include "mozilla/PodOperations.h"
 #include "mozilla/UniquePtr.h"
@@ -20,6 +21,7 @@
 #include "jscntxt.h"
 #include "jspubtd.h"
 
+#include "frontend/TokenKind.h"
 #include "js/Vector.h"
 #include "vm/RegExpObject.h"
 
@@ -28,168 +30,6 @@ struct KeywordInfo;
 namespace js {
 namespace frontend {
 
-// Values of this type are used to index into arrays such as isExprEnding[],
-// so the first value must be zero.
-enum TokenKind {
-    TOK_ERROR = 0,                 // well-known as the only code < EOF
-    TOK_EOF,                       // end of file
-    TOK_EOL,                       // end of line; only returned by peekTokenSameLine()
-    TOK_SEMI,                      // semicolon
-    TOK_COMMA,                     // comma operator
-    TOK_HOOK, TOK_COLON,           // conditional (?:)
-    TOK_INC, TOK_DEC,              // increment/decrement (++ --)
-    TOK_DOT,                       // member operator (.)
-    TOK_TRIPLEDOT,                 // for rest arguments (...)
-    TOK_LB, TOK_RB,                // left and right brackets
-    TOK_LC, TOK_RC,                // left and right curlies (braces)
-    TOK_LP, TOK_RP,                // left and right parentheses
-    TOK_NAME,                      // identifier
-    TOK_NUMBER,                    // numeric constant
-    TOK_STRING,                    // string constant
-    TOK_TEMPLATE_HEAD,             // start of template literal with substitutions
-    TOK_NO_SUBS_TEMPLATE,          // template literal without substitutions
-    TOK_REGEXP,                    // RegExp constant
-    TOK_TRUE,                      // true
-    TOK_FALSE,                     // false
-    TOK_NULL,                      // null
-    TOK_THIS,                      // this
-    TOK_FUNCTION,                  // function keyword
-    TOK_IF,                        // if keyword
-    TOK_ELSE,                      // else keyword
-    TOK_SWITCH,                    // switch keyword
-    TOK_CASE,                      // case keyword
-    TOK_DEFAULT,                   // default keyword
-    TOK_WHILE,                     // while keyword
-    TOK_DO,                        // do keyword
-    TOK_FOR,                       // for keyword
-    TOK_BREAK,                     // break keyword
-    TOK_CONTINUE,                  // continue keyword
-    TOK_VAR,                       // var keyword
-    TOK_CONST,                     // const keyword
-    TOK_WITH,                      // with keyword
-    TOK_RETURN,                    // return keyword
-    TOK_NEW,                       // new keyword
-    TOK_DELETE,                    // delete keyword
-    TOK_TRY,                       // try keyword
-    TOK_CATCH,                     // catch keyword
-    TOK_FINALLY,                   // finally keyword
-    TOK_THROW,                     // throw keyword
-    TOK_DEBUGGER,                  // debugger keyword
-    TOK_YIELD,                     // yield from generator function
-    TOK_LET,                       // let keyword
-    TOK_EXPORT,                    // export keyword
-    TOK_IMPORT,                    // import keyword
-    TOK_RESERVED,                  // reserved keywords
-    TOK_STRICT_RESERVED,           // reserved keywords in strict mode
-
-    // The following token types occupy contiguous ranges to enable easy
-    // range-testing.
-
-    // Binary operators tokens, TOK_OR thru TOK_MOD. These must be in the same
-    // order as F(OR) and friends in FOR_EACH_PARSE_NODE_KIND in ParseNode.h.
-    TOK_OR,                        // logical or (||)
-    TOK_BINOP_FIRST = TOK_OR,
-    TOK_AND,                       // logical and (&&)
-    TOK_BITOR,                     // bitwise-or (|)
-    TOK_BITXOR,                    // bitwise-xor (^)
-    TOK_BITAND,                    // bitwise-and (&)
-
-    // Equality operation tokens, per TokenKindIsEquality.
-    TOK_STRICTEQ,
-    TOK_EQUALITY_START = TOK_STRICTEQ,
-    TOK_EQ,
-    TOK_STRICTNE,
-    TOK_NE,
-    TOK_EQUALITY_LAST = TOK_NE,
-
-    // Relational ops (< <= > >=), per TokenKindIsRelational.
-    TOK_LT,
-    TOK_RELOP_START = TOK_LT,
-    TOK_LE,
-    TOK_GT,
-    TOK_GE,
-    TOK_RELOP_LAST = TOK_GE,
-
-    TOK_INSTANCEOF,                // |instanceof| keyword
-    TOK_IN,                        // |in| keyword
-
-    // Shift ops (<< >> >>>), per TokenKindIsShift.
-    TOK_LSH,
-    TOK_SHIFTOP_START = TOK_LSH,
-    TOK_RSH,
-    TOK_URSH,
-    TOK_SHIFTOP_LAST = TOK_URSH,
-
-    TOK_ADD,
-    TOK_SUB,
-    TOK_MUL,
-    TOK_DIV,
-    TOK_MOD,
-    TOK_BINOP_LAST = TOK_MOD,
-
-    // Unary operation tokens.
-    TOK_TYPEOF,
-    TOK_VOID,
-    TOK_NOT,
-    TOK_BITNOT,
-
-    TOK_ARROW,                     // function arrow (=>)
-
-    // Assignment ops (= += -= etc.), per TokenKindIsAssignment
-    TOK_ASSIGN,
-    TOK_ASSIGNMENT_START = TOK_ASSIGN,
-    TOK_ADDASSIGN,
-    TOK_SUBASSIGN,
-    TOK_BITORASSIGN,
-    TOK_BITXORASSIGN,
-    TOK_BITANDASSIGN,
-    TOK_LSHASSIGN,
-    TOK_RSHASSIGN,
-    TOK_URSHASSIGN,
-    TOK_MULASSIGN,
-    TOK_DIVASSIGN,
-    TOK_MODASSIGN,
-    TOK_ASSIGNMENT_LAST = TOK_MODASSIGN,
-
-    TOK_LIMIT                      // domain size
-};
-
-inline bool
-TokenKindIsBinaryOp(TokenKind tt)
-{
-    return TOK_BINOP_FIRST <= tt && tt <= TOK_BINOP_LAST;
-}
-
-inline bool
-TokenKindIsEquality(TokenKind tt)
-{
-    return TOK_EQUALITY_START <= tt && tt <= TOK_EQUALITY_LAST;
-}
-
-inline bool
-TokenKindIsRelational(TokenKind tt)
-{
-    return TOK_RELOP_START <= tt && tt <= TOK_RELOP_LAST;
-}
-
-inline bool
-TokenKindIsShift(TokenKind tt)
-{
-    return TOK_SHIFTOP_START <= tt && tt <= TOK_SHIFTOP_LAST;
-}
-
-inline bool
-TokenKindIsAssignment(TokenKind tt)
-{
-    return TOK_ASSIGNMENT_START <= tt && tt <= TOK_ASSIGNMENT_LAST;
-}
-
-inline bool
-TokenKindIsDecl(TokenKind tt)
-{
-    return tt == TOK_VAR || tt == TOK_LET;
-}
-
 struct TokenPos {
     uint32_t    begin;  // Offset of the token's first char.
     uint32_t    end;    // Offset of 1 past the token's last char.
@@ -199,9 +39,9 @@ struct TokenPos {
 
     // Return a TokenPos that covers left, right, and anything in between.
     static TokenPos box(const TokenPos &left, const TokenPos &right) {
-        JS_ASSERT(left.begin <= left.end);
-        JS_ASSERT(left.end <= right.begin);
-        JS_ASSERT(right.begin <= right.end);
+        MOZ_ASSERT(left.begin <= left.end);
+        MOZ_ASSERT(left.end <= right.begin);
+        MOZ_ASSERT(right.begin <= right.end);
         return TokenPos(left.begin, right.end);
     }
 
@@ -266,35 +106,35 @@ struct Token
     // pointer, and it's only initialized a very few places, so having a
     // user-defined constructor won't hurt perf.)  See also bug 920318.
     Token()
-      : type(TOK_ERROR),
-        pos(0, 0)
+      : pos(0, 0)
     {
+        MOZ_MAKE_MEM_UNDEFINED(&type, sizeof(type));
     }
 
     // Mutators
 
     void setName(PropertyName *name) {
-        JS_ASSERT(type == TOK_NAME);
-        JS_ASSERT(!IsPoisonedPtr(name));
+        MOZ_ASSERT(type == TOK_NAME);
+        MOZ_ASSERT(!IsPoisonedPtr(name));
         u.name = name;
     }
 
     void setAtom(JSAtom *atom) {
-        JS_ASSERT (type == TOK_STRING ||
+        MOZ_ASSERT(type == TOK_STRING ||
                    type == TOK_TEMPLATE_HEAD ||
                    type == TOK_NO_SUBS_TEMPLATE);
-        JS_ASSERT(!IsPoisonedPtr(atom));
+        MOZ_ASSERT(!IsPoisonedPtr(atom));
         u.atom = atom;
     }
 
     void setRegExpFlags(js::RegExpFlag flags) {
-        JS_ASSERT(type == TOK_REGEXP);
-        JS_ASSERT((flags & AllFlags) == flags);
+        MOZ_ASSERT(type == TOK_REGEXP);
+        MOZ_ASSERT((flags & AllFlags) == flags);
         u.reflags = flags;
     }
 
     void setNumber(double n, DecimalPoint decimalPoint) {
-        JS_ASSERT(type == TOK_NUMBER);
+        MOZ_ASSERT(type == TOK_NUMBER);
         u.number.value = n;
         u.number.decimalPoint = decimalPoint;
     }
@@ -302,30 +142,30 @@ struct Token
     // Type-safe accessors
 
     PropertyName *name() const {
-        JS_ASSERT(type == TOK_NAME);
+        MOZ_ASSERT(type == TOK_NAME);
         return u.name->asPropertyName(); // poor-man's type verification
     }
 
     JSAtom *atom() const {
-        JS_ASSERT (type == TOK_STRING ||
+        MOZ_ASSERT(type == TOK_STRING ||
                    type == TOK_TEMPLATE_HEAD ||
                    type == TOK_NO_SUBS_TEMPLATE);
         return u.atom;
     }
 
     js::RegExpFlag regExpFlags() const {
-        JS_ASSERT(type == TOK_REGEXP);
-        JS_ASSERT((u.reflags & AllFlags) == u.reflags);
+        MOZ_ASSERT(type == TOK_REGEXP);
+        MOZ_ASSERT((u.reflags & AllFlags) == u.reflags);
         return u.reflags;
     }
 
     double number() const {
-        JS_ASSERT(type == TOK_NUMBER);
+        MOZ_ASSERT(type == TOK_NUMBER);
         return u.number.value;
     }
 
     DecimalPoint decimalPoint() const {
-        JS_ASSERT(type == TOK_NUMBER);
+        MOZ_ASSERT(type == TOK_NUMBER);
         return u.number.decimalPoint;
     }
 };
@@ -334,11 +174,7 @@ struct CompileError {
     JSErrorReport report;
     char *message;
     ErrorArgumentsType argumentsType;
-    CompileError()
-      : message(nullptr), argumentsType(ArgumentsAreUnicode)
-    {
-        mozilla::PodZero(&report);
-    }
+    CompileError() : message(nullptr), argumentsType(ArgumentsAreUnicode) {}
     ~CompileError();
     void throwError(JSContext *cx);
 
@@ -363,7 +199,7 @@ class StrictModeGetter {
 
 // TokenStream is the lexical scanner for Javascript source text.
 //
-// It takes a buffer of jschars and linearly scans it into |Token|s.
+// It takes a buffer of char16_t characters and linearly scans it into |Token|s.
 // Internally the class uses a four element circular buffer |tokens| of
 // |Token|s. As an index for |tokens|, the member |cursor| points to the
 // current token.
@@ -415,13 +251,15 @@ class MOZ_STACK_CLASS TokenStream
     static const unsigned ntokensMask = ntokens - 1;
 
   public:
-    typedef Vector<jschar, 32> CharBuffer;
+    typedef Vector<char16_t, 32> CharBuffer;
 
     TokenStream(ExclusiveContext *cx, const ReadOnlyCompileOptions &options,
-                const jschar *base, size_t length, StrictModeGetter *smg);
+                const char16_t *base, size_t length, StrictModeGetter *smg);
 
     ~TokenStream();
 
+    bool checkOptions();
+
     // Accessors.
     const Token &currentToken() const { return tokens[cursor]; }
     bool isCurrentTokenType(TokenKind type) const {
@@ -430,15 +268,15 @@ class MOZ_STACK_CLASS TokenStream
     const CharBuffer &getTokenbuf() const { return tokenbuf; }
     const char *getFilename() const { return filename; }
     unsigned getLineno() const { return lineno; }
-    unsigned getColumn() const { return userbuf.addressOfNextRawChar() - linebase - 1; }
-    JSPrincipals *getOriginPrincipals() const { return originPrincipals; }
+    unsigned getColumn() const { return userbuf.offset() - linebase - 1; }
+    bool getMutedErrors() const { return mutedErrors; }
     JSVersion versionNumber() const { return VersionNumber(options().version); }
     JSVersion versionWithFlags() const { return options().version; }
 
     PropertyName *currentName() const {
         if (isCurrentTokenType(TOK_YIELD))
             return cx->names().yield;
-        JS_ASSERT(isCurrentTokenType(TOK_NAME));
+        MOZ_ASSERT(isCurrentTokenType(TOK_NAME));
         return currentToken().name();
     }
 
@@ -453,6 +291,7 @@ class MOZ_STACK_CLASS TokenStream
 
     // TokenStream-specific error reporters.
     bool reportError(unsigned errorNumber, ...);
+    bool reportErrorNoOffset(unsigned errorNumber, ...);
     bool reportWarning(unsigned errorNumber, ...);
 
     static const uint32_t NoOffset = UINT32_MAX;
@@ -470,22 +309,50 @@ class MOZ_STACK_CLASS TokenStream
     // asm.js reporter
     void reportAsmJSError(uint32_t offset, unsigned errorNumber, ...);
 
+    JSAtom *getRawTemplateStringAtom() {
+        MOZ_ASSERT(currentToken().type == TOK_TEMPLATE_HEAD ||
+                   currentToken().type == TOK_NO_SUBS_TEMPLATE);
+        const char16_t *cur = userbuf.rawCharPtrAt(currentToken().pos.begin + 1);
+        const char16_t *end;
+        if (currentToken().type == TOK_TEMPLATE_HEAD) {
+            // Of the form    |`...${|   or   |}...${|
+            end = userbuf.rawCharPtrAt(currentToken().pos.end - 2);
+        } else {
+            // NO_SUBS_TEMPLATE is of the form   |`...`|   or   |}...`|
+            end = userbuf.rawCharPtrAt(currentToken().pos.end - 1);
+        }
+
+        CharBuffer charbuf(cx);
+        while (cur < end) {
+            int32_t ch = *cur;
+            if (ch == '\r') {
+                ch = '\n';
+                if ((cur + 1 < end) && (*(cur + 1) == '\n'))
+                    cur++;
+            }
+            if (!charbuf.append(ch))
+                return nullptr;
+            cur++;
+        }
+        return AtomizeChars(cx, charbuf.begin(), charbuf.length());
+    }
+
   private:
     // These are private because they should only be called by the tokenizer
     // while tokenizing not by, for example, BytecodeEmitter.
     bool reportStrictModeError(unsigned errorNumber, ...);
     bool strictMode() const { return strictModeGetter && strictModeGetter->strictMode(); }
 
-    void onError();
     static JSAtom *atomize(ExclusiveContext *cx, CharBuffer &cb);
-    bool putIdentInTokenbuf(const jschar *identStart);
+    bool putIdentInTokenbuf(const char16_t *identStart);
 
     struct Flags
     {
         bool isEOF:1;           // Hit end of file.
         bool isDirtyLine:1;     // Non-whitespace since start of line.
         bool sawOctalEscape:1;  // Saw an octal character escape.
-        bool hadError:1;        // Returned TOK_ERROR from getToken.
+        bool hadError:1;        // Hit a syntax error, at start or during a
+                                // token.
 
         Flags()
           : isEOF(), isDirtyLine(), sawOctalEscape(), hadError()
@@ -505,60 +372,76 @@ class MOZ_STACK_CLASS TokenStream
         TemplateTail,   // Treat next characters as part of a template string
     };
 
-    // Get the next token from the stream, make it the current token, and
-    // return its kind.
-    TokenKind getToken(Modifier modifier = None) {
+    // Advance to the next token.  If the token stream encountered an error,
+    // return false.  Otherwise return true and store the token kind in |*ttp|.
+    bool getToken(TokenKind *ttp, Modifier modifier = None) {
         // Check for a pushed-back token resulting from mismatching lookahead.
         if (lookahead != 0) {
+            MOZ_ASSERT(!flags.hadError);
             lookahead--;
             cursor = (cursor + 1) & ntokensMask;
             TokenKind tt = currentToken().type;
-            JS_ASSERT(tt != TOK_EOL);
-            return tt;
+            MOZ_ASSERT(tt != TOK_EOL);
+            *ttp = tt;
+            return true;
         }
 
-        return getTokenInternal(modifier);
+        return getTokenInternal(ttp, modifier);
     }
 
     // Push the last scanned token back into the stream.
     void ungetToken() {
-        JS_ASSERT(lookahead < maxLookahead);
+        MOZ_ASSERT(lookahead < maxLookahead);
         lookahead++;
         cursor = (cursor - 1) & ntokensMask;
     }
 
-    TokenKind peekToken(Modifier modifier = None) {
-        if (lookahead != 0)
-            return tokens[(cursor + 1) & ntokensMask].type;
-        TokenKind tt = getTokenInternal(modifier);
+    bool peekToken(TokenKind *ttp, Modifier modifier = None) {
+        if (lookahead > 0) {
+            MOZ_ASSERT(!flags.hadError);
+            *ttp = tokens[(cursor + 1) & ntokensMask].type;
+            return true;
+        }
+        if (!getTokenInternal(ttp, modifier))
+            return false;
         ungetToken();
-        return tt;
+        return true;
     }
 
-    TokenPos peekTokenPos(Modifier modifier = None) {
-        if (lookahead != 0)
-            return tokens[(cursor + 1) & ntokensMask].pos;
-        getTokenInternal(modifier);
-        ungetToken();
-        JS_ASSERT(lookahead != 0);
-        return tokens[(cursor + 1) & ntokensMask].pos;
+    bool peekTokenPos(TokenPos *posp, Modifier modifier = None) {
+        if (lookahead == 0) {
+            TokenKind tt;
+            if (!getTokenInternal(&tt, modifier))
+                return false;
+            ungetToken();
+            MOZ_ASSERT(lookahead != 0);
+        } else {
+            MOZ_ASSERT(!flags.hadError);
+        }
+        *posp = tokens[(cursor + 1) & ntokensMask].pos;
+        return true;
     }
 
     // This is like peekToken(), with one exception:  if there is an EOL
     // between the end of the current token and the start of the next token, it
-    // returns TOK_EOL.  In that case, no token with TOK_EOL is actually
-    // created, just a TOK_EOL TokenKind is returned, and currentToken()
-    // shouldn't be consulted.  (This is the only place TOK_EOL is produced.)
-    MOZ_ALWAYS_INLINE TokenKind peekTokenSameLine(Modifier modifier = None) {
-       const Token &curr = currentToken();
+    // return true and store TOK_EOL in |*ttp|.  In that case, no token with
+    // TOK_EOL is actually created, just a TOK_EOL TokenKind is returned, and
+    // currentToken() shouldn't be consulted.  (This is the only place TOK_EOL
+    // is produced.)
+    MOZ_ALWAYS_INLINE bool
+    peekTokenSameLine(TokenKind *ttp, Modifier modifier = None) {
+        const Token &curr = currentToken();
 
         // If lookahead != 0, we have scanned ahead at least one token, and
         // |lineno| is the line that the furthest-scanned token ends on.  If
         // it's the same as the line that the current token ends on, that's a
         // stronger condition than what we are looking for, and we don't need
         // to return TOK_EOL.
-        if (lookahead != 0 && srcCoords.isOnThisLine(curr.pos.end, lineno))
-            return tokens[(cursor + 1) & ntokensMask].type;
+        if (lookahead != 0 && srcCoords.isOnThisLine(curr.pos.end, lineno)) {
+            MOZ_ASSERT(!flags.hadError);
+            *ttp = tokens[(cursor + 1) & ntokensMask].type;
+            return true;
+        }
 
         // The above check misses two cases where we don't have to return
         // TOK_EOL.
@@ -567,35 +450,58 @@ class MOZ_STACK_CLASS TokenStream
         //   is a newline between the next token and the one after that.
         // The following test is somewhat expensive but gets these cases (and
         // all others) right.
-        (void)getToken(modifier);
+        TokenKind tmp;
+        if (!getToken(&tmp, modifier))
+            return false;
         const Token &next = currentToken();
         ungetToken();
-        return srcCoords.lineNum(curr.pos.end) == srcCoords.lineNum(next.pos.begin)
-               ? next.type
-               : TOK_EOL;
+
+        *ttp = srcCoords.lineNum(curr.pos.end) == srcCoords.lineNum(next.pos.begin)
+             ? next.type
+             : TOK_EOL;
+        return true;
     }
 
     // Get the next token from the stream if its kind is |tt|.
-    bool matchToken(TokenKind tt, Modifier modifier = None) {
-        if (getToken(modifier) == tt)
-            return true;
-        ungetToken();
-        return false;
+    bool matchToken(bool *matchedp, TokenKind tt, Modifier modifier = None) {
+        TokenKind token;
+        if (!getToken(&token, modifier))
+            return false;
+        if (token == tt) {
+            *matchedp = true;
+        } else {
+            ungetToken();
+            *matchedp = false;
+        }
+        return true;
     }
 
     void consumeKnownToken(TokenKind tt) {
-        JS_ALWAYS_TRUE(matchToken(tt));
+        bool matched;
+        MOZ_ASSERT(lookahead != 0);
+        MOZ_ALWAYS_TRUE(matchToken(&matched, tt));
+        MOZ_ALWAYS_TRUE(matched);
     }
 
-    bool matchContextualKeyword(Handle<PropertyName*> keyword) {
-        if (getToken() == TOK_NAME && currentToken().name() == keyword)
-            return true;
-        ungetToken();
-        return false;
+    bool matchContextualKeyword(bool *matchedp, Handle<PropertyName*> keyword) {
+        TokenKind token;
+        if (!getToken(&token))
+            return false;
+        if (token == TOK_NAME && currentToken().name() == keyword) {
+            *matchedp = true;
+        } else {
+            *matchedp = false;
+            ungetToken();
+        }
+        return true;
     }
 
-    bool nextTokenEndsExpr() {
-        return isExprEnding[peekToken()];
+    bool nextTokenEndsExpr(bool *endsExpr) {
+        TokenKind tt;
+        if (!peekToken(&tt))
+            return false;
+        *endsExpr = isExprEnding[tt];
+        return true;
     }
 
     class MOZ_STACK_CLASS Position {
@@ -611,11 +517,11 @@ class MOZ_STACK_CLASS TokenStream
       private:
         Position(const Position&) MOZ_DELETE;
         friend class TokenStream;
-        const jschar *buf;
+        const char16_t *buf;
         Flags flags;
         unsigned lineno;
-        const jschar *linebase;
-        const jschar *prevLinebase;
+        size_t linebase;
+        size_t prevLinebase;
         Token currentToken;
         unsigned lookahead;
         Token lookaheadTokens[maxLookahead];
@@ -626,15 +532,11 @@ class MOZ_STACK_CLASS TokenStream
     void seek(const Position &pos);
     bool seek(const Position &pos, const TokenStream &other);
 
-    size_t positionToOffset(const Position &pos) const {
-        return pos.buf - userbuf.base();
+    const char16_t *rawCharPtrAt(size_t offset) const {
+        return userbuf.rawCharPtrAt(offset);
     }
 
-    const jschar *rawBase() const {
-        return userbuf.base();
-    }
-
-    const jschar *rawLimit() const {
+    const char16_t *rawLimit() const {
         return userbuf.limit();
     }
 
@@ -642,7 +544,7 @@ class MOZ_STACK_CLASS TokenStream
         return displayURL_ != nullptr;
     }
 
-    jschar *displayURL() {
+    char16_t *displayURL() {
         return displayURL_.get();
     }
 
@@ -650,7 +552,7 @@ class MOZ_STACK_CLASS TokenStream
         return sourceMapURL_ != nullptr;
     }
 
-    jschar *sourceMapURL() {
+    char16_t *sourceMapURL() {
         return sourceMapURL_.get();
     }
 
@@ -665,7 +567,7 @@ class MOZ_STACK_CLASS TokenStream
     // false. If ttp is non-null, return true with the keyword's TokenKind in
     // *ttp.
     bool checkForKeyword(const KeywordInfo *kw, TokenKind *ttp);
-    bool checkForKeyword(const jschar *s, size_t length, TokenKind *ttp);
+    bool checkForKeyword(const char16_t *s, size_t length, TokenKind *ttp);
     bool checkForKeyword(JSAtom *atom, TokenKind *ttp);
 
     // This class maps a userbuf offset (which is 0-indexed) to a line number
@@ -726,7 +628,7 @@ class MOZ_STACK_CLASS TokenStream
 
         bool isOnThisLine(uint32_t offset, uint32_t lineNum) const {
             uint32_t lineIndex = lineNumToIndex(lineNum);
-            JS_ASSERT(lineIndex + 1 < lineStartOffsets_.length());  // +1 due to sentinel
+            MOZ_ASSERT(lineIndex + 1 < lineStartOffsets_.length());  // +1 due to sentinel
             return lineStartOffsets_[lineIndex] <= offset &&
                    offset < lineStartOffsets_[lineIndex + 1];
         }
@@ -756,10 +658,18 @@ class MOZ_STACK_CLASS TokenStream
     // and do some extra stuff like converting all EOL sequences to '\n',
     // tracking the line number, and setting |flags.isEOF|.  (The "raw" in "raw
     // chars" refers to the lack of EOL sequence normalization.)
+    //
+    // buf[0..length-1] often represents a substring of some larger source,
+    // where we have only the substring in memory. The |startOffset| argument
+    // indicates the offset within this larger string at which our string
+    // begins, the offset of |buf[0]|.
     class TokenBuf {
       public:
-        TokenBuf(ExclusiveContext *cx, const jschar *buf, size_t length)
-          : base_(buf), limit_(buf + length), ptr(buf)
+        TokenBuf(ExclusiveContext *cx, const char16_t *buf, size_t length, size_t startOffset)
+          : base_(buf),
+            startOffset_(startOffset),
+            limit_(buf + length),
+            ptr(buf)
         { }
 
         bool hasRawChars() const {
@@ -767,26 +677,36 @@ class MOZ_STACK_CLASS TokenStream
         }
 
         bool atStart() const {
-            return ptr == base_;
+            return offset() == 0;
         }
 
-        const jschar *base() const {
-            return base_;
+        size_t startOffset() const {
+            return startOffset_;
         }
 
-        const jschar *limit() const {
+        size_t offset() const {
+            return startOffset_ + mozilla::PointerRangeSize(base_, ptr);
+        }
+
+        const char16_t *rawCharPtrAt(size_t offset) const {
+            MOZ_ASSERT(startOffset_ <= offset);
+            MOZ_ASSERT(offset - startOffset_ <= mozilla::PointerRangeSize(base_, limit_));
+            return base_ + (offset - startOffset_);
+        }
+
+        const char16_t *limit() const {
             return limit_;
         }
 
-        jschar getRawChar() {
+        char16_t getRawChar() {
             return *ptr++;      // this will nullptr-crash if poisoned
         }
 
-        jschar peekRawChar() const {
+        char16_t peekRawChar() const {
             return *ptr;        // this will nullptr-crash if poisoned
         }
 
-        bool matchRawChar(jschar c) {
+        bool matchRawChar(char16_t c) {
             if (*ptr == c) {    // this will nullptr-crash if poisoned
                 ptr++;
                 return true;
@@ -794,8 +714,8 @@ class MOZ_STACK_CLASS TokenStream
             return false;
         }
 
-        bool matchRawCharBackwards(jschar c) {
-            JS_ASSERT(ptr);     // make sure it hasn't been poisoned
+        bool matchRawCharBackwards(char16_t c) {
+            MOZ_ASSERT(ptr);     // make sure it hasn't been poisoned
             if (*(ptr - 1) == c) {
                 ptr--;
                 return true;
@@ -804,18 +724,18 @@ class MOZ_STACK_CLASS TokenStream
         }
 
         void ungetRawChar() {
-            JS_ASSERT(ptr);     // make sure it hasn't been poisoned
+            MOZ_ASSERT(ptr);     // make sure it hasn't been poisoned
             ptr--;
         }
 
-        const jschar *addressOfNextRawChar(bool allowPoisoned = false) const {
-            JS_ASSERT_IF(!allowPoisoned, ptr);     // make sure it hasn't been poisoned
+        const char16_t *addressOfNextRawChar(bool allowPoisoned = false) const {
+            MOZ_ASSERT_IF(!allowPoisoned, ptr);     // make sure it hasn't been poisoned
             return ptr;
         }
 
         // Use this with caution!
-        void setAddressOfNextRawChar(const jschar *a, bool allowPoisoned = false) {
-            JS_ASSERT_IF(!allowPoisoned, a);
+        void setAddressOfNextRawChar(const char16_t *a, bool allowPoisoned = false) {
+            MOZ_ASSERT_IF(!allowPoisoned, a);
             ptr = a;
         }
 
@@ -830,17 +750,18 @@ class MOZ_STACK_CLASS TokenStream
             return c == '\n' || c == '\r' || c == LINE_SEPARATOR || c == PARA_SEPARATOR;
         }
 
-        // Finds the next EOL, but stops once 'max' jschars have been scanned
-        // (*including* the starting jschar).
-        const jschar *findEOLMax(const jschar *p, size_t max);
+        // Returns the offset of the next EOL, but stops once 'max' characters
+        // have been scanned (*including* the char at startOffset_).
+        size_t findEOLMax(size_t start, size_t max);
 
       private:
-        const jschar *base_;            // base of buffer
-        const jschar *limit_;           // limit for quick bounds check
-        const jschar *ptr;              // next char to get
+        const char16_t *base_;          // base of buffer
+        uint32_t startOffset_;          // offset of base_[0]
+        const char16_t *limit_;         // limit for quick bounds check
+        const char16_t *ptr;            // next char to get
     };
 
-    TokenKind getTokenInternal(Modifier modifier);
+    bool getTokenInternal(TokenKind *ttp, Modifier modifier);
 
     bool getStringOrTemplateToken(int qc, Token **tp);
 
@@ -852,13 +773,13 @@ class MOZ_STACK_CLASS TokenStream
     bool peekUnicodeEscape(int32_t *c);
     bool matchUnicodeEscapeIdStart(int32_t *c);
     bool matchUnicodeEscapeIdent(int32_t *c);
-    bool peekChars(int n, jschar *cp);
+    bool peekChars(int n, char16_t *cp);
 
     bool getDirectives(bool isMultiline, bool shouldWarnDeprecated);
     bool getDirective(bool isMultiline, bool shouldWarnDeprecated,
                       const char *directive, int directiveLength,
                       const char *errorMsgPragma,
-                      mozilla::UniquePtr<jschar[], JS::FreePolicy> *destination);
+                      mozilla::UniquePtr<char16_t[], JS::FreePolicy> *destination);
     bool getDisplayURL(bool isMultiline, bool shouldWarnDeprecated);
     bool getSourceMappingURL(bool isMultiline, bool shouldWarnDeprecated);
 
@@ -871,7 +792,7 @@ class MOZ_STACK_CLASS TokenStream
 
     void consumeKnownChar(int32_t expect) {
         mozilla::DebugOnly<int32_t> c = getChar();
-        JS_ASSERT(c == expect);
+        MOZ_ASSERT(c == expect);
     }
 
     int32_t peekChar() {
@@ -896,25 +817,28 @@ class MOZ_STACK_CLASS TokenStream
     unsigned            lookahead;          // count of lookahead tokens
     unsigned            lineno;             // current line number
     Flags               flags;              // flags -- see above
-    const jschar        *linebase;          // start of current line;  points into userbuf
-    const jschar        *prevLinebase;      // start of previous line;  nullptr if on the first line
+    size_t              linebase;           // start of current line
+    size_t              prevLinebase;       // start of previous line;  size_t(-1) if on the first line
     TokenBuf            userbuf;            // user input buffer
     const char          *filename;          // input filename or null
-    mozilla::UniquePtr<jschar[], JS::FreePolicy> displayURL_; // the user's requested source URL or null
-    mozilla::UniquePtr<jschar[], JS::FreePolicy> sourceMapURL_; // source map's filename or null
+    mozilla::UniquePtr<char16_t[], JS::FreePolicy> displayURL_; // the user's requested source URL or null
+    mozilla::UniquePtr<char16_t[], JS::FreePolicy> sourceMapURL_; // source map's filename or null
     CharBuffer          tokenbuf;           // current token string buffer
     bool                maybeEOL[256];      // probabilistic EOL lookup table
     bool                maybeStrSpecial[256];   // speeds up string scanning
     uint8_t             isExprEnding[TOK_LIMIT];// which tokens definitely terminate exprs?
     ExclusiveContext    *const cx;
-    JSPrincipals        *const originPrincipals;
+    bool                mutedErrors;
     StrictModeGetter    *strictModeGetter;  // used to test for strict mode
 };
 
 // Steal one JSREPORT_* bit (see jsapi.h) to tell that arguments to the error
-// message have const jschar* type, not const char*.
+// message have const char16_t* type, not const char*.
 #define JSREPORT_UC 0x100
 
+extern const char *
+TokenKindToDesc(TokenKind tt);
+
 } // namespace frontend
 } // namespace js
 
