@@ -40,27 +40,87 @@ using namespace js::types;
 
 using mozilla::ArrayLength;
 using mozilla::PodArrayZero;
-using mozilla::PodZero;
 
 static void
 exn_finalize(FreeOp *fop, JSObject *obj);
 
-const Class ErrorObject::class_ = {
-    js_Error_str,
-    JSCLASS_IMPLEMENTS_BARRIERS |
-    JSCLASS_HAS_CACHED_PROTO(JSProto_Error) |
-    JSCLASS_HAS_RESERVED_SLOTS(ErrorObject::RESERVED_SLOTS),
-    JS_PropertyStub,         /* addProperty */
-    JS_DeletePropertyStub,   /* delProperty */
-    JS_PropertyStub,         /* getProperty */
-    JS_StrictPropertyStub,   /* setProperty */
-    JS_EnumerateStub,
-    JS_ResolveStub,
-    JS_ConvertStub,
-    exn_finalize,
-    nullptr,                 /* call        */
-    nullptr,                 /* hasInstance */
-    nullptr                  /* construct   */
+bool
+Error(JSContext *cx, unsigned argc, Value *vp);
+
+static bool
+exn_toSource(JSContext *cx, unsigned argc, Value *vp);
+
+static const JSFunctionSpec exception_methods[] = {
+#if JS_HAS_TOSOURCE
+    JS_FN(js_toSource_str, exn_toSource, 0, 0),
+#endif
+    JS_SELF_HOSTED_FN(js_toString_str, "ErrorToString", 0,0),
+    JS_FS_END
+};
+
+#define IMPLEMENT_ERROR_SUBCLASS(name) \
+    { \
+        js_Error_str, /* yes, really */ \
+        JSCLASS_IMPLEMENTS_BARRIERS | \
+        JSCLASS_HAS_CACHED_PROTO(JSProto_##name) | \
+        JSCLASS_HAS_RESERVED_SLOTS(ErrorObject::RESERVED_SLOTS), \
+        JS_PropertyStub,         /* addProperty */ \
+        JS_DeletePropertyStub,   /* delProperty */ \
+        JS_PropertyStub,         /* getProperty */ \
+        JS_StrictPropertyStub,   /* setProperty */ \
+        JS_EnumerateStub, \
+        JS_ResolveStub, \
+        JS_ConvertStub, \
+        exn_finalize, \
+        nullptr,                 /* call        */ \
+        nullptr,                 /* hasInstance */ \
+        nullptr,                 /* construct   */ \
+        nullptr,                 /* trace       */ \
+        { \
+            ErrorObject::createConstructor, \
+            ErrorObject::createProto, \
+            nullptr, \
+            exception_methods, \
+            nullptr, \
+            nullptr, \
+            JSProto_Error \
+        } \
+    }
+
+const Class
+ErrorObject::classes[JSEXN_LIMIT] = {
+    {
+        js_Error_str,
+        JSCLASS_IMPLEMENTS_BARRIERS |
+        JSCLASS_HAS_CACHED_PROTO(JSProto_Error) |
+        JSCLASS_HAS_RESERVED_SLOTS(ErrorObject::RESERVED_SLOTS),
+        JS_PropertyStub,         /* addProperty */
+        JS_DeletePropertyStub,   /* delProperty */
+        JS_PropertyStub,         /* getProperty */
+        JS_StrictPropertyStub,   /* setProperty */
+        JS_EnumerateStub,
+        JS_ResolveStub,
+        JS_ConvertStub,
+        exn_finalize,
+        nullptr,                 /* call        */
+        nullptr,                 /* hasInstance */
+        nullptr,                 /* construct   */
+        nullptr,                 /* trace       */
+        {
+            ErrorObject::createConstructor,
+            ErrorObject::createProto,
+            nullptr,
+            exception_methods,
+            0
+        }
+    },
+    IMPLEMENT_ERROR_SUBCLASS(InternalError),
+    IMPLEMENT_ERROR_SUBCLASS(EvalError),
+    IMPLEMENT_ERROR_SUBCLASS(RangeError),
+    IMPLEMENT_ERROR_SUBCLASS(ReferenceError),
+    IMPLEMENT_ERROR_SUBCLASS(SyntaxError),
+    IMPLEMENT_ERROR_SUBCLASS(TypeError),
+    IMPLEMENT_ERROR_SUBCLASS(URIError)
 };
 
 JSErrorReport *
@@ -71,16 +131,16 @@ js::CopyErrorReport(JSContext *cx, JSErrorReport *report)
      * the following layout:
      *   JSErrorReport
      *   array of copies of report->messageArgs
-     *   jschar array with characters for all messageArgs
-     *   jschar array with characters for ucmessage
-     *   jschar array with characters for uclinebuf and uctokenptr
+     *   char16_t array with characters for all messageArgs
+     *   char16_t array with characters for ucmessage
+     *   char16_t array with characters for uclinebuf and uctokenptr
      *   char array with characters for linebuf and tokenptr
      *   char array with characters for filename
      * Such layout together with the properties enforced by the following
      * asserts does not need any extra alignment padding.
      */
     JS_STATIC_ASSERT(sizeof(JSErrorReport) % sizeof(const char *) == 0);
-    JS_STATIC_ASSERT(sizeof(const char *) % sizeof(jschar) == 0);
+    JS_STATIC_ASSERT(sizeof(const char *) % sizeof(char16_t) == 0);
 
     size_t filenameSize;
     size_t linebufSize;
@@ -91,7 +151,7 @@ js::CopyErrorReport(JSContext *cx, JSErrorReport *report)
     JSErrorReport *copy;
     uint8_t *cursor;
 
-#define JS_CHARS_SIZE(jschars) ((js_strlen(jschars) + 1) * sizeof(jschar))
+#define JS_CHARS_SIZE(chars) ((js_strlen(chars) + 1) * sizeof(char16_t))
 
     filenameSize = report->filename ? strlen(report->filename) + 1 : 0;
     linebufSize = report->linebuf ? strlen(report->linebuf) + 1 : 0;
@@ -106,8 +166,8 @@ js::CopyErrorReport(JSContext *cx, JSErrorReport *report)
                 argsCopySize += JS_CHARS_SIZE(report->messageArgs[i]);
 
             /* Non-null messageArgs should have at least one non-null arg. */
-            JS_ASSERT(i != 0);
-            argsArraySize = (i + 1) * sizeof(const jschar *);
+            MOZ_ASSERT(i != 0);
+            argsArraySize = (i + 1) * sizeof(const char16_t *);
         }
     }
 
@@ -126,26 +186,26 @@ js::CopyErrorReport(JSContext *cx, JSErrorReport *report)
     cursor += sizeof(JSErrorReport);
 
     if (argsArraySize != 0) {
-        copy->messageArgs = (const jschar **)cursor;
+        copy->messageArgs = (const char16_t **)cursor;
         cursor += argsArraySize;
         for (i = 0; report->messageArgs[i]; ++i) {
-            copy->messageArgs[i] = (const jschar *)cursor;
+            copy->messageArgs[i] = (const char16_t *)cursor;
             argSize = JS_CHARS_SIZE(report->messageArgs[i]);
             js_memcpy(cursor, report->messageArgs[i], argSize);
             cursor += argSize;
         }
         copy->messageArgs[i] = nullptr;
-        JS_ASSERT(cursor == (uint8_t *)copy->messageArgs[0] + argsCopySize);
+        MOZ_ASSERT(cursor == (uint8_t *)copy->messageArgs[0] + argsCopySize);
     }
 
     if (report->ucmessage) {
-        copy->ucmessage = (const jschar *)cursor;
+        copy->ucmessage = (const char16_t *)cursor;
         js_memcpy(cursor, report->ucmessage, ucmessageSize);
         cursor += ucmessageSize;
     }
 
     if (report->uclinebuf) {
-        copy->uclinebuf = (const jschar *)cursor;
+        copy->uclinebuf = (const char16_t *)cursor;
         js_memcpy(cursor, report->uclinebuf, uclinebufSize);
         cursor += uclinebufSize;
         if (report->uctokenptr) {
@@ -168,12 +228,10 @@ js::CopyErrorReport(JSContext *cx, JSErrorReport *report)
         copy->filename = (const char *)cursor;
         js_memcpy(cursor, report->filename, filenameSize);
     }
-    JS_ASSERT(cursor + filenameSize == (uint8_t *)copy + mallocSize);
-
-    /* HOLD called by the destination error object. */
-    copy->originPrincipals = report->originPrincipals;
+    MOZ_ASSERT(cursor + filenameSize == (uint8_t *)copy + mallocSize);
 
     /* Copy non-pointer members. */
+    copy->isMuted = report->isMuted;
     copy->lineno = report->lineno;
     copy->column = report->column;
     copy->errorNumber = report->errorNumber;
@@ -194,13 +252,13 @@ struct SuppressErrorsGuard
 
     explicit SuppressErrorsGuard(JSContext *cx)
       : cx(cx),
-        prevReporter(JS_SetErrorReporter(cx, nullptr)),
+        prevReporter(JS_SetErrorReporter(cx->runtime(), nullptr)),
         prevState(cx)
     {}
 
     ~SuppressErrorsGuard()
     {
-        JS_SetErrorReporter(cx, prevReporter);
+        JS_SetErrorReporter(cx->runtime(), prevReporter);
     }
 };
 
@@ -230,11 +288,17 @@ js::ComputeStackString(JSContext *cx)
                 return nullptr;
 
             /* Now the filename. */
-            const char *cfilename = i.scriptFilename();
-            if (!cfilename)
-                cfilename = "";
-            if (!sb.appendInflated(cfilename, strlen(cfilename)))
-                return nullptr;
+
+            /* First, try the `//# sourceURL=some-display-url.js` directive. */
+            if (const char16_t *display = i.scriptDisplayURL()) {
+                if (!sb.append(display, js_strlen(display)))
+                    return nullptr;
+            }
+            /* Second, try the actual filename. */
+            else if (const char *filename = i.scriptFilename()) {
+                if (!sb.append(filename, strlen(filename)))
+                    return nullptr;
+            }
 
             uint32_t column = 0;
             uint32_t line = i.computeLine(&column);
@@ -266,12 +330,8 @@ js::ComputeStackString(JSContext *cx)
 static void
 exn_finalize(FreeOp *fop, JSObject *obj)
 {
-    if (JSErrorReport *report = obj->as<ErrorObject>().getErrorReport()) {
-        /* These were held by ErrorObject::init. */
-        if (JSPrincipals *prin = report->originPrincipals)
-            JS_DropPrincipals(fop->runtime(), prin);
+    if (JSErrorReport *report = obj->as<ErrorObject>().getErrorReport())
         fop->free_(report);
-    }
 }
 
 JSErrorReport *
@@ -290,7 +350,7 @@ js_ErrorFromException(JSContext *cx, HandleObject objArg)
     return obj->as<ErrorObject>().getOrCreateErrorReport(cx);
 }
 
-static bool
+bool
 Error(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
@@ -350,82 +410,6 @@ Error(JSContext *cx, unsigned argc, Value *vp)
     return true;
 }
 
-/* ES5 15.11.4.4 (NB: with subsequent errata). */
-static bool
-exn_toString(JSContext *cx, unsigned argc, Value *vp)
-{
-    JS_CHECK_RECURSION(cx, return false);
-    CallArgs args = CallArgsFromVp(argc, vp);
-
-    /* Step 2. */
-    if (!args.thisv().isObject()) {
-        JS_ReportErrorNumber(cx, js_GetErrorMessage, nullptr, JSMSG_BAD_PROTOTYPE, "Error");
-        return false;
-    }
-
-    /* Step 1. */
-    RootedObject obj(cx, &args.thisv().toObject());
-
-    /* Step 3. */
-    RootedValue nameVal(cx);
-    if (!JSObject::getProperty(cx, obj, obj, cx->names().name, &nameVal))
-        return false;
-
-    /* Step 4. */
-    RootedString name(cx);
-    if (nameVal.isUndefined()) {
-        name = cx->names().Error;
-    } else {
-        name = ToString<CanGC>(cx, nameVal);
-        if (!name)
-            return false;
-    }
-
-    /* Step 5. */
-    RootedValue msgVal(cx);
-    if (!JSObject::getProperty(cx, obj, obj, cx->names().message, &msgVal))
-        return false;
-
-    /* Step 6. */
-    RootedString message(cx);
-    if (msgVal.isUndefined()) {
-        message = cx->runtime()->emptyString;
-    } else {
-        message = ToString<CanGC>(cx, msgVal);
-        if (!message)
-            return false;
-    }
-
-    /* Step 7. */
-    if (name->empty() && message->empty()) {
-        args.rval().setString(cx->names().Error);
-        return true;
-    }
-
-    /* Step 8. */
-    if (name->empty()) {
-        args.rval().setString(message);
-        return true;
-    }
-
-    /* Step 9. */
-    if (message->empty()) {
-        args.rval().setString(name);
-        return true;
-    }
-
-    /* Step 10. */
-    StringBuffer sb(cx);
-    if (!sb.append(name) || !sb.append(": ") || !sb.append(message))
-        return false;
-
-    JSString *str = sb.finishString();
-    if (!str)
-        return false;
-    args.rval().setString(str);
-    return true;
-}
-
 #if JS_HAS_TOSOURCE
 /*
  * Return a string that may eval to something similar to the original object.
@@ -506,41 +490,21 @@ exn_toSource(JSContext *cx, unsigned argc, Value *vp)
 }
 #endif
 
-static const JSFunctionSpec exception_methods[] = {
-#if JS_HAS_TOSOURCE
-    JS_FN(js_toSource_str,   exn_toSource,           0,0),
-#endif
-    JS_FN(js_toString_str,   exn_toString,           0,0),
-    JS_FS_END
-};
-
-/* JSProto_ ordering for exceptions shall match JSEXN_ constants. */
-JS_STATIC_ASSERT(JSEXN_ERR == 0);
-JS_STATIC_ASSERT(JSProto_Error + JSEXN_INTERNALERR  == JSProto_InternalError);
-JS_STATIC_ASSERT(JSProto_Error + JSEXN_EVALERR      == JSProto_EvalError);
-JS_STATIC_ASSERT(JSProto_Error + JSEXN_RANGEERR     == JSProto_RangeError);
-JS_STATIC_ASSERT(JSProto_Error + JSEXN_REFERENCEERR == JSProto_ReferenceError);
-JS_STATIC_ASSERT(JSProto_Error + JSEXN_SYNTAXERR    == JSProto_SyntaxError);
-JS_STATIC_ASSERT(JSProto_Error + JSEXN_TYPEERR      == JSProto_TypeError);
-JS_STATIC_ASSERT(JSProto_Error + JSEXN_URIERR       == JSProto_URIError);
-
-/* static */ ErrorObject *
-ErrorObject::createProto(JSContext *cx, JS::Handle<GlobalObject*> global, JSExnType type,
-                         JS::HandleObject proto)
+/* static */ JSObject *
+ErrorObject::createProto(JSContext *cx, JSProtoKey key)
 {
-    RootedObject errorProto(cx);
-    errorProto = global->createBlankPrototypeInheriting(cx, &ErrorObject::class_, *proto);
+    RootedObject errorProto(cx, GenericCreatePrototype(cx, key));
     if (!errorProto)
         return nullptr;
 
     Rooted<ErrorObject*> err(cx, &errorProto->as<ErrorObject>());
     RootedString emptyStr(cx, cx->names().empty);
+    JSExnType type = ExnTypeFromProtoKey(key);
     if (!ErrorObject::init(cx, err, type, nullptr, emptyStr, emptyStr, 0, 0, emptyStr))
         return nullptr;
 
     // The various prototypes also have .name in addition to the normal error
     // instance properties.
-    JSProtoKey key = GetExceptionProtoKey(type);
     RootedPropertyName name(cx, ClassName(key, cx));
     RootedValue nameValue(cx, StringValue(name));
     if (!JSObject::defineProperty(cx, err, cx->names().name, nameValue,
@@ -549,76 +513,23 @@ ErrorObject::createProto(JSContext *cx, JS::Handle<GlobalObject*> global, JSExnT
         return nullptr;
     }
 
-    // Create the corresponding constructor.
-    RootedFunction ctor(cx, global->createConstructor(cx, Error, name, 1,
-                                                      JSFunction::ExtendedFinalizeKind));
-    if (!ctor)
-        return nullptr;
-    ctor->setExtendedSlot(0, Int32Value(int32_t(type)));
-
-    if (!LinkConstructorAndPrototype(cx, ctor, err))
-        return nullptr;
-
-    if (!GlobalObject::initBuiltinConstructor(cx, global, key, ctor, err))
-        return nullptr;
-
-    return err;
-}
-
-JSObject *
-js_InitExceptionClasses(JSContext *cx, HandleObject obj)
-{
-    JS_ASSERT(obj->is<GlobalObject>());
-    JS_ASSERT(obj->isNative());
-
-    Rooted<GlobalObject*> global(cx, &obj->as<GlobalObject>());
-
-    RootedObject objProto(cx, global->getOrCreateObjectPrototype(cx));
-    if (!objProto)
-        return nullptr;
-
-    /* Initialize the base Error class first. */
-    RootedObject errorProto(cx, ErrorObject::createProto(cx, global, JSEXN_ERR, objProto));
-    if (!errorProto)
-        return nullptr;
-
-    /* |Error.prototype| alone has method properties. */
-    if (!DefinePropertiesAndBrand(cx, errorProto, nullptr, exception_methods))
-        return nullptr;
-
-    /* Define all remaining *Error constructors. */
-    for (int i = JSEXN_ERR + 1; i < JSEXN_LIMIT; i++) {
-        if (!ErrorObject::createProto(cx, global, JSExnType(i), errorProto))
-            return nullptr;
-    }
-
     return errorProto;
 }
 
-const JSErrorFormatString*
-js_GetLocalizedErrorMessage(ExclusiveContext *cx, void *userRef, const char *locale,
-                            const unsigned errorNumber)
+/* static */ JSObject *
+ErrorObject::createConstructor(JSContext *cx, JSProtoKey key)
 {
-    const JSErrorFormatString *errorString = nullptr;
+    RootedObject ctor(cx);
+    ctor = GenericCreateConstructor<Error, 1, JSFunction::ExtendedFinalizeKind>(cx, key);
+    if (!ctor)
+        return nullptr;
 
-    // The locale callbacks might not be thread safe, so don't call them if
-    // we're not on the main thread. When used with XPConnect,
-    // |localeGetErrorMessage| will be nullptr anyways.
-    if (cx->isJSContext() &&
-        cx->asJSContext()->runtime()->localeCallbacks &&
-        cx->asJSContext()->runtime()->localeCallbacks->localeGetErrorMessage)
-    {
-        JSLocaleCallbacks *callbacks = cx->asJSContext()->runtime()->localeCallbacks;
-        errorString = callbacks->localeGetErrorMessage(userRef, locale, errorNumber);
-    }
-
-    if (!errorString)
-        errorString = js_GetErrorMessage(userRef, locale, errorNumber);
-    return errorString;
+    ctor->as<JSFunction>().setExtendedSlot(0, Int32Value(ExnTypeFromProtoKey(key)));
+    return ctor;
 }
 
-JS_FRIEND_API(const jschar*)
-js::GetErrorTypeName(JSRuntime* rt, int16_t exnType)
+JS_FRIEND_API(JSFlatString *)
+js::GetErrorTypeName(JSRuntime *rt, int16_t exnType)
 {
     /*
      * JSEXN_INTERNALERR returns null to prevent that "InternalError: "
@@ -630,7 +541,7 @@ js::GetErrorTypeName(JSRuntime* rt, int16_t exnType)
         return nullptr;
     }
     JSProtoKey key = GetExceptionProtoKey(JSExnType(exnType));
-    return ClassName(key, rt)->chars();
+    return ClassName(key, rt);
 }
 
 bool
@@ -638,17 +549,15 @@ js_ErrorToException(JSContext *cx, const char *message, JSErrorReport *reportp,
                     JSErrorCallback callback, void *userRef)
 {
     // Tell our caller to report immediately if this report is just a warning.
-    JS_ASSERT(reportp);
+    MOZ_ASSERT(reportp);
     if (JSREPORT_IS_WARNING(reportp->flags))
         return false;
 
     // Find the exception index associated with this error.
     JSErrNum errorNumber = static_cast<JSErrNum>(reportp->errorNumber);
-    const JSErrorFormatString *errorString;
-    if (!callback || callback == js_GetErrorMessage)
-        errorString = js_GetLocalizedErrorMessage(cx, nullptr, nullptr, errorNumber);
-    else
-        errorString = callback(userRef, nullptr, errorNumber);
+    if (!callback)
+        callback = js_GetErrorMessage;
+    const JSErrorFormatString *errorString = callback(userRef, errorNumber);
     JSExnType exnType = errorString ? static_cast<JSExnType>(errorString->exnType) : JSEXN_NONE;
     MOZ_ASSERT(exnType < JSEXN_LIMIT);
 
@@ -748,33 +657,74 @@ js_ReportUncaughtException(JSContext *cx)
     if (!cx->getPendingException(&exn))
         return false;
 
-    /*
-     * Because ToString below could error and an exception object could become
-     * unrooted, we must root exnObject.  Later, if exnObject is non-null, we
-     * need to root other intermediates, so allocate an operand stack segment
-     * to protect all of these values.
-     */
-    RootedObject exnObject(cx);
-    if (exn.isPrimitive()) {
-        exnObject = nullptr;
-    } else {
-        exnObject = exn.toObjectOrNull();
+    cx->clearPendingException();
+
+    ErrorReport err(cx);
+    if (!err.init(cx, exn)) {
+        cx->clearPendingException();
+        return false;
     }
 
-    JS_ClearPendingException(cx);
-    JSErrorReport *reportp = exnObject ? js_ErrorFromException(cx, exnObject)
-                                       : nullptr;
+    cx->setPendingException(exn);
+    CallErrorReporter(cx, err.message(), err.report());
+    cx->clearPendingException();
+    return true;
+}
+
+ErrorReport::ErrorReport(JSContext *cx)
+  : reportp(nullptr),
+    message_(nullptr),
+    ownedMessage(nullptr),
+    str(cx),
+    strChars(cx),
+    exnObject(cx)
+{
+}
+
+ErrorReport::~ErrorReport()
+{
+    if (!ownedMessage)
+        return;
+
+    js_free(ownedMessage);
+    if (ownedReport.messageArgs) {
+        /*
+         * js_ExpandErrorArguments owns its messageArgs only if it had to
+         * inflate the arguments (from regular |char *|s), which is always in
+         * our case.
+         */
+        size_t i = 0;
+        while (ownedReport.messageArgs[i])
+            js_free(const_cast<char16_t*>(ownedReport.messageArgs[i++]));
+        js_free(ownedReport.messageArgs);
+    }
+    js_free(const_cast<char16_t*>(ownedReport.ucmessage));
+}
+
+bool
+ErrorReport::init(JSContext *cx, HandleValue exn)
+{
+    MOZ_ASSERT(!cx->isExceptionPending());
+
+    /*
+     * Because ToString below could error and an exception object could become
+     * unrooted, we must root our exception object, if any.
+     */
+    if (exn.isObject()) {
+        exnObject = &exn.toObject();
+        reportp = js_ErrorFromException(cx, exnObject);
+    }
 
     // Be careful not to invoke ToString if we've already successfully extracted
     // an error report, since the exception might be wrapped in a security
     // wrapper, and ToString-ing it might throw.
-    RootedString str(cx);
     if (reportp)
         str = ErrorReportToString(cx, reportp);
     else
         str = ToString<CanGC>(cx, exn);
 
-    JSErrorReport report;
+    if (!str)
+        cx->clearPendingException();
 
     // If js_ErrorFromException didn't get us a JSErrorReport, then the object
     // was not an ErrorObject, security-wrapped or otherwise. However, it might
@@ -785,7 +735,6 @@ js_ReportUncaughtException(JSContext *cx)
     // on their proto chain, and hence also have a "fileName" property, but its
     // value is "".
     const char *filename_str = "filename";
-    JSAutoByteString filename;
     if (!reportp && exnObject && IsDuckTypedErrorObject(cx, exnObject, &filename_str))
     {
         // Temporary value for pulling properties off of duck-typed objects.
@@ -794,10 +743,14 @@ js_ReportUncaughtException(JSContext *cx)
         RootedString name(cx);
         if (JS_GetProperty(cx, exnObject, js_name_str, &val) && val.isString())
             name = val.toString();
+        else
+            cx->clearPendingException();
 
         RootedString msg(cx);
         if (JS_GetProperty(cx, exnObject, js_message_str, &val) && val.isString())
             msg = val.toString();
+        else
+            cx->clearPendingException();
 
         // If we have the right fields, override the ToString we performed on
         // the exception object above with something built out of its quacks
@@ -826,12 +779,17 @@ js_ReportUncaughtException(JSContext *cx)
             JSString *tmp = ToString<CanGC>(cx, val);
             if (tmp)
                 filename.encodeLatin1(cx, tmp);
+            else
+                cx->clearPendingException();
+        } else {
+            cx->clearPendingException();
         }
 
         uint32_t lineno;
         if (!JS_GetProperty(cx, exnObject, js_lineNumber_str, &val) ||
             !ToUint32(cx, val, &lineno))
         {
+            cx->clearPendingException();
             lineno = 0;
         }
 
@@ -839,15 +797,16 @@ js_ReportUncaughtException(JSContext *cx)
         if (!JS_GetProperty(cx, exnObject, js_columnNumber_str, &val) ||
             !ToUint32(cx, val, &column))
         {
+            cx->clearPendingException();
             column = 0;
         }
 
-        reportp = &report;
-        PodZero(&report);
-        report.filename = filename.ptr();
-        report.lineno = (unsigned) lineno;
-        report.exnType = int16_t(JSEXN_NONE);
-        report.column = (unsigned) column;
+        reportp = &ownedReport;
+        new (reportp) JSErrorReport();
+        ownedReport.filename = filename.ptr();
+        ownedReport.lineno = lineno;
+        ownedReport.exnType = int16_t(JSEXN_NONE);
+        ownedReport.column = column;
         if (str) {
             // Note that using |str| for |ucmessage| here is kind of wrong,
             // because |str| is supposed to be of the format
@@ -856,39 +815,72 @@ js_ReportUncaughtException(JSContext *cx)
             // done for duck-typed error objects.
             //
             // If only this stuff could get specced one day...
-            if (JSFlatString *flat = str->ensureFlat(cx))
-                report.ucmessage = flat->chars();
+            if (str->ensureFlat(cx) && strChars.initTwoByte(cx, str))
+                ownedReport.ucmessage = strChars.twoByteChars();
         }
     }
 
-    JSAutoByteString bytesStorage;
-    const char *bytes = nullptr;
     if (str)
-        bytes = bytesStorage.encodeLatin1(cx, str);
-    if (!bytes)
-        bytes = "unknown (can't convert to string)";
+        message_ = bytesStorage.encodeLatin1(cx, str);
+    if (!message_)
+        message_ = "unknown (can't convert to string)";
 
     if (!reportp) {
-        JS_ReportErrorNumber(cx, js_GetErrorMessage, nullptr,
-                             JSMSG_UNCAUGHT_EXCEPTION, bytes);
+        // This is basically an inlined version of
+        //
+        //   JS_ReportErrorNumber(cx, js_GetErrorMessage, nullptr,
+        //                        JSMSG_UNCAUGHT_EXCEPTION, message_);
+        //
+        // but without the reporting bits.  Instead it just puts all
+        // the stuff we care about in our ownedReport and message_.
+        populateUncaughtExceptionReport(cx, message_);
     } else {
         /* Flag the error as an exception. */
         reportp->flags |= JSREPORT_EXCEPTION;
-
-        /* Pass the exception object. */
-        JS_SetPendingException(cx, exn);
-        CallErrorReporter(cx, bytes, reportp);
     }
 
-    JS_ClearPendingException(cx);
     return true;
 }
 
-JSObject *
-js_CopyErrorObject(JSContext *cx, Handle<ErrorObject*> err, HandleObject scope)
+void
+ErrorReport::populateUncaughtExceptionReport(JSContext *cx, ...)
 {
-    assertSameCompartment(cx, scope);
+    va_list ap;
+    va_start(ap, cx);
+    populateUncaughtExceptionReportVA(cx, ap);
+    va_end(ap);
+}
 
+void
+ErrorReport::populateUncaughtExceptionReportVA(JSContext *cx, va_list ap)
+{
+    new (&ownedReport) JSErrorReport();
+    ownedReport.flags = JSREPORT_ERROR;
+    ownedReport.errorNumber = JSMSG_UNCAUGHT_EXCEPTION;
+    // XXXbz this assumes the stack we have right now is still
+    // related to our exception object.  It would be better if we
+    // could accept a passed-in stack of some sort instead.
+    NonBuiltinFrameIter iter(cx);
+    if (!iter.done()) {
+        ownedReport.filename = iter.scriptFilename();
+        ownedReport.lineno = iter.computeLine(&ownedReport.column);
+        ownedReport.isMuted = iter.mutedErrors();
+    }
+
+    if (!js_ExpandErrorArguments(cx, js_GetErrorMessage, nullptr,
+                                 JSMSG_UNCAUGHT_EXCEPTION, &ownedMessage,
+                                 &ownedReport, ArgumentsAreASCII, ap)) {
+        return;
+    }
+
+    reportp = &ownedReport;
+    message_ = ownedMessage;
+    ownsMessageAndReport = true;
+}
+
+JSObject *
+js_CopyErrorObject(JSContext *cx, Handle<ErrorObject*> err)
+{
     js::ScopedJSFreePtr<JSErrorReport> copyReport;
     if (JSErrorReport *errorReport = err->getErrorReport()) {
         copyReport = CopyErrorReport(cx, errorReport);
@@ -897,13 +889,13 @@ js_CopyErrorObject(JSContext *cx, Handle<ErrorObject*> err, HandleObject scope)
     }
 
     RootedString message(cx, err->getMessage());
-    if (message && !cx->compartment()->wrap(cx, message.address()))
+    if (message && !cx->compartment()->wrap(cx, &message))
         return nullptr;
     RootedString fileName(cx, err->fileName(cx));
-    if (!cx->compartment()->wrap(cx, fileName.address()))
+    if (!cx->compartment()->wrap(cx, &fileName))
         return nullptr;
     RootedString stack(cx, err->stack(cx));
-    if (!cx->compartment()->wrap(cx, stack.address()))
+    if (!cx->compartment()->wrap(cx, &stack))
         return nullptr;
     uint32_t lineNumber = err->lineNumber();
     uint32_t columnNumber = err->columnNumber();
@@ -915,7 +907,7 @@ js_CopyErrorObject(JSContext *cx, Handle<ErrorObject*> err, HandleObject scope)
 }
 
 JS_PUBLIC_API(bool)
-JS::CreateTypeError(JSContext *cx, HandleString stack, HandleString fileName,
+JS::CreateError(JSContext *cx, JSExnType type, HandleString stack, HandleString fileName,
                     uint32_t lineNumber, uint32_t columnNumber, JSErrorReport *report,
                     HandleString message, MutableHandleValue rval)
 {
@@ -925,7 +917,7 @@ JS::CreateTypeError(JSContext *cx, HandleString stack, HandleString fileName,
         rep = CopyErrorReport(cx, report);
 
     RootedObject obj(cx,
-        js::ErrorObject::create(cx, JSEXN_TYPEERR, stack, fileName,
+        js::ErrorObject::create(cx, type, stack, fileName,
                                 lineNumber, columnNumber, &rep, message));
     if (!obj)
         return false;
