@@ -565,8 +565,8 @@ js::math_log(JSContext *cx, unsigned argc, Value *vp)
     return true;
 }
 
-static double
-max_double(double x, double y)
+double
+js::math_max_impl(double x, double y)
 {
     // Math.max(num, NaN) => NaN, Math.max(-0, +0) => +0
     if (x > y || IsNaN(x) || (x == y && IsNegative(y)))
@@ -584,14 +584,14 @@ js::math_max(JSContext *cx, unsigned argc, Value *vp)
         double x;
         if (!ToNumber(cx, args[i], &x))
             return false;
-        maxval = max_double(x, maxval);
+        maxval = math_max_impl(x, maxval);
     }
     args.rval().setNumber(maxval);
     return true;
 }
 
-static double
-min_double(double x, double y)
+double
+js::math_min_impl(double x, double y)
 {
     // Math.min(num, NaN) => NaN, Math.min(-0, +0) => -0
     if (x < y || IsNaN(x) || (x == y && IsNegativeZero(x)))
@@ -609,7 +609,7 @@ js::math_min(JSContext *cx, unsigned argc, Value *vp)
         double x;
         if (!ToNumber(cx, args[i], &x))
             return false;
-        minval = min_double(x, minval);
+        minval = math_min_impl(x, minval);
     }
     args.rval().setNumber(minval);
     return true;
@@ -626,9 +626,9 @@ js::minmax_impl(JSContext *cx, bool max, HandleValue a, HandleValue b, MutableHa
         return false;
 
     if (max)
-        res.setNumber(max_double(x, y));
+        res.setNumber(math_max_impl(x, y));
     else
-        res.setNumber(min_double(x, y));
+        res.setNumber(math_min_impl(x, y));
 
     return true;
 }
@@ -1377,7 +1377,12 @@ bool
 js::math_hypot(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
+    return math_hypot_handle(cx, args, args.rval());
+}
 
+bool
+js::math_hypot_handle(JSContext *cx, HandleValueArray args, MutableHandleValue res)
+{
     // IonMonkey calls the system hypot function directly if two arguments are
     // given. Do that here as well to get the same results.
     if (args.length() == 2) {
@@ -1388,7 +1393,7 @@ js::math_hypot(JSContext *cx, unsigned argc, Value *vp)
             return false;
 
         double result = ecmaHypot(x, y);
-        args.rval().setNumber(result);
+        res.setNumber(result);
         return true;
     }
 
@@ -1419,7 +1424,7 @@ js::math_hypot(JSContext *cx, unsigned argc, Value *vp)
     double result = isInfinite ? PositiveInfinity<double>() :
                     isNaN ? GenericNaN() :
                     scale * sqrt(sumsq);
-    args.rval().setNumber(result);
+    res.setNumber(result);
     return true;
 }
 
@@ -1568,7 +1573,7 @@ js_InitMathClass(JSContext *cx, HandleObject obj)
         return nullptr;
 
     if (!JS_DefineProperty(cx, obj, js_Math_str, Math, 0,
-                           JS_PropertyStub, JS_StrictPropertyStub))
+                           JS_STUBGETTER, JS_STUBSETTER))
     {
         return nullptr;
     }
