@@ -1,59 +1,28 @@
 /* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
- * vim: set ts=8 sw=4 et tw=99:
- *
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is JavaScript heap tools.
- *
- * The Initial Developer of the Original Code is
- * Mozilla Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2010
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Jim Blandy <jimb@mozilla.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * vim: set ts=8 sts=4 et sw=4 tw=99:
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
+
+#include "shell/jsheaptools.h"
+
+#include "mozilla/Move.h"
 
 #include <string.h>
 
-#include "jsapi.h"
-
 #include "jsalloc.h"
+#include "jsapi.h"
 #include "jscntxt.h"
 #include "jscompartment.h"
-#include "jsfun.h"
 #include "jsobj.h"
 #include "jsprf.h"
-#include "jsutil.h"
 
 #include "jsobjinlines.h"
 
 using namespace js;
 
+using mozilla::Move;
+
 #ifdef DEBUG
 
 
@@ -78,16 +47,18 @@ using namespace js;
  */
 
 /*
- * A JSTracer that produces a map of the heap with edges reversed. 
+ * A JSTracer that produces a map of the heap with edges reversed.
  *
- * HeapReversers must be allocated in a stack frame. (They contain an AutoArrayRooter,
- * and those must be allocated and destroyed in a stack-like order.)
+ * HeapReversers must be allocated in a stack frame. (They are derived from
+ * CustomAutoRooter, and those must be allocated and destroyed in a stack-like
+ * order.)
  *
  * HeapReversers keep all the roots they find in their traversal alive until
  * they are destroyed. So you don't need to worry about nodes going away while
  * you're using them.
  */
-class HeapReverser : public JSTracer {
+class HeapReverser : public JSTracer, public JS::CustomAutoRooter
+{
   public:
     struct Edge;
 
@@ -95,7 +66,7 @@ class HeapReverser : public JSTracer {
     class Node {
       public:
         Node() { }
-        Node(JSGCTraceKind kind)
+        explicit Node(JSGCTraceKind kind)
           : kind(kind), incoming(), marked(false) { }
 
         /*
@@ -103,14 +74,20 @@ class HeapReverser : public JSTracer {
          * incoming edge Vector in the hash table: Vectors support moves, but
          * not assignments or copy construction.
          */
-        Node(MoveRef<Node> rhs)
-          : kind(rhs->kind), incoming(Move(rhs->incoming)), marked(rhs->marked) { }
-        Node &operator=(MoveRef<Node> rhs) {
+        Node(Node &&rhs)
+          : kind(rhs.kind), incoming(Move(rhs.incoming)), marked(rhs.marked) { }
+        Node &operator=(Node &&rhs) {
+            MOZ_ASSERT(this != &rhs, "self-move assignment is prohibited");
             this->~Node();
-            new(this) Node(rhs);
+            new(this) Node(Move(rhs));
             return *this;
         }
 
+        void trace(JSTracer *trc) {
+            for (Edge *e = incoming.begin(); e != incoming.end(); e++)
+                e->trace(trc);
+        }
+
         /* What kind of Cell this is. */
         JSGCTraceKind kind;
 
@@ -125,36 +102,42 @@ class HeapReverser : public JSTracer {
         bool marked;
 
       private:
-        Node(const Node &);
-        Node &operator=(const Node &);
+        Node(const Node &) MOZ_DELETE;
+        Node &operator=(const Node &) MOZ_DELETE;
     };
 
     /* Metadata for a heap edge we have traversed. */
     struct Edge {
       public:
         Edge(char *name, void *origin) : name(name), origin(origin) { }
-        ~Edge() { free(name); }
+        ~Edge() { js_free(name); }
 
         /*
          * Move constructor and move assignment. These allow us to live in
          * Vectors without needing to copy our name string when the vector is
          * resized.
          */
-        Edge(MoveRef<Edge> rhs) : name(rhs->name), origin(rhs->origin) {
-            rhs->name = NULL;
+        Edge(Edge &&rhs) : name(rhs.name), origin(rhs.origin) {
+            rhs.name = nullptr;
         }
-        Edge &operator=(MoveRef<Edge> rhs) {
+        Edge &operator=(Edge &&rhs) {
+            MOZ_ASSERT(this != &rhs, "self-move assignment is prohibited");
             this->~Edge();
-            new(this) Edge(rhs);
+            new(this) Edge(Move(rhs));
             return *this;
         }
 
+        void trace(JSTracer *trc) {
+            if (origin)
+                gc::MarkGCThingRoot(trc, &origin, "HeapReverser::Edge");
+        }
+
         /* The name of this heap edge. Owned by this Edge. */
         char *name;
 
         /*
-         * The Cell from which this edge originates. NULL means a root. This is
-         * a cell address instead of a Node * because Nodes live in HashMap
+         * The Cell from which this edge originates. nullptr means a root. This
+         * is a cell address instead of a Node * because Nodes live in HashMap
          * table entries; if the HashMap reallocates its table, all pointers to
          * the Nodes it contains would become invalid. You should look up the
          * address here in |map| to find its Node.
@@ -166,12 +149,18 @@ class HeapReverser : public JSTracer {
      * The result of a reversal is a map from Cells' addresses to Node
      * structures describing their incoming edges.
      */
-    typedef HashMap<void *, Node> Map;
+    typedef HashMap<void *, Node, DefaultHasher<void *>, SystemAllocPolicy> Map;
     Map map;
 
     /* Construct a HeapReverser for |context|'s heap. */
-    HeapReverser(JSContext *cx) : map(cx), roots(cx), rooter(cx, 0, NULL), work(cx), parent(NULL) {
-        JS_TRACER_INIT(this, cx, traverseEdgeWithThis);
+    explicit HeapReverser(JSContext *cx)
+      : JSTracer(cx->runtime(), traverseEdgeWithThis),
+        JS::CustomAutoRooter(cx),
+        noggc(JS_GetRuntime(cx)),
+        nocgc(JS_GetRuntime(cx)),
+        runtime(JS_GetRuntime(cx)),
+        parent(nullptr)
+    {
     }
 
     bool init() { return map.init(); }
@@ -179,27 +168,17 @@ class HeapReverser : public JSTracer {
     /* Build a reversed map of the heap in |map|. */
     bool reverseHeap();
 
-  private:    
-    /*
-     * Conservative scanning can, on a whim, decide that a root is no longer a
-     * root, and cause bits of our graph to disappear. The 'roots' vector holds
-     * all the roots we find alive, and 'rooter' keeps them alive until we're
-     * destroyed.
-     *
-     * Note that AutoArrayRooters must be constructed and destroyed in a
-     * stack-like order, so the same rule applies to this HeapReverser. The
-     * easiest way to satisfy this requirement is to only allocate HeapReversers
-     * as local variables in functions, or in types that themselves follow that
-     * rule. This is kind of dumb, but JSAPI doesn't provide any less restricted
-     * way to register arrays of roots.
-     */
-    Vector<jsval> roots;
-    AutoArrayRooter rooter;
+  private:
+    JS::AutoDisableGenerationalGC noggc;
+    js::AutoDisableCompactingGC nocgc;
+
+    /* A runtime pointer for use by the destructor. */
+    JSRuntime *runtime;
 
     /*
      * Return the name of the most recent edge this JSTracer has traversed. The
      * result is allocated with malloc; if we run out of memory, raise an error
-     * in this HeapReverser's context and return NULL.
+     * in this HeapReverser's context and return nullptr.
      *
      * This may not be called after that edge's call to traverseEdge has
      * returned.
@@ -214,7 +193,7 @@ class HeapReverser : public JSTracer {
             reverser->parent = newParent;
         }
         ~AutoParent() {
-            reverser->parent = savedParent; 
+            reverser->parent = savedParent;
         }
       private:
         HeapReverser *reverser;
@@ -232,7 +211,7 @@ class HeapReverser : public JSTracer {
      * A stack of work items. We represent the stack explicitly to avoid
      * overflowing the C++ stack when traversing long chains of objects.
      */
-    Vector<Child> work; 
+    Vector<Child, 0, SystemAllocPolicy> work;
 
     /* When traverseEdge is called, the Cell and kind at which the edge originated. */
     void *parent;
@@ -249,31 +228,36 @@ class HeapReverser : public JSTracer {
     bool traversalStatus;
 
     /* Static member function wrapping 'traverseEdge'. */
-    static void traverseEdgeWithThis(JSTracer *tracer, void *cell, JSGCTraceKind kind) {
+    static void traverseEdgeWithThis(JSTracer *tracer, void **thingp, JSGCTraceKind kind) {
         HeapReverser *reverser = static_cast<HeapReverser *>(tracer);
-        reverser->traversalStatus = reverser->traverseEdge(cell, kind);
+        if (!reverser->traverseEdge(*thingp, kind))
+            reverser->traversalStatus = false;
     }
 
     /* Return a jsval representing a node, if possible; otherwise, return JSVAL_VOID. */
     jsval nodeToValue(void *cell, int kind) {
-        if (kind == JSTRACE_OBJECT) {
-            JSObject *object = static_cast<JSObject *>(cell);
-            return OBJECT_TO_JSVAL(object);
-        } else {
+        if (kind != JSTRACE_OBJECT)
             return JSVAL_VOID;
+        JSObject *object = static_cast<JSObject *>(cell);
+        return OBJECT_TO_JSVAL(object);
+    }
+
+    /* Keep all tracked objects live across GC. */
+    virtual void trace(JSTracer *trc) MOZ_OVERRIDE {
+        if (!map.initialized())
+            return;
+        for (Map::Enum e(map); !e.empty(); e.popFront()) {
+            gc::MarkGCThingRoot(trc, const_cast<void **>(&e.front().key()), "HeapReverser::map::key");
+            e.front().value().trace(trc);
         }
+        for (Child *c = work.begin(); c != work.end(); ++c)
+            gc::MarkGCThingRoot(trc, &c->cell, "HeapReverser::Child");
     }
 };
 
 bool
-HeapReverser::traverseEdge(void *cell, JSGCTraceKind kind) {
-    /* If this is a root, make our own root for it as well. */
-    if (!parent) {
-        if (!roots.append(nodeToValue(cell, kind)))
-            return false;
-        rooter.changeArray(roots.begin(), roots.length());
-    }
-
+HeapReverser::traverseEdge(void *cell, JSGCTraceKind kind)
+{
     /* Capture this edge before the JSTracer members get overwritten. */
     char *edgeDescription = getEdgeDescription();
     if (!edgeDescription)
@@ -298,11 +282,14 @@ HeapReverser::traverseEdge(void *cell, JSGCTraceKind kind) {
     }
 
     /* Add this edge to the reversed map. */
-    return a->value.incoming.append(Move(e));
+    return a->value().incoming.append(Move(e));
 }
 
 bool
-HeapReverser::reverseHeap() {
+HeapReverser::reverseHeap()
+{
+    traversalStatus = true;
+
     /* Prime the work stack with the roots of collection. */
     JS_TraceRuntime(this);
     if (!traversalStatus)
@@ -323,28 +310,28 @@ HeapReverser::reverseHeap() {
 char *
 HeapReverser::getEdgeDescription()
 {
-    if (!debugPrinter && debugPrintIndex == (size_t) -1) {
-        const char *arg = static_cast<const char *>(debugPrintArg);
-        char *name = static_cast<char *>(context->malloc_(strlen(arg) + 1));
+    if (!debugPrinter() && debugPrintIndex() == (size_t) -1) {
+        const char *arg = static_cast<const char *>(debugPrintArg());
+        char *name = js_pod_malloc<char>(strlen(arg) + 1);
         if (!name)
-            return NULL;
+            return nullptr;
         strcpy(name, arg);
         return name;
     }
 
     /* Lovely; but a fixed size is required by JSTraceNamePrinter. */
     static const int nameSize = 200;
-    char *name = static_cast<char *>(context->malloc_(nameSize));
+    char *name = js_pod_malloc<char>(nameSize);
     if (!name)
-        return NULL;
-    if (debugPrinter)
-        debugPrinter(this, name, nameSize);
+        return nullptr;
+    if (debugPrinter())
+        debugPrinter()(this, name, nameSize);
     else
         JS_snprintf(name, nameSize, "%s[%lu]",
-                    static_cast<const char *>(debugPrintArg), debugPrintIndex);
+                    static_cast<const char *>(debugPrintArg()), debugPrintIndex());
 
     /* Shrink storage to fit. */
-    return static_cast<char *>(context->realloc_(name, strlen(name) + 1));
+    return static_cast<char *>(js_realloc(name, strlen(name) + 1));
 }
 
 
@@ -353,11 +340,11 @@ HeapReverser::getEdgeDescription()
 /* A class for finding an object's referrers, given a reversed heap map. */
 class ReferenceFinder {
   public:
-    ReferenceFinder(JSContext *cx, const HeapReverser &reverser) 
-      : context(cx), reverser(reverser) { }
+    ReferenceFinder(JSContext *cx, const HeapReverser &reverser)
+      : context(cx), reverser(reverser), result(cx) { }
 
     /* Produce an object describing all references to |target|. */
-    JSObject *findReferences(JSObject *target);
+    JSObject *findReferences(HandleObject target);
 
   private:
     /* The context in which to do allocation and error-handling. */
@@ -367,13 +354,13 @@ class ReferenceFinder {
     const HeapReverser &reverser;
 
     /* The results object we're currently building. */
-    JSObject *result;
+    RootedObject result;
 
     /* A list of edges we've traversed to get to a certain point. */
     class Path {
       public:
         Path(const HeapReverser::Edge &edge, Path *next) : edge(edge), next(next) { }
-        
+
         /*
          * Compute the full path represented by this Path. The result is
          * owned by the caller.
@@ -386,13 +373,13 @@ class ReferenceFinder {
     };
 
     struct AutoNodeMarker {
-        AutoNodeMarker(HeapReverser::Node *node) : node(node) { node->marked = true; }
+        explicit AutoNodeMarker(HeapReverser::Node *node) : node(node) { node->marked = true; }
         ~AutoNodeMarker() { node->marked = false; }
       private:
         HeapReverser::Node *node;
     };
 
-    /* 
+    /*
      * Given that we've reached |cell| via |path|, with all Nodes along that
      * path marked, add paths from all reportable objects reachable from cell
      * to |result|.
@@ -408,10 +395,11 @@ class ReferenceFinder {
             JSObject *object = static_cast<JSObject *>(cell);
 
             /* Certain classes of object are for internal use only. */
-            if (object->isBlock() ||
-                object->isCall() ||
-                object->isWith() ||
-                object->isDeclEnv()) {
+            if (object->is<BlockObject>() ||
+                object->is<CallObject>() ||
+                object->is<StaticWithObject>() ||
+                object->is<DynamicWithObject>() ||
+                object->is<DeclEnvObject>()) {
                 return JSVAL_VOID;
             }
 
@@ -438,15 +426,15 @@ ReferenceFinder::visit(void *cell, Path *path)
     /* Have we reached a root? Always report that. */
     if (!cell)
         return addReferrer(JSVAL_NULL, path);
-        
+
     HeapReverser::Map::Ptr p = reverser.map.lookup(cell);
-    JS_ASSERT(p);
-    HeapReverser::Node *node = &p->value;
+    MOZ_ASSERT(p);
+    HeapReverser::Node *node = &p->value();
 
     /* Is |cell| a representable cell, reached via a non-empty path? */
-    if (path != NULL) {
+    if (path != nullptr) {
         jsval representation = representable(cell, node->kind);
-        if (!JSVAL_IS_VOID(representation))
+        if (!representation.isUndefined())
             return addReferrer(representation, path);
     }
 
@@ -475,13 +463,13 @@ ReferenceFinder::Path::computeName(JSContext *cx)
 {
     /* Walk the edge list and compute the total size of the path. */
     size_t size = 6;
-    for (Path *l = this; l; l = l->next) 
+    for (Path *l = this; l; l = l->next)
         size += strlen(l->edge.name) + (l->next ? 2 : 0);
     size += 1;
 
-    char *path = static_cast<char *>(cx->malloc_(size));
+    char *path = cx->pod_malloc<char>(size);
     if (!path)
-        return NULL;
+        return nullptr;
 
     /*
      * Walk the edge list again, and copy the edge names into place, with
@@ -499,71 +487,73 @@ ReferenceFinder::Path::computeName(JSContext *cx)
             next += 2;
         }
     }
-    JS_ASSERT(next + 1 == path + size);
+    MOZ_ASSERT(next + 1 == path + size);
 
     return path;
 }
 
 bool
-ReferenceFinder::addReferrer(jsval referrer, Path *path)
+ReferenceFinder::addReferrer(jsval referrerArg, Path *path)
 {
-    if (!context->compartment->wrap(context, &referrer))
-        return NULL;
+    RootedValue referrer(context, referrerArg);
 
-    char *pathName = path->computeName(context);
+    if (!context->compartment()->wrap(context, &referrer))
+        return false;
+
+    ScopedJSFreePtr<char> pathName(path->computeName(context));
     if (!pathName)
         return false;
-    AutoReleasePtr releasePathName(context, pathName);
 
     /* Find the property of the results object named |pathName|. */
-    jsval v;
+    RootedValue v(context);
+
     if (!JS_GetProperty(context, result, pathName, &v))
         return false;
-    if (JSVAL_IS_VOID(v)) {
+    if (v.isUndefined()) {
         /* Create an array to accumulate referents under this path. */
-        JSObject *array = JS_NewArrayObject(context, 1, &referrer);
+        JSObject *array = JS_NewArrayObject(context, HandleValueArray(referrer));
         if (!array)
             return false;
-        v = OBJECT_TO_JSVAL(array);
-        return !!JS_SetProperty(context, result, pathName, &v);
+        v.setObject(*array);
+        return !!JS_SetProperty(context, result, pathName, v);
     }
 
     /* The property's value had better be an array. */
-    JS_ASSERT(JSVAL_IS_OBJECT(v) && !JSVAL_IS_NULL(v));
-    JSObject *array = JSVAL_TO_OBJECT(v);
-    JS_ASSERT(JS_IsArrayObject(context, array));
+    RootedObject array(context, &v.toObject());
+    MOZ_ASSERT(JS_IsArrayObject(context, array));
 
     /* Append our referrer to this array. */
-    jsuint length;
+    uint32_t length;
     return JS_GetArrayLength(context, array, &length) &&
-           JS_SetElement(context, array, length, &referrer);
+           JS_SetElement(context, array, length, referrer);
 }
 
 JSObject *
-ReferenceFinder::findReferences(JSObject *target)
+ReferenceFinder::findReferences(HandleObject target)
 {
-    result = JS_NewObject(context, NULL, NULL, NULL);
+    result = JS_NewObject(context, nullptr, JS::NullPtr(), JS::NullPtr());
     if (!result)
-        return NULL;
-    if (!visit(target, NULL))
-        return NULL;
+        return nullptr;
+    if (!visit(target, nullptr))
+        return nullptr;
 
     return result;
 }
 
 /* See help(findReferences). */
-JSBool
-FindReferences(JSContext *cx, uintN argc, jsval *vp)
+bool
+FindReferences(JSContext *cx, unsigned argc, jsval *vp)
 {
-    if (argc < 1) {
-        JS_ReportErrorNumber(cx, js_GetErrorMessage, NULL, JSMSG_MORE_ARGS_NEEDED,
+    CallArgs args = CallArgsFromVp(argc, vp);
+    if (args.length() < 1) {
+        JS_ReportErrorNumber(cx, js_GetErrorMessage, nullptr, JSMSG_MORE_ARGS_NEEDED,
                              "findReferences", "0", "s");
         return false;
     }
 
-    jsval target = JS_ARGV(cx, vp)[0];
-    if (!JSVAL_IS_OBJECT(target) || JSVAL_IS_NULL(target)) {
-        JS_ReportErrorNumber(cx, js_GetErrorMessage, NULL, JSMSG_UNEXPECTED_TYPE,
+    RootedValue target(cx, args[0]);
+    if (!target.isObject()) {
+        JS_ReportErrorNumber(cx, js_GetErrorMessage, nullptr, JSMSG_UNEXPECTED_TYPE,
                              "argument", "not an object");
         return false;
     }
@@ -575,11 +565,12 @@ FindReferences(JSContext *cx, uintN argc, jsval *vp)
 
     /* Given the reversed map, find the referents of target. */
     ReferenceFinder finder(cx, reverser);
-    JSObject *references = finder.findReferences(JSVAL_TO_OBJECT(target));
+    Rooted<JSObject*> targetObj(cx, &target.toObject());
+    JSObject *references = finder.findReferences(targetObj);
     if (!references)
         return false;
-    
-    JS_SET_RVAL(cx, vp, OBJECT_TO_JSVAL(references));
+
+    args.rval().setObject(*references);
     return true;
 }
 
