@@ -1,11 +1,13 @@
 /* -*- indent-tabs-mode: nil; js-indent-level: 2 -*- */
-/* This Source Code Form is subject to the terms of the Mozilla Public
- * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
+/*
+ * Any copyright is dedicated to the Public Domain.
+ * http://creativecommons.org/licenses/publicdomain/
+ * Contributor: Jason Orendorff
+ */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 452498;
-var summary = 'TM: upvar2 regression tests';
+var BUGNUMBER = 469625;
+var summary = 'Do not assert: script->objectsOffset != 0';
 var actual = '';
 var expect = '';
 
@@ -19,22 +21,23 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
+ 
+  function f(x) {
+    var [a, b, [c0, c1]] = [x, x, x];
+  }
 
-// ------- Comment #92 From Jesse Ruderman
-
-  expect = 'TypeError: redeclaration of formal parameter e';
+  var ITERATOR = JS_HAS_SYMBOLS ? "Symbol.iterator" : "'@@iterator'";
+  expect = `TypeError: (intermediate value)[${ITERATOR}](...).next(...).value is null`;
+  actual = 'No Error';
   try
   {
-    eval('(function (e) { var e; const e; });');
+    f(null);
   }
   catch(ex)
   {
     actual = ex + '';
   }
-// Without patch: "TypeError: redeclaration of var e"
-// expected new behavior // With patch:    "TypeError: redeclaration of formal parameter e:"
-
   reportCompare(expect, actual, summary);
-
+  
   exitFunc ('test');
 }
