@@ -50,11 +50,21 @@
 
 using namespace js;
 
+using mozilla::BitwiseCast;
 using mozilla::IsNaN;
 using mozilla::LittleEndian;
 using mozilla::NativeEndian;
 using JS::CanonicalizeNaN;
 
+// When you make updates here, make sure you consider whether you need to bump the
+// value of JS_STRUCTURED_CLONE_VERSION in js/public/StructuredClone.h.  You will
+// likely need to increment the version if anything at all changes in the serialization
+// format.
+//
+// Note that SCTAG_END_OF_KEYS is written into the serialized form and should have
+// a stable ID, it need not be at the end of the list and should not be used for
+// sizing data structures.
+
 enum StructuredDataType MOZ_ENUM_TYPE(uint32_t) {
     /* Structured data types provided by the engine */
     SCTAG_FLOAT_MAX = 0xFFF00000,
@@ -78,6 +88,7 @@ enum StructuredDataType MOZ_ENUM_TYPE(uint32_t) {
     SCTAG_MAP_OBJECT,
     SCTAG_SET_OBJECT,
     SCTAG_END_OF_KEYS,
+    SCTAG_SHARED_TYPED_ARRAY_OBJECT,
 
     SCTAG_TYPED_ARRAY_V1_MIN = 0xFFFF0100,
     SCTAG_TYPED_ARRAY_V1_INT8 = SCTAG_TYPED_ARRAY_V1_MIN + Scalar::Int8,
@@ -141,7 +152,7 @@ struct SCOutput {
     bool writeDouble(double d);
     bool writeBytes(const void *p, size_t nbytes);
     bool writeChars(const Latin1Char *p, size_t nchars);
-    bool writeChars(const jschar *p, size_t nchars);
+    bool writeChars(const char16_t *p, size_t nchars);
     bool writePtr(const void *);
 
     template <class T>
@@ -172,7 +183,7 @@ class SCInput {
     bool readDouble(double *p);
     bool readBytes(void *p, size_t nbytes);
     bool readChars(Latin1Char *p, size_t nchars);
-    bool readChars(jschar *p, size_t nchars);
+    bool readChars(char16_t *p, size_t nchars);
     bool readPtr(void **);
 
     bool get(uint64_t *p);
@@ -192,9 +203,8 @@ class SCInput {
 
   private:
     void staticAssertions() {
-        JS_STATIC_ASSERT(sizeof(jschar) == 2);
+        JS_STATIC_ASSERT(sizeof(char16_t) == 2);
         JS_STATIC_ASSERT(sizeof(uint32_t) == 4);
-        JS_STATIC_ASSERT(sizeof(double) == 8);
     }
 
     JSContext *cx;
@@ -212,7 +222,7 @@ struct JSStructuredCloneReader {
           callbacks(cb), closure(cbClosure) { }
 
     SCInput &input() { return in; }
-    bool read(Value *vp);
+    bool read(MutableHandleValue vp);
 
   private:
     JSContext *context() { return in.context(); }
@@ -224,10 +234,12 @@ struct JSStructuredCloneReader {
     JSString *readString(uint32_t data);
 
     bool checkDouble(double d);
-    bool readTypedArray(uint32_t arrayType, uint32_t nelems, Value *vp, bool v1Read = false);
-    bool readArrayBuffer(uint32_t nbytes, Value *vp);
-    bool readV1ArrayBuffer(uint32_t arrayType, uint32_t nelems, Value *vp);
-    bool startRead(Value *vp);
+    bool readTypedArray(uint32_t arrayType, uint32_t nelems, MutableHandleValue vp,
+                        bool v1Read = false);
+    bool readSharedTypedArray(uint32_t arrayType, uint32_t nelems, MutableHandleValue vp);
+    bool readArrayBuffer(uint32_t nbytes, MutableHandleValue vp);
+    bool readV1ArrayBuffer(uint32_t arrayType, uint32_t nelems, MutableHandleValue vp);
+    bool startRead(MutableHandleValue vp);
 
     SCInput &in;
 
@@ -261,7 +273,7 @@ struct JSStructuredCloneWriter {
 
     bool init() { return memory.init() && parseTransferable() && writeTransferMap(); }
 
-    bool write(const Value &v);
+    bool write(HandleValue v);
 
     SCOutput &output() { return out; }
 
@@ -277,8 +289,10 @@ struct JSStructuredCloneWriter {
     bool writeString(uint32_t tag, JSString *str);
     bool writeArrayBuffer(HandleObject obj);
     bool writeTypedArray(HandleObject obj);
+    bool writeSharedArrayBuffer(HandleObject obj);
+    bool writeSharedTypedArray(HandleObject obj);
     bool startObject(HandleObject obj, bool *backref);
-    bool startWrite(const Value &v);
+    bool startWrite(HandleValue v);
     bool traverseObject(HandleObject obj);
     bool traverseMap(HandleObject obj);
     bool traverseSet(HandleObject obj);
@@ -328,7 +342,7 @@ struct JSStructuredCloneWriter {
 JS_FRIEND_API(uint64_t)
 js_GetSCOffset(JSStructuredCloneWriter* writer)
 {
-    JS_ASSERT(writer);
+    MOZ_ASSERT(writer);
     return writer->output().count() * sizeof(uint64_t);
 }
 
@@ -360,7 +374,7 @@ ReadStructuredClone(JSContext *cx, uint64_t *data, size_t nbytes, MutableHandleV
 {
     SCInput in(cx, data, nbytes);
     JSStructuredCloneReader r(in, cb, cbClosure);
-    return r.read(vp.address());
+    return r.read(vp);
 }
 
 // If the given buffer contains Transferables, free them. Note that custom
@@ -369,7 +383,7 @@ ReadStructuredClone(JSContext *cx, uint64_t *data, size_t nbytes, MutableHandleV
 static void
 Discard(uint64_t *buffer, size_t nbytes, const JSStructuredCloneCallbacks *cb, void *cbClosure)
 {
-    JS_ASSERT(nbytes % sizeof(uint64_t) == 0);
+    MOZ_ASSERT(nbytes % sizeof(uint64_t) == 0);
     if (nbytes < sizeof(uint64_t))
         return; // Empty buffer
 
@@ -389,7 +403,7 @@ Discard(uint64_t *buffer, size_t nbytes, const JSStructuredCloneCallbacks *cb, v
     while (numTransferables--) {
         uint32_t ownership;
         SCInput::getPair(point++, &tag, &ownership);
-        JS_ASSERT(tag >= SCTAG_TRANSFER_MAP_PENDING_ENTRY);
+        MOZ_ASSERT(tag >= SCTAG_TRANSFER_MAP_PENDING_ENTRY);
 
         void *content;
         SCInput::getPtr(point++, &content);
@@ -445,8 +459,8 @@ SCInput::SCInput(JSContext *cx, uint64_t *data, size_t nbytes)
 {
     // On 32-bit, we sometimes construct an SCInput from an SCOutput buffer,
     // which is not guaranteed to be 8-byte aligned
-    JS_ASSERT((uintptr_t(data) & (sizeof(int) - 1)) == 0);
-    JS_ASSERT((nbytes & 7) == 0);
+    MOZ_ASSERT((uintptr_t(data) & (sizeof(int) - 1)) == 0);
+    MOZ_ASSERT((nbytes & 7) == 0);
 }
 
 bool
@@ -573,9 +587,9 @@ SCInput::readChars(Latin1Char *p, size_t nchars)
 }
 
 bool
-SCInput::readChars(jschar *p, size_t nchars)
+SCInput::readChars(char16_t *p, size_t nchars)
 {
-    JS_ASSERT(sizeof(jschar) == sizeof(uint16_t));
+    MOZ_ASSERT(sizeof(char16_t) == sizeof(uint16_t));
     return readArray((uint16_t *) p, nchars);
 }
 
@@ -620,38 +634,16 @@ SCOutput::writePair(uint32_t tag, uint32_t data)
     return write(PairToUInt64(tag, data));
 }
 
-static inline uint64_t
-ReinterpretDoubleAsUInt64(double d)
-{
-    union {
-        double d;
-        uint64_t u;
-    } pun;
-    pun.d = d;
-    return pun.u;
-}
-
-static inline double
-ReinterpretUInt64AsDouble(uint64_t u)
-{
-    union {
-        uint64_t u;
-        double d;
-    } pun;
-    pun.u = u;
-    return pun.d;
-}
-
 static inline double
 ReinterpretPairAsDouble(uint32_t tag, uint32_t data)
 {
-    return ReinterpretUInt64AsDouble(PairToUInt64(tag, data));
+    return BitwiseCast<double>(PairToUInt64(tag, data));
 }
 
 bool
 SCOutput::writeDouble(double d)
 {
-    return write(ReinterpretDoubleAsUInt64(CanonicalizeNaN(d)));
+    return write(BitwiseCast<uint64_t>(CanonicalizeNaN(d)));
 }
 
 template <typename T>
@@ -673,8 +665,8 @@ template <class T>
 bool
 SCOutput::writeArray(const T *p, size_t nelems)
 {
-    JS_ASSERT(8 % sizeof(T) == 0);
-    JS_ASSERT(sizeof(uint64_t) % sizeof(T) == 0);
+    MOZ_ASSERT(8 % sizeof(T) == 0);
+    MOZ_ASSERT(sizeof(uint64_t) % sizeof(T) == 0);
 
     if (nelems == 0)
         return true;
@@ -702,9 +694,11 @@ SCOutput::writeBytes(const void *p, size_t nbytes)
 }
 
 bool
-SCOutput::writeChars(const jschar *p, size_t nchars)
+SCOutput::writeChars(const char16_t *p, size_t nchars)
 {
-    JS_ASSERT(sizeof(jschar) == sizeof(uint16_t));
+    static_assert(sizeof(char16_t) == sizeof(uint16_t),
+                  "required so that treating char16_t[] memory as uint16_t[] "
+                  "memory is permissible");
     return writeArray((const uint16_t *) p, nchars);
 }
 
@@ -770,13 +764,7 @@ JSStructuredCloneWriter::parseTransferable()
 
         if (!v.isObject())
             return reportErrorTransferable();
-
-        RootedObject tObj(context(), CheckedUnwrap(&v.toObject()));
-
-        if (!tObj) {
-            JS_ReportErrorNumber(context(), js_GetErrorMessage, nullptr, JSMSG_UNWRAP_DENIED);
-            return false;
-        }
+        RootedObject tObj(context(), &v.toObject());
 
         // No duplicates allowed
         if (std::find(transferableObjects.begin(), transferableObjects.end(), tObj) != transferableObjects.end()) {
@@ -826,20 +814,20 @@ JSStructuredCloneWriter::checkStack()
     const size_t MAX = 10;
 
     size_t limit = Min(counts.length(), MAX);
-    JS_ASSERT(objs.length() == counts.length());
+    MOZ_ASSERT(objs.length() == counts.length());
     size_t total = 0;
     for (size_t i = 0; i < limit; i++) {
-        JS_ASSERT(total + counts[i] >= total);
+        MOZ_ASSERT(total + counts[i] >= total);
         total += counts[i];
     }
     if (counts.length() <= MAX)
-        JS_ASSERT(total == entries.length());
+        MOZ_ASSERT(total == entries.length());
     else
-        JS_ASSERT(total <= entries.length());
+        MOZ_ASSERT(total <= entries.length());
 
     size_t j = objs.length();
     for (size_t i = 0; i < limit; i++)
-        JS_ASSERT(memory.has(&objs[--j].toObject()));
+        MOZ_ASSERT(memory.has(&objs[--j].toObject()));
 #endif
 }
 
@@ -855,7 +843,8 @@ JSStructuredCloneWriter::checkStack()
 bool
 JSStructuredCloneWriter::writeTypedArray(HandleObject obj)
 {
-    Rooted<TypedArrayObject*> tarr(context(), &obj->as<TypedArrayObject>());
+    Rooted<TypedArrayObject*> tarr(context(), &CheckedUnwrap(obj)->as<TypedArrayObject>());
+    JSAutoCompartment ac(context(), tarr);
 
     if (!TypedArrayObject::ensureHasBuffer(context(), tarr))
         return false;
@@ -877,12 +866,40 @@ JSStructuredCloneWriter::writeTypedArray(HandleObject obj)
 bool
 JSStructuredCloneWriter::writeArrayBuffer(HandleObject obj)
 {
-    ArrayBufferObject &buffer = obj->as<ArrayBufferObject>();
+    ArrayBufferObject &buffer = CheckedUnwrap(obj)->as<ArrayBufferObject>();
+    JSAutoCompartment ac(context(), &buffer);
 
     return out.writePair(SCTAG_ARRAY_BUFFER_OBJECT, buffer.byteLength()) &&
            out.writeBytes(buffer.dataPointer(), buffer.byteLength());
 }
 
+bool
+JSStructuredCloneWriter::writeSharedArrayBuffer(HandleObject obj)
+{
+    JS_ReportErrorNumber(context(), js_GetErrorMessage, nullptr, JSMSG_SC_SHMEM_MUST_TRANSFER);
+    return false;
+}
+
+bool
+JSStructuredCloneWriter::writeSharedTypedArray(HandleObject obj)
+{
+    Rooted<SharedTypedArrayObject*> tarr(context(), &CheckedUnwrap(obj)->as<SharedTypedArrayObject>());
+    JSAutoCompartment ac(context(), tarr);
+
+    if (!out.writePair(SCTAG_SHARED_TYPED_ARRAY_OBJECT, tarr->length()))
+        return false;
+    uint64_t type = tarr->type();
+    if (!out.write(type))
+        return false;
+
+    // Write out the SharedArrayBuffer tag and contents.
+    RootedValue val(context(), SharedTypedArrayObject::bufferValue(tarr));
+    if (!startWrite(val))
+        return false;
+
+    return out.write(tarr->byteOffset());
+}
+
 bool
 JSStructuredCloneWriter::startObject(HandleObject obj, bool *backref)
 {
@@ -910,7 +927,7 @@ JSStructuredCloneWriter::traverseObject(HandleObject obj)
      * will come off the stack in forward order.
      */
     AutoIdVector properties(context());
-    if (!GetPropertyNames(context(), obj, JSITER_OWNONLY, &properties))
+    if (!GetPropertyKeys(context(), obj, JSITER_OWNONLY, &properties))
         return false;
 
     for (size_t i = properties.length(); i > 0; --i) {
@@ -927,14 +944,22 @@ JSStructuredCloneWriter::traverseObject(HandleObject obj)
     checkStack();
 
     /* Write the header for obj. */
-    return out.writePair(obj->is<ArrayObject>() ? SCTAG_ARRAY_OBJECT : SCTAG_OBJECT_OBJECT, 0);
+    return out.writePair(ObjectClassIs(obj, ESClass_Array, context()) ? SCTAG_ARRAY_OBJECT : SCTAG_OBJECT_OBJECT, 0);
 }
 
 bool
 JSStructuredCloneWriter::traverseMap(HandleObject obj)
 {
     AutoValueVector newEntries(context());
-    if (!MapObject::entries(context(), obj, &newEntries))
+    {
+        // If there is no wrapper, the compartment munging is a no-op.
+        RootedObject unwrapped(context(), CheckedUnwrap(obj));
+        MOZ_ASSERT(unwrapped);
+        JSAutoCompartment ac(context(), unwrapped);
+        if (!MapObject::getKeysAndValuesInterleaved(context(), unwrapped, &newEntries))
+            return false;
+    }
+    if (!context()->compartment()->wrap(context(), newEntries))
         return false;
 
     for (size_t i = newEntries.length(); i > 0; --i) {
@@ -956,7 +981,15 @@ bool
 JSStructuredCloneWriter::traverseSet(HandleObject obj)
 {
     AutoValueVector keys(context());
-    if (!SetObject::keys(context(), obj, &keys))
+    {
+        // If there is no wrapper, the compartment munging is a no-op.
+        RootedObject unwrapped(context(), CheckedUnwrap(obj));
+        MOZ_ASSERT(unwrapped);
+        JSAutoCompartment ac(context(), unwrapped);
+        if (!SetObject::keys(context(), unwrapped, &keys))
+            return false;
+    }
+    if (!context()->compartment()->wrap(context(), keys))
         return false;
 
     for (size_t i = keys.length(); i > 0; --i) {
@@ -974,19 +1007,8 @@ JSStructuredCloneWriter::traverseSet(HandleObject obj)
     return out.writePair(SCTAG_SET_OBJECT, 0);
 }
 
-static bool
-PrimitiveToObject(JSContext *cx, Value *vp)
-{
-    JSObject *obj = PrimitiveToObject(cx, *vp);
-    if (!obj)
-        return false;
-
-    vp->setObject(*obj);
-    return true;
-}
-
 bool
-JSStructuredCloneWriter::startWrite(const Value &v)
+JSStructuredCloneWriter::startWrite(HandleValue v)
 {
     assertSameCompartment(context(), v);
 
@@ -1005,45 +1027,53 @@ JSStructuredCloneWriter::startWrite(const Value &v)
     } else if (v.isObject()) {
         RootedObject obj(context(), &v.toObject());
 
-        // The object might be a security wrapper. See if we can clone what's
-        // behind it. If we can, unwrap the object.
-        obj = CheckedUnwrap(obj);
-        if (!obj) {
-            JS_ReportErrorNumber(context(), js_GetErrorMessage, nullptr, JSMSG_UNWRAP_DENIED);
-            return false;
-        }
-
-        AutoCompartment ac(context(), obj);
-
         bool backref;
         if (!startObject(obj, &backref))
             return false;
         if (backref)
             return true;
 
-        if (obj->is<RegExpObject>()) {
-            RegExpObject &reobj = obj->as<RegExpObject>();
-            return out.writePair(SCTAG_REGEXP_OBJECT, reobj.getFlags()) &&
-                   writeString(SCTAG_STRING, reobj.getSource());
-        } else if (obj->is<DateObject>()) {
-            double d = js_DateGetMsecSinceEpoch(obj);
-            return out.writePair(SCTAG_DATE_OBJECT, 0) && out.writeDouble(d);
-        } else if (obj->is<TypedArrayObject>()) {
+        if (ObjectClassIs(obj, ESClass_RegExp, context())) {
+            RegExpGuard re(context());
+            if (!RegExpToShared(context(), obj, &re))
+                return false;
+            return out.writePair(SCTAG_REGEXP_OBJECT, re->getFlags()) &&
+                   writeString(SCTAG_STRING, re->getSource());
+        } else if (ObjectClassIs(obj, ESClass_Date, context())) {
+            RootedValue unboxed(context());
+            if (!Unbox(context(), obj, &unboxed))
+                return false;
+            return out.writePair(SCTAG_DATE_OBJECT, 0) && out.writeDouble(unboxed.toNumber());
+        } else if (JS_IsTypedArrayObject(obj)) {
             return writeTypedArray(obj);
-        } else if (obj->is<ArrayBufferObject>() && obj->as<ArrayBufferObject>().hasData()) {
+        } else if (JS_IsArrayBufferObject(obj) && JS_ArrayBufferHasData(obj)) {
             return writeArrayBuffer(obj);
-        } else if (obj->is<JSObject>() || obj->is<ArrayObject>()) {
+        } else if (JS_IsSharedTypedArrayObject(obj)) {
+            return writeSharedTypedArray(obj);
+        } else if (JS_IsSharedArrayBufferObject(obj)) {
+            return writeSharedArrayBuffer(obj);
+        } else if (ObjectClassIs(obj, ESClass_Object, context())) {
             return traverseObject(obj);
-        } else if (obj->is<BooleanObject>()) {
-            return out.writePair(SCTAG_BOOLEAN_OBJECT, obj->as<BooleanObject>().unbox());
-        } else if (obj->is<NumberObject>()) {
-            return out.writePair(SCTAG_NUMBER_OBJECT, 0) &&
-                   out.writeDouble(obj->as<NumberObject>().unbox());
-        } else if (obj->is<StringObject>()) {
-            return writeString(SCTAG_STRING_OBJECT, obj->as<StringObject>().unbox());
-        } else if (obj->is<MapObject>()) {
+        } else if (ObjectClassIs(obj, ESClass_Array, context())) {
+            return traverseObject(obj);
+        } else if (ObjectClassIs(obj, ESClass_Boolean, context())) {
+            RootedValue unboxed(context());
+            if (!Unbox(context(), obj, &unboxed))
+                return false;
+            return out.writePair(SCTAG_BOOLEAN_OBJECT, unboxed.toBoolean());
+        } else if (ObjectClassIs(obj, ESClass_Number, context())) {
+            RootedValue unboxed(context());
+            if (!Unbox(context(), obj, &unboxed))
+                return false;
+            return out.writePair(SCTAG_NUMBER_OBJECT, 0) && out.writeDouble(unboxed.toNumber());
+        } else if (ObjectClassIs(obj, ESClass_String, context())) {
+            RootedValue unboxed(context());
+            if (!Unbox(context(), obj, &unboxed))
+                return false;
+            return writeString(SCTAG_STRING_OBJECT, unboxed.toString());
+        } else if (ObjectClassIs(obj, ESClass_Map, context())) {
             return traverseMap(obj);
-        } else if (obj->is<SetObject>()) {
+        } else if (ObjectClassIs(obj, ESClass_Set, context())) {
             return traverseSet(obj);
         }
 
@@ -1097,9 +1127,9 @@ JSStructuredCloneWriter::transferOwnership()
     // grabbing out pointers from the transferables and stuffing them into the
     // transfer map.
     uint64_t *point = out.rawBuffer();
-    JS_ASSERT(uint32_t(LittleEndian::readUint64(point) >> 32) == SCTAG_TRANSFER_MAP_HEADER);
+    MOZ_ASSERT(uint32_t(LittleEndian::readUint64(point) >> 32) == SCTAG_TRANSFER_MAP_HEADER);
     point++;
-    JS_ASSERT(LittleEndian::readUint64(point) == transferableObjects.length());
+    MOZ_ASSERT(LittleEndian::readUint64(point) == transferableObjects.length());
     point++;
 
     for (JS::AutoObjectVector::Range tr = transferableObjects.all(); !tr.empty(); tr.popFront()) {
@@ -1116,19 +1146,32 @@ JSStructuredCloneWriter::transferOwnership()
         MOZ_ASSERT(ownership == JS::SCTAG_TMO_UNFILLED);
 #endif
 
-        if (obj->is<ArrayBufferObject>()) {
-            size_t nbytes = obj->as<ArrayBufferObject>().byteLength();
-            content = JS_StealArrayBufferContents(context(), obj);
-            if (!content)
-                return false; // Destructor will clean up the already-transferred data
+        if (ObjectClassIs(obj, ESClass_ArrayBuffer, context())) {
+            // The current setup of the array buffer inheritance hierarchy doesn't
+            // lend itself well to generic manipulation via proxies.
+            Rooted<ArrayBufferObject *> arrayBuffer(context(), &CheckedUnwrap(obj)->as<ArrayBufferObject>());
+            size_t nbytes = arrayBuffer->byteLength();
+
+            // Structured cloning currently only has optimizations for mapped
+            // and malloc'd buffers, not asm.js-ified buffers.
+            bool hasStealableContents = arrayBuffer->hasStealableContents() &&
+                                        (arrayBuffer->isMapped() || arrayBuffer->hasMallocedContents());
+
+            ArrayBufferObject::BufferContents bufContents =
+                ArrayBufferObject::stealContents(context(), arrayBuffer, hasStealableContents);
+            if (!bufContents)
+                return false; // Destructor will clean up the already-transferred data.
+
+            content = bufContents.data();
             tag = SCTAG_TRANSFER_MAP_ARRAY_BUFFER;
-            if (obj->as<ArrayBufferObject>().isMappedArrayBuffer())
+            if (bufContents.kind() == ArrayBufferObject::MAPPED)
                 ownership = JS::SCTAG_TMO_MAPPED_DATA;
             else
                 ownership = JS::SCTAG_TMO_ALLOC_DATA;
             extraData = nbytes;
-        } else if (obj->is<SharedArrayBufferObject>()) {
-            SharedArrayRawBuffer *rawbuf = obj->as<SharedArrayBufferObject>().rawBufferObject();
+        } else if (ObjectClassIs(obj, ESClass_SharedArrayBuffer, context())) {
+            Rooted<SharedArrayBufferObject *> sharedArrayBuffer(context(), &CheckedUnwrap(obj)->as<SharedArrayBufferObject>());
+            SharedArrayRawBuffer *rawbuf = sharedArrayBuffer->rawBufferObject();
 
             // Avoids a race condition where the parent thread frees the buffer
             // before the child has accepted the transferable.
@@ -1143,7 +1186,7 @@ JSStructuredCloneWriter::transferOwnership()
                 return reportErrorTransferable();
             if (!callbacks->writeTransfer(context(), obj, closure, &tag, &ownership, &content, &extraData))
                 return false;
-            JS_ASSERT(tag > SCTAG_TRANSFER_MAP_PENDING_ENTRY);
+            MOZ_ASSERT(tag > SCTAG_TRANSFER_MAP_PENDING_ENTRY);
         }
 
         LittleEndian::writeUint64(point++, PairToUInt64(tag, ownership));
@@ -1151,15 +1194,15 @@ JSStructuredCloneWriter::transferOwnership()
         LittleEndian::writeUint64(point++, extraData);
     }
 
-    JS_ASSERT(point <= out.rawBuffer() + out.count());
-    JS_ASSERT_IF(point < out.rawBuffer() + out.count(),
-                 uint32_t(LittleEndian::readUint64(point) >> 32) < SCTAG_TRANSFER_MAP_HEADER);
+    MOZ_ASSERT(point <= out.rawBuffer() + out.count());
+    MOZ_ASSERT_IF(point < out.rawBuffer() + out.count(),
+                  uint32_t(LittleEndian::readUint64(point) >> 32) < SCTAG_TRANSFER_MAP_HEADER);
 
     return true;
 }
 
 bool
-JSStructuredCloneWriter::write(const Value &v)
+JSStructuredCloneWriter::write(HandleValue v)
 {
     if (!startWrite(v))
         return false;
@@ -1244,7 +1287,7 @@ class Chars {
     ~Chars() { js_free(p); }
 
     bool allocate(size_t len) {
-        JS_ASSERT(!p);
+        MOZ_ASSERT(!p);
         // We're going to null-terminate!
         p = cx->pod_malloc<CharT>(len + 1);
         if (p) {
@@ -1282,18 +1325,18 @@ JSStructuredCloneReader::readString(uint32_t data)
 {
     uint32_t nchars = data & JS_BITMASK(31);
     bool latin1 = data & (1 << 31);
-    return latin1 ? readStringImpl<Latin1Char>(nchars) : readStringImpl<jschar>(nchars);
+    return latin1 ? readStringImpl<Latin1Char>(nchars) : readStringImpl<char16_t>(nchars);
 }
 
 static uint32_t
 TagToV1ArrayType(uint32_t tag)
 {
-    JS_ASSERT(tag >= SCTAG_TYPED_ARRAY_V1_MIN && tag <= SCTAG_TYPED_ARRAY_V1_MAX);
+    MOZ_ASSERT(tag >= SCTAG_TYPED_ARRAY_V1_MIN && tag <= SCTAG_TYPED_ARRAY_V1_MAX);
     return tag - SCTAG_TYPED_ARRAY_V1_MIN;
 }
 
 bool
-JSStructuredCloneReader::readTypedArray(uint32_t arrayType, uint32_t nelems, Value *vp,
+JSStructuredCloneReader::readTypedArray(uint32_t arrayType, uint32_t nelems, MutableHandleValue vp,
                                         bool v1Read)
 {
     if (arrayType > Scalar::Uint8Clamped) {
@@ -1304,7 +1347,7 @@ JSStructuredCloneReader::readTypedArray(uint32_t arrayType, uint32_t nelems, Val
 
     // Push a placeholder onto the allObjs list to stand in for the typed array
     uint32_t placeholderIndex = allObjs.length();
-    Value dummy = JSVAL_NULL;
+    Value dummy = UndefinedValue();
     if (!allObjs.append(dummy))
         return false;
 
@@ -1312,11 +1355,11 @@ JSStructuredCloneReader::readTypedArray(uint32_t arrayType, uint32_t nelems, Val
     RootedValue v(context());
     uint32_t byteOffset;
     if (v1Read) {
-        if (!readV1ArrayBuffer(arrayType, nelems, v.address()))
+        if (!readV1ArrayBuffer(arrayType, nelems, &v))
             return false;
         byteOffset = 0;
     } else {
-        if (!startRead(v.address()))
+        if (!startRead(&v))
             return false;
         uint64_t n;
         if (!in.read(&n))
@@ -1360,22 +1403,90 @@ JSStructuredCloneReader::readTypedArray(uint32_t arrayType, uint32_t nelems, Val
 
     if (!obj)
         return false;
-    vp->setObject(*obj);
+    vp.setObject(*obj);
 
-    allObjs[placeholderIndex].set(*vp);
+    allObjs[placeholderIndex].set(vp);
 
     return true;
 }
 
 bool
-JSStructuredCloneReader::readArrayBuffer(uint32_t nbytes, Value *vp)
+JSStructuredCloneReader::readSharedTypedArray(uint32_t arrayType, uint32_t nelems, MutableHandleValue vp)
+{
+    if (arrayType > Scalar::Uint8Clamped) {
+        JS_ReportErrorNumber(context(), js_GetErrorMessage, nullptr,
+                             JSMSG_SC_BAD_SERIALIZED_DATA, "unhandled typed array element type");
+        return false;
+    }
+
+    // Push a placeholder onto the allObjs list to stand in for the typed array.
+    uint32_t placeholderIndex = allObjs.length();
+    Value dummy = UndefinedValue();
+    if (!allObjs.append(dummy))
+        return false;
+
+    // Read the ArrayBuffer object and its contents (but no properties).
+    RootedValue v(context());
+    uint32_t byteOffset;
+    if (!startRead(&v))
+        return false;
+    uint64_t n;
+    if (!in.read(&n))
+        return false;
+    byteOffset = n;
+    RootedObject buffer(context(), &v.toObject());
+    RootedObject obj(context());
+
+    switch (arrayType) {
+      case Scalar::Int8:
+        obj = JS_NewSharedInt8ArrayWithBuffer(context(), buffer, byteOffset, nelems);
+        break;
+      case Scalar::Uint8:
+        obj = JS_NewSharedUint8ArrayWithBuffer(context(), buffer, byteOffset, nelems);
+        break;
+      case Scalar::Int16:
+        obj = JS_NewSharedInt16ArrayWithBuffer(context(), buffer, byteOffset, nelems);
+        break;
+      case Scalar::Uint16:
+        obj = JS_NewSharedUint16ArrayWithBuffer(context(), buffer, byteOffset, nelems);
+        break;
+      case Scalar::Int32:
+        obj = JS_NewSharedInt32ArrayWithBuffer(context(), buffer, byteOffset, nelems);
+        break;
+      case Scalar::Uint32:
+        obj = JS_NewSharedUint32ArrayWithBuffer(context(), buffer, byteOffset, nelems);
+        break;
+      case Scalar::Float32:
+        obj = JS_NewSharedFloat32ArrayWithBuffer(context(), buffer, byteOffset, nelems);
+        break;
+      case Scalar::Float64:
+        obj = JS_NewSharedFloat64ArrayWithBuffer(context(), buffer, byteOffset, nelems);
+        break;
+      case Scalar::Uint8Clamped:
+        obj = JS_NewSharedUint8ClampedArrayWithBuffer(context(), buffer, byteOffset, nelems);
+        break;
+      default:
+        MOZ_CRASH("Can't happen: arrayType range checked above");
+    }
+
+    if (!obj)
+        return false;
+    vp.setObject(*obj);
+
+    allObjs[placeholderIndex].set(vp);
+
+    return true;
+}
+
+bool
+JSStructuredCloneReader::readArrayBuffer(uint32_t nbytes, MutableHandleValue vp)
 {
     JSObject *obj = ArrayBufferObject::create(context(), nbytes);
     if (!obj)
         return false;
-    vp->setObject(*obj);
+    vp.setObject(*obj);
     ArrayBufferObject &buffer = obj->as<ArrayBufferObject>();
-    JS_ASSERT(buffer.byteLength() == nbytes);
+    MOZ_ASSERT(buffer.byteLength() == nbytes);
     return in.readArray(buffer.dataPointer(), nbytes);
 }
 
@@ -1384,17 +1495,18 @@ JSStructuredCloneReader::readArrayBuffer(uint32_t nbytes, Value *vp)
  * endianness-conversion while reading.
  */
 bool
-JSStructuredCloneReader::readV1ArrayBuffer(uint32_t arrayType, uint32_t nelems, Value *vp)
+JSStructuredCloneReader::readV1ArrayBuffer(uint32_t arrayType, uint32_t nelems,
+                                           MutableHandleValue vp)
 {
-    JS_ASSERT(arrayType <= Scalar::Uint8Clamped);
+    MOZ_ASSERT(arrayType <= Scalar::Uint8Clamped);
 
     uint32_t nbytes = nelems << TypedArrayShift(static_cast<Scalar::Type>(arrayType));
     JSObject *obj = ArrayBufferObject::create(context(), nbytes);
     if (!obj)
         return false;
-    vp->setObject(*obj);
+    vp.setObject(*obj);
     ArrayBufferObject &buffer = obj->as<ArrayBufferObject>();
-    JS_ASSERT(buffer.byteLength() == nbytes);
+    MOZ_ASSERT(buffer.byteLength() == nbytes);
 
     switch (arrayType) {
       case Scalar::Int8:
@@ -1415,29 +1527,41 @@ JSStructuredCloneReader::readV1ArrayBuffer(uint32_t arrayType, uint32_t nelems,
     }
 }
 
+static bool
+PrimitiveToObject(JSContext *cx, MutableHandleValue vp)
+{
+    JSObject *obj = js::PrimitiveToObject(cx, vp);
+    if (!obj)
+        return false;
+
+    vp.setObject(*obj);
+    return true;
+}
+
 bool
-JSStructuredCloneReader::startRead(Value *vp)
+JSStructuredCloneReader::startRead(MutableHandleValue vp)
 {
     uint32_t tag, data;
 
     if (!in.readPair(&tag, &data))
         return false;
+
     switch (tag) {
       case SCTAG_NULL:
-        vp->setNull();
+        vp.setNull();
         break;
 
       case SCTAG_UNDEFINED:
-        vp->setUndefined();
+        vp.setUndefined();
         break;
 
       case SCTAG_INT32:
-        vp->setInt32(data);
+        vp.setInt32(data);
         break;
 
       case SCTAG_BOOLEAN:
       case SCTAG_BOOLEAN_OBJECT:
-        vp->setBoolean(!!data);
+        vp.setBoolean(!!data);
         if (tag == SCTAG_BOOLEAN_OBJECT && !PrimitiveToObject(context(), vp))
             return false;
         break;
@@ -1447,7 +1571,7 @@ JSStructuredCloneReader::startRead(Value *vp)
         JSString *str = readString(data);
         if (!str)
             return false;
-        vp->setString(str);
+        vp.setString(str);
         if (tag == SCTAG_STRING_OBJECT && !PrimitiveToObject(context(), vp))
             return false;
         break;
@@ -1457,7 +1581,7 @@ JSStructuredCloneReader::startRead(Value *vp)
         double d;
         if (!in.readDouble(&d) || !checkDouble(d))
             return false;
-        vp->setDouble(d);
+        vp.setDouble(d);
         if (!PrimitiveToObject(context(), vp))
             return false;
         break;
@@ -1475,7 +1599,7 @@ JSStructuredCloneReader::startRead(Value *vp)
         JSObject *obj = js_NewDateObjectMsec(context(), d);
         if (!obj)
             return false;
-        vp->setObject(*obj);
+        vp.setObject(*obj);
         break;
       }
 
@@ -1501,7 +1625,7 @@ JSStructuredCloneReader::startRead(Value *vp)
                                                             context()->tempLifoAlloc());
         if (!reobj)
             return false;
-        vp->setObject(*reobj);
+        vp.setObject(*reobj);
         break;
       }
 
@@ -1512,7 +1636,7 @@ JSStructuredCloneReader::startRead(Value *vp)
                         : NewBuiltinClassInstance(context(), &JSObject::class_);
         if (!obj || !objs.append(ObjectValue(*obj)))
             return false;
-        vp->setObject(*obj);
+        vp.setObject(*obj);
         break;
       }
 
@@ -1523,7 +1647,7 @@ JSStructuredCloneReader::startRead(Value *vp)
                                  "invalid back reference in input");
             return false;
         }
-        *vp = allObjs[data];
+        vp.set(allObjs[data]);
         return true;
       }
 
@@ -1540,18 +1664,27 @@ JSStructuredCloneReader::startRead(Value *vp)
             return false;
         break;
 
-      case SCTAG_TYPED_ARRAY_OBJECT:
-        // readTypedArray adds the array to allObjs
+      case SCTAG_TYPED_ARRAY_OBJECT: {
+        // readTypedArray adds the array to allObjs.
         uint64_t arrayType;
         if (!in.read(&arrayType))
             return false;
         return readTypedArray(arrayType, data, vp);
+      }
+
+      case SCTAG_SHARED_TYPED_ARRAY_OBJECT: {
+        // readSharedTypedArray adds the array to allObjs.
+        uint64_t arrayType;
+        if (!in.read(&arrayType))
+            return false;
+        return readSharedTypedArray(arrayType, data, vp);
+      }
 
       case SCTAG_MAP_OBJECT: {
         JSObject *obj = MapObject::create(context());
         if (!obj || !objs.append(ObjectValue(*obj)))
             return false;
-        vp->setObject(*obj);
+        vp.setObject(*obj);
         break;
       }
 
@@ -1559,7 +1692,7 @@ JSStructuredCloneReader::startRead(Value *vp)
         JSObject *obj = SetObject::create(context());
         if (!obj || !objs.append(ObjectValue(*obj)))
             return false;
-        vp->setObject(*obj);
+        vp.setObject(*obj);
         break;
       }
 
@@ -1568,7 +1701,7 @@ JSStructuredCloneReader::startRead(Value *vp)
             double d = ReinterpretPairAsDouble(tag, data);
             if (!checkDouble(d))
                 return false;
-            vp->setNumber(d);
+            vp.setNumber(d);
             break;
         }
 
@@ -1586,11 +1719,11 @@ JSStructuredCloneReader::startRead(Value *vp)
         JSObject *obj = callbacks->read(context(), this, tag, data, closure);
         if (!obj)
             return false;
-        vp->setObject(*obj);
+        vp.setObject(*obj);
       }
     }
 
-    if (vp->isObject() && !allObjs.append(*vp))
+    if (vp.isObject() && !allObjs.append(vp))
         return false;
 
     return true;
@@ -1620,7 +1753,7 @@ JSStructuredCloneReader::readTransferMap()
         if (!in.readPair(&tag, &data))
             return false;
 
-        JS_ASSERT(tag != SCTAG_TRANSFER_MAP_PENDING_ENTRY);
+        MOZ_ASSERT(tag != SCTAG_TRANSFER_MAP_PENDING_ENTRY);
         RootedObject obj(cx);
 
         void *content;
@@ -1633,14 +1766,14 @@ JSStructuredCloneReader::readTransferMap()
 
         if (tag == SCTAG_TRANSFER_MAP_ARRAY_BUFFER) {
             size_t nbytes = extraData;
-            JS_ASSERT(data == JS::SCTAG_TMO_ALLOC_DATA ||
-                      data == JS::SCTAG_TMO_MAPPED_DATA);
+            MOZ_ASSERT(data == JS::SCTAG_TMO_ALLOC_DATA ||
+                       data == JS::SCTAG_TMO_MAPPED_DATA);
             if (data == JS::SCTAG_TMO_ALLOC_DATA)
                 obj = JS_NewArrayBufferWithContents(cx, nbytes, content);
             else if (data == JS::SCTAG_TMO_MAPPED_DATA)
                 obj = JS_NewMappedArrayBufferWithContents(cx, nbytes, content);
         } else if (tag == SCTAG_TRANSFER_MAP_SHARED_BUFFER) {
-            JS_ASSERT(data == JS::SCTAG_TMO_SHARED_BUFFER);
+            MOZ_ASSERT(data == JS::SCTAG_TMO_SHARED_BUFFER);
             obj = SharedArrayBufferObject::New(context(), (SharedArrayRawBuffer *)content);
         } else {
             if (!callbacks || !callbacks->readTransfer) {
@@ -1680,7 +1813,7 @@ JSStructuredCloneReader::readTransferMap()
 }
 
 bool
-JSStructuredCloneReader::read(Value *vp)
+JSStructuredCloneReader::read(MutableHandleValue vp)
 {
     if (!readTransferMap())
         return false;
@@ -1702,7 +1835,7 @@ JSStructuredCloneReader::read(Value *vp)
         }
 
         RootedValue key(context());
-        if (!startRead(key.address()))
+        if (!startRead(&key))
             return false;
 
         if (key.isNull() && !(obj->is<MapObject>() || obj->is<SetObject>())) {
@@ -1719,7 +1852,7 @@ JSStructuredCloneReader::read(Value *vp)
         }
 
         RootedValue val(context());
-        if (!startRead(val.address()))
+        if (!startRead(&val))
             return false;
 
         if (obj->is<MapObject>()) {
@@ -1752,7 +1885,7 @@ JS_ReadStructuredClone(JSContext *cx, uint64_t *buf, size_t nbytes,
     CHECK_REQUEST(cx);
 
     if (version > JS_STRUCTURED_CLONE_VERSION) {
-        JS_ReportErrorNumber(cx, js_GetErrorMessage, nullptr, JSMSG_BAD_CLONE_VERSION);
+        JS_ReportErrorNumber(cx, js_GetErrorMessage, nullptr, JSMSG_SC_BAD_CLONE_VERSION);
         return false;
     }
     const JSStructuredCloneCallbacks *callbacks =
@@ -1811,7 +1944,7 @@ JS_StructuredClone(JSContext *cx, HandleValue value, MutableHandleValue vp,
     // so we copy the string by wrapping it.
     if (value.isString()) {
       RootedString strValue(cx, value.toString());
-      if (!cx->compartment()->wrap(cx, strValue.address())) {
+      if (!cx->compartment()->wrap(cx, &strValue)) {
         return false;
       }
       vp.setString(strValue);
@@ -1849,7 +1982,7 @@ JSAutoStructuredCloneBuffer::JSAutoStructuredCloneBuffer(JSAutoStructuredCloneBu
 JSAutoStructuredCloneBuffer&
 JSAutoStructuredCloneBuffer::operator=(JSAutoStructuredCloneBuffer &&other)
 {
-    JS_ASSERT(&other != this);
+    MOZ_ASSERT(&other != this);
     clear();
     other.steal(&data_, &nbytes_, &version_);
     return *this;
@@ -1915,8 +2048,8 @@ JSAutoStructuredCloneBuffer::read(JSContext *cx, MutableHandleValue vp,
                                   const JSStructuredCloneCallbacks *optionalCallbacks,
                                   void *closure)
 {
-    JS_ASSERT(cx);
-    JS_ASSERT(data_);
+    MOZ_ASSERT(cx);
+    MOZ_ASSERT(data_);
     return !!JS_ReadStructuredClone(cx, data_, nbytes_, version_, vp,
                                     optionalCallbacks, closure);
 }
@@ -1973,12 +2106,12 @@ JS_ReadTypedArray(JSStructuredCloneReader *r, MutableHandleValue vp)
     if (!r->input().readPair(&tag, &nelems))
         return false;
     if (tag >= SCTAG_TYPED_ARRAY_V1_MIN && tag <= SCTAG_TYPED_ARRAY_V1_MAX) {
-        return r->readTypedArray(TagToV1ArrayType(tag), nelems, vp.address(), true);
+        return r->readTypedArray(TagToV1ArrayType(tag), nelems, vp, true);
     } else if (tag == SCTAG_TYPED_ARRAY_OBJECT) {
         uint64_t arrayType;
         if (!r->input().read(&arrayType))
             return false;
-        return r->readTypedArray(arrayType, nelems, vp.address());
+        return r->readTypedArray(arrayType, nelems, vp);
     } else {
         JS_ReportErrorNumber(r->context(), js_GetErrorMessage, nullptr,
                              JSMSG_SC_BAD_SERIALIZED_DATA, "expected type array");
@@ -2001,17 +2134,8 @@ JS_WriteBytes(JSStructuredCloneWriter *w, const void *p, size_t len)
 JS_PUBLIC_API(bool)
 JS_WriteTypedArray(JSStructuredCloneWriter *w, HandleValue v)
 {
-    JS_ASSERT(v.isObject());
+    MOZ_ASSERT(v.isObject());
     assertSameCompartment(w->context(), v);
     RootedObject obj(w->context(), &v.toObject());
-
-    // If the object is a security wrapper, see if we're allowed to unwrap it.
-    // If we aren't, throw.
-    if (obj->is<WrapperObject>())
-        obj = CheckedUnwrap(obj);
-    if (!obj) {
-        JS_ReportErrorNumber(w->context(), js_GetErrorMessage, nullptr, JSMSG_UNWRAP_DENIED);
-        return false;
-    }
     return w->writeTypedArray(obj);
 }
