@@ -19,6 +19,7 @@
 #include "nsIObserverService.h"
 #include "nsDirectoryServiceUtils.h"
 #include "nsDirectoryServiceDefs.h"
+#include "nsProfilerStartParams.h"
 #include "mozilla/Services.h"
 #include "nsThreadUtils.h"
 #include "ProfilerMarkers.h"
@@ -44,6 +45,7 @@ int         sFrameNumber = 0;
 int         sLastFrameNumber = 0;
 int         sInitCount = 0; // Each init must have a matched shutdown.
 static bool sIsProfiling = false; // is raced on
+static bool sIsGPUProfiling = false; // is raced on
 
 // env variables to control the profiler
 const char* PROFILER_MODE = "MOZ_PROFILER_MODE";
@@ -86,6 +88,10 @@ void Sampler::Startup() {
 
 void Sampler::Shutdown() {
   while (sRegisteredThreads->size() > 0) {
+    // Any stack that's still referenced at this point are
+    // still active and we don't have a way to clean them up
+    // safetly and still handle the pop call on that object.
+    sRegisteredThreads->back()->ForgetStack();
     delete sRegisteredThreads->back();
     sRegisteredThreads->pop_back();
   }
@@ -109,6 +115,7 @@ ThreadInfo::ThreadInfo(const char* aName, int aThreadId,
   , mPlatformData(Sampler::AllocPlatformData(aThreadId))
   , mProfile(nullptr)
   , mStackTop(aStackTop)
+  , mPendingDelete(false)
 {
   mThread = NS_GetCurrentThread();
 }
@@ -120,6 +127,20 @@ ThreadInfo::~ThreadInfo() {
     delete mProfile;
 
   Sampler::FreePlatformData(mPlatformData);
+
+  delete mPseudoStack;
+  mPseudoStack = nullptr;
+}
+
+void
+ThreadInfo::SetPendingDelete()
+{
+  mPendingDelete = true;
+  // We don't own the pseudostack so disconnect it.
+  mPseudoStack = nullptr;
+  if (mProfile) {
+    mProfile->SetPendingDelete();
+  }
 }
 
 ProfilerMarker::ProfilerMarker(const char* aMarkerName,
@@ -218,11 +239,6 @@ bool sps_version2()
 
     bool req2 = PR_GetEnv("MOZ_PROFILER_NEW") != nullptr; // Has v2 been requested?
 
-    bool elfhackd = false;
-#   if defined(USE_ELF_HACK)
-    bool elfhackd = true;
-#   endif
-
     if (req2 && allow2) {
       version = 2;
       LOG("------------------- MOZ_PROFILER_NEW set -------------------");
@@ -230,10 +246,6 @@ bool sps_version2()
       version = 1;
       LOG("--------------- MOZ_PROFILER_NEW requested, ----------------");
       LOG("---------- but is not available on this platform -----------");
-    } else if (req2 && elfhackd) {
-      version = 1;
-      LOG("--------------- MOZ_PROFILER_NEW requested, ----------------");
-      LOG("--- but this build was not done with --disable-elf-hack ----");
     } else {
       version = 1;
       LOG("----------------- MOZ_PROFILER_NEW not set -----------------");
@@ -662,6 +674,8 @@ const char** mozilla_sampler_get_features()
     // Tell the JS engine to emmit pseudostack entries in the
     // pro/epilogue.
     "js",
+    // GPU Profiling (may not be supported by the GL)
+    "gpu",
     // Profile the registered secondary threads.
     "threads",
     // Do not include user-identifiable information
@@ -670,6 +684,10 @@ const char** mozilla_sampler_get_features()
     "mainthreadio",
     // Add RSS collection
     "memory",
+#ifdef MOZ_TASK_TRACER
+    // Start profiling with feature TaskTracer.
+    "tasktracer",
+#endif
 #if defined(XP_WIN)
     // Add power collection
     "power",
@@ -721,6 +739,9 @@ void mozilla_sampler_start(int aProfileEntries, double aInterval,
 
       for (uint32_t i = 0; i < threads.size(); i++) {
         ThreadInfo* info = threads[i];
+        if (info->IsPendingDelete()) {
+          continue;
+        }
         ThreadProfile* thread_profile = info->Profile();
         if (!thread_profile) {
           continue;
@@ -756,11 +777,28 @@ void mozilla_sampler_start(int aProfileEntries, double aInterval,
   }
 
   sIsProfiling = true;
+  sIsGPUProfiling = t->ProfileGPU();
 
   if (Sampler::CanNotifyObservers()) {
     nsCOMPtr<nsIObserverService> os = mozilla::services::GetObserverService();
-    if (os)
-      os->NotifyObservers(nullptr, "profiler-started", nullptr);
+    if (os) {
+      nsTArray<nsCString> featuresArray;
+      nsTArray<nsCString> threadNameFiltersArray;
+
+      for (size_t i = 0; i < aFeatureCount; ++i) {
+        featuresArray.AppendElement(aFeatures[i]);
+      }
+
+      for (size_t i = 0; i < aFilterCount; ++i) {
+        threadNameFiltersArray.AppendElement(aThreadNameFilters[i]);
+      }
+
+      nsCOMPtr<nsIProfilerStartParams> params =
+        new nsProfilerStartParams(aProfileEntries, aInterval, featuresArray,
+                                  threadNameFiltersArray);
+
+      os->NotifyObservers(params, "profiler-started", nullptr);
+    }
   }
 
   LOG("END   mozilla_sampler_start");
@@ -839,6 +877,19 @@ void mozilla_sampler_resume() {
   }
 }
 
+bool mozilla_sampler_feature_active(const char* aName)
+{
+  if (!profiler_is_active()) {
+    return false;
+  }
+
+  if (strcmp(aName, "gpu") == 0) {
+    return sIsGPUProfiling;
+  }
+
+  return false;
+}
+
 bool mozilla_sampler_is_active()
 {
   return sIsProfiling;
@@ -878,6 +929,10 @@ void mozilla_sampler_unlock()
 
 bool mozilla_sampler_register_thread(const char* aName, void* stackTop)
 {
+  if (sInitCount == 0) {
+    return false;
+  }
+
 #if defined(MOZ_WIDGET_GONK) && !defined(MOZ_PROFILING)
   // The only way to profile secondary threads on b2g
   // is to build with profiling OR have the profiler
@@ -895,17 +950,24 @@ bool mozilla_sampler_register_thread(const char* aName, void* stackTop)
 
 void mozilla_sampler_unregister_thread()
 {
-  Sampler::UnregisterCurrentThread();
+  if (sInitCount == 0) {
+    return;
+  }
 
   PseudoStack *stack = tlsPseudoStack.get();
   if (!stack) {
     return;
   }
-  delete stack;
   tlsPseudoStack.set(nullptr);
+
+  Sampler::UnregisterCurrentThread();
 }
 
 void mozilla_sampler_sleep_start() {
+    if (sInitCount == 0) {
+	return;
+    }
+
     PseudoStack *stack = tlsPseudoStack.get();
     if (stack == nullptr) {
       return;
@@ -914,6 +976,10 @@ void mozilla_sampler_sleep_start() {
 }
 
 void mozilla_sampler_sleep_end() {
+    if (sInitCount == 0) {
+	return;
+    }
+
     PseudoStack *stack = tlsPseudoStack.get();
     if (stack == nullptr) {
       return;
@@ -1000,7 +1066,10 @@ void mozilla_sampler_add_marker(const char *aMarker, ProfilerMarkerPayload *aPay
   if (!stack) {
     return;
   }
-  mozilla::TimeDuration delta = mozilla::TimeStamp::Now() - sStartTime;
+
+  mozilla::TimeStamp origin = (aPayload && !aPayload->GetStartTime().IsNull()) ?
+                     aPayload->GetStartTime() : mozilla::TimeStamp::Now();
+  mozilla::TimeDuration delta = origin - sStartTime;
   stack->addMarker(aMarker, payload.forget(), static_cast<float>(delta.ToMilliseconds()));
 }
 
