@@ -745,23 +745,21 @@ nsContextMenu.prototype = {
   // Open linked-to URL in a new tab.
   openLinkInTab: function(aEvent) {
     // Determine linked-to URL.
-    return openNewTabWith(this.linkURL, this.target.ownerDocument, null,
-                          aEvent);
+    return openNewTabWith(this.linkURL, this.link, null, aEvent);
   },
 
   // Open linked-to URL in a new window.
   openLinkInWindow: function() {
-    return openNewWindowWith(this.linkURL, this.target.ownerDocument);
+    return openNewWindowWith(this.linkURL, this.link);
   },
 
   // Open linked-to URL in a private window.
   openLinkInPrivateWindow: function() {
-    return openNewPrivateWith(this.linkURL, this.target.ownerDocument);
+    return openNewPrivateWith(this.linkURL, this.link);
   },
 
   // Open frame in a new tab.
   openFrameInTab: function(aEvent) {
-    // Determine linked-to URL.
     return openNewTabWith(this.target.ownerDocument.location.href,
                           this.target.ownerDocument, null, aEvent);
   },
