@@ -2,6 +2,8 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+Components.utils.import("resource:///modules/mailServices.js");
+
 top.MAX_RECIPIENTS = 1; /* for the initial listitem created in the XUL */
 
 var inputElementType = "";
@@ -10,8 +12,9 @@ var selectElementIndexTable = null;
 
 var gNumberOfCols = 0;
 
-var gDragService = Components.classes["@mozilla.org/widget/dragservice;1"].getService();
-gDragService = gDragService.QueryInterface(Components.interfaces.nsIDragService);
+var gDragService = Components.classes["@mozilla.org/widget/dragservice;1"]
+                             .getService(Components.interfaces.nsIDragService);
+
 var gMimeHeaderParser = null;
 
 /**
@@ -626,19 +629,12 @@ function awCopyNode(node, parentNode, beforeNode)
 
 function awRemoveRow(row)
 {
-  var listbox = document.getElementById('addressingWidget');
-
-  awRemoveNodeAndChildren(listbox, awGetListItem(row));
+  awGetListItem(row).remove();
   awFitDummyRows();
 
   top.MAX_RECIPIENTS --;
 }
 
-function awRemoveNodeAndChildren(parent, nodeToRemove)
-{
-  nodeToRemove.parentNode.removeChild(nodeToRemove);
-}
-
 function awSetFocus(row, inputElement)
 {
   top.awRow = row;
@@ -870,7 +866,7 @@ function awCreateOrRemoveDummyRows()
   for (var i = kids.length-1; gAWContentHeight > listboxHeight && i >= 0; --i) {
     if (kids[i].hasAttribute("_isDummyRow")) {
       gAWContentHeight -= gAWRowHeight;
-      listbox.removeChild(kids[i]);
+      kids[i].remove();
     }
   }
 
@@ -993,13 +989,10 @@ function parseAndAddAddresses(addressText, recipientType)
   // strip any leading >> characters inserted by the autocomplete widget
   var strippedAddresses = addressText.replace(/.* >> /, "");
 
-  var hdrParser = Components.classes["@mozilla.org/messenger/headerparser;1"].getService(Components.interfaces.nsIMsgHeaderParser);
-  var addresses = {};
-  var names = {};
-  var fullNames = {};
-  var numAddresses = hdrParser.parseHeadersWithArray(strippedAddresses, addresses, names, fullNames);
+  var addresses = MailServices.headerParser
+                              .makeFromDisplayAddress(strippedAddresses);
 
-  if (numAddresses > 0)
+  if (addresses.length)
   {
     // we need to set up our own autocomplete session and search for results
 
@@ -1007,7 +1000,8 @@ function parseAndAddAddresses(addressText, recipientType)
     if (!gAutomatedAutoCompleteListener)
       gAutomatedAutoCompleteListener = new AutomatedAutoCompleteHandler();
 
-    gAutomatedAutoCompleteListener.init(fullNames.value, numAddresses, recipientType);
+    gAutomatedAutoCompleteListener.init(addresses.map(addr => addr.toString()),
+                                        addresses.length, recipientType);
   }
 }
 
